Package: SampleCore
Title: Sampling Strategies for Constructing Core Collections
Version: 0.1.0
Authors@R: c(
    person(given = "J.", family = "Aravind",
           email = "j.aravind@icar.org.in",
           role = c("aut", "cre"),
		       comment = c(ORCID = "0000-0002-4791-442X")),
    person(given = "Suman", family = "Roy",
           email = "suman.roy@icar.org.in",
           role = c("aut"),
		       comment = c(ORCID = "0000-0001-5612-8415")),
    person(given = "Anju Mahendru", family = "Singh",
           email = "anju.singh@icar.org.in",
		       role = "aut",
		       comment = c(ORCID = "0000-0001-6958-1630")),
    person("ICAR-NBGPR",
           role = c("cph"),
		       comment = c(ROR = "https://ror.org/00scbd467",
		                   url = "https://nbpgr.org.in/")))
Description: Implements multiple allocation and selection strategies of 
    sampling to construct core collections primarily from clustered or grouped 
    germplasm collection data. Provides methods for allocating entries to 
    clusters/groups based on group sizes, group-wise distance-based diversity 
    metrics, and group-wise diversity index estimates. Includes procedures for 
    selecting entries within clusters/groups through random sampling, genetic 
    distance-based approaches, and optimized diversity metric–based selection 
    methods. See the package documentation for more, including full list of 
    references for the methods implemented.
License: GPL (>= 2)
Encoding: UTF-8
BuildManual: TRUE
Imports: cluster, DiversityStats, ggplot2, igraph, MASS, mathjaxr,
        prospectr, Rdpack, Rtsne, stats, vegan
Suggests: biotools, dbscan, EvaluateCore, fastcluster, knitr, pander,
        rmarkdown
RdMacros: mathjaxr, Rdpack
Copyright: 2024-2026, ICAR-NBPGR
URL: https://github.com/aravind-j/SampleCore
        https://aravind-j.github.io/SampleCore/
BugReports: https://github.com/aravind-j/SampleCore/issues
Depends: R (>= 3.5)
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-21 04:21:51 UTC; Aravind-DGC
Author: J. Aravind [aut, cre] (ORCID: <https://orcid.org/0000-0002-4791-442X>),
  Suman Roy [aut] (ORCID: <https://orcid.org/0000-0001-5612-8415>),
  Anju Mahendru Singh [aut] (ORCID:
    <https://orcid.org/0000-0001-6958-1630>),
  ICAR-NBGPR [cph] (ROR: <https://ror.org/00scbd467>, url:
    https://nbpgr.org.in/)
Maintainer: J. Aravind <j.aravind@icar.org.in>
Repository: CRAN
Date/Publication: 2026-05-28 13:00:02 UTC
Built: R 4.5.2; ; 2026-05-28 16:20:36 UTC; unix

Package: SampleCore
Title: Sampling Strategies for Constructing Core Collections
Version: 0.1.0
Authors@R: c(
    person(given = "J.", family = "Aravind",
           email = "j.aravind@icar.org.in",
           role = c("aut", "cre"),
		       comment = c(ORCID = "0000-0002-4791-442X")),
    person(given = "Suman", family = "Roy",
           email = "suman.roy@icar.org.in",
           role = c("aut"),
		       comment = c(ORCID = "0000-0001-5612-8415")),
    person(given = "Anju Mahendru", family = "Singh",
           email = "anju.singh@icar.org.in",
		       role = "aut",
		       comment = c(ORCID = "0000-0001-6958-1630")),
    person("ICAR-NBGPR",
           role = c("cph"),
		       comment = c(ROR = "https://ror.org/00scbd467",
		                   url = "https://nbpgr.org.in/")))
Description: Implements multiple allocation and selection strategies of 
    sampling to construct core collections primarily from clustered or grouped 
    germplasm collection data. Provides methods for allocating entries to 
    clusters/groups based on group sizes, group-wise distance-based diversity 
    metrics, and group-wise diversity index estimates. Includes procedures for 
    selecting entries within clusters/groups through random sampling, genetic 
    distance-based approaches, and optimized diversity metric–based selection 
    methods. See the package documentation for more, including full list of 
    references for the methods implemented.
License: GPL (>= 2)
Encoding: UTF-8
BuildManual: TRUE
Imports: cluster, DiversityStats, ggplot2, igraph, MASS, mathjaxr,
        prospectr, Rdpack, Rtsne, stats, vegan
Suggests: biotools, dbscan, EvaluateCore, fastcluster, knitr, pander,
        rmarkdown
RdMacros: mathjaxr, Rdpack
Copyright: 2024-2026, ICAR-NBPGR
URL: https://github.com/aravind-j/SampleCore
        https://aravind-j.github.io/SampleCore/
BugReports: https://github.com/aravind-j/SampleCore/issues
Depends: R (>= 3.5)
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-21 04:21:51 UTC; Aravind-DGC
Author: J. Aravind [aut, cre] (ORCID: <https://orcid.org/0000-0002-4791-442X>),
  Suman Roy [aut] (ORCID: <https://orcid.org/0000-0001-5612-8415>),
  Anju Mahendru Singh [aut] (ORCID:
    <https://orcid.org/0000-0001-6958-1630>),
  ICAR-NBGPR [cph] (ROR: <https://ror.org/00scbd467>, url:
    https://nbpgr.org.in/)
Maintainer: J. Aravind <j.aravind@icar.org.in>
Repository: CRAN
Date/Publication: 2026-05-28 13:00:02 UTC
Built: R 4.6.0; ; 2026-05-28 13:32:59 UTC; unix

# Generated by roxygen2: do not edit by hand

export(allocate.basic)
export(allocate.distance)
export(allocate.diversity)
export(plot_dist)
export(select.distance)
export(select.diversity)
export(select.random)
import(mathjaxr)
importFrom(DiversityStats,mcintosh_diversity)
importFrom(DiversityStats,shannon)
importFrom(DiversityStats,simpson)
importFrom(MASS,isoMDS)
importFrom(Rdpack,reprompt)
importFrom(Rtsne,Rtsne)
importFrom(cluster,pam)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(grDevices,n2mfrow)
importFrom(graphics,par)
importFrom(graphics,title)
importFrom(igraph,E)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,mst)
importFrom(prospectr,duplex)
importFrom(prospectr,honigs)
importFrom(prospectr,kenStone)
importFrom(prospectr,naes)
importFrom(stats,as.dist)
importFrom(stats,cmdscale)
importFrom(stats,cutree)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,setNames)
importFrom(vegan,betadisper)

<!-- 
<img src="https://raw.githubusercontent.com/aravind-j/SampleCore/master/inst/extdata/SampleCore.png" alt="" width="20%" />
-->

## `SampleCore`: Sampling Strategies for Constructing Core Collections

<img src="https://raw.githubusercontent.com/aravind-j/SampleCore/master/inst/extdata/SampleCore.png" align="right" alt="logo" width="173" height = "200" style = "border: none; float: right;">

###### Version : [0.1.0](https://aravind-j.github.io/SampleCore/); License: [GPL-2\|GPL-3](https://www.r-project.org/Licenses/)

##### *Aravind, J., Roy, Suman and Singh, Anju M.*

Division of Germplasm Conservation, ICAR-National Bureau of Plant
Genetic Resources, New Delhi.

------------------------------------------------------------------------

[![minimal R
version](https://img.shields.io/badge/R%3E%3D-3.5.0-6666ff.svg?logo=R)](https://cran.r-project.org/)
[![License: GPL
v3](https://img.shields.io/badge/License-GPL%20v3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)

<!-- [![CRAN_Status_Badge](https://www.r-pkg.org/badges/version-last-release/SampleCore)](https://cran.r-project.org/package=SampleCore)
[![Dependencies](https://tinyverse.netlify.app/status/SampleCore)](https://cran.r-project.org/package=SampleCore) -->

<!-- [![rstudio mirror downloads](https://cranlogs.r-pkg.org/badges/grand-total/SampleCore?color=green)](https://CRAN.R-project.org/package=SampleCore) -->

<!-- 
[![develVersion](https://img.shields.io/badge/devel%20version-0.0.0.9000-orange.svg)](https://github.com/aravind-j/SampleCore)
-->

[![Github Code
Size](https://img.shields.io/github/languages/code-size/aravind-j/SampleCore.svg)](https://github.com/aravind-j/SampleCore)

<!-- [![R-CMD-check](https://github.com/aravind-j/SampleCore/workflows/R-CMD-check/badge.svg)](https://github.com/aravind-j/SampleCore/actions) -->

[![Project Status:
WIP](https://www.repostatus.org/badges/latest/wip.svg)](https://www.repostatus.org/#wip)
[![lifecycle](https://lifecycle.r-lib.org/articles/figures/lifecycle-maturing.svg)](https://lifecycle.r-lib.org/articles/stages.html#maturing)
[![Last-changedate](https://img.shields.io/badge/last%20change-2026--05--15-yellowgreen.svg)](https://github.com/aravind-j/SampleCore/)
<!-- [![Zenodo DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.zenodo.14889174.svg)](https://doi.org/10.5281/zenodo.14889174) -->
[![Website -
pkgdown](https://img.shields.io/website-up-down-green-red/https/aravind-j.github.io/SampleCore.svg)](https://aravind-j.github.io/SampleCore/)
<!-- [![.](https://pro-pulsar-193905.appspot.com/G-xxxxxxxx/welcome-page)](https://github.com/aravind-j/google-analytics-beacon) -->
[![GoatCounter](https://samplecore-gh.goatcounter.com/count?p=/test)](https://samplecore-gh.goatcounter.com/)
<!-- [![packageversion](https://img.shields.io/badge/Package%20version-0.2.3.3-orange.svg)](https://github.com/aravind-j/SampleCore) -->
<!-- [![GitHub Download Count](https://github-basic-badges.herokuapp.com/downloads/aravind-j/SampleCore/total.svg)] -->
<!-- [![Rdoc](http://www.rdocumentation.org/badges/version/SampleCore)](http://www.rdocumentation.org/packages/SampleCore) -->

------------------------------------------------------------------------

## Description

<!-- Implements multiple allocation and selection strategies of      sampling to construct core collections primarily from clustered or grouped      germplasm collection data. Provides methods for allocating entries to      clusters/groups based on group sizes, group-wise distance-based diversity      metrics, and group-wise diversity index estimates. Includes procedures for      selecting entries within clusters/groups through random sampling, genetic      distance-based approaches, and optimized diversity metric–based selection      methods. See the package documentation for more, including full list of      references for the methods implemented. -->

Implements multiple allocation and selection strategies of sampling to
construct core collections primarily from clustered or grouped germplasm
collection data. Provides methods for allocating entries to
clusters/groups based on group sizes, group-wise distance-based
diversity metrics, and group-wise diversity index estimates. Includes
procedures for selecting entries within clusters/groups through random
sampling, genetic distance-based approaches, and optimized diversity
metric–based selection methods. See the package documentation for more,
including full list of references for the methods implemented.

## Installation

<!-- 
The package can be installed from CRAN as follows: 
&#10;
-->

The development version can be installed from github as follows:

``` r
# Install development version from Github
devtools::install_github("aravind-j/SampleCore")
```

<!-- ## Detailed tutorial
For a detailed tutorial (vignette) on how to used this package type:
&#10;
``` r
browseVignettes(package = 'SampleCore')
```
The vignette for the latest version is also available [online](https://aravind-j.github.io/SampleCore/articles.html).-->

## What’s new

To know whats new in this version type:

``` r
news(package='SampleCore')
```

## Links

<!--[CRAN page](https://cran.r-project.org/package=SampleCore) -->

[Github page](https://github.com/aravind-j/SampleCore)

[Documentation website](https://aravind-j.github.io/SampleCore/)

<!--[Zenodo DOI](https://doi.org/10.5281/zenodo.14889174) -->

<!--
## CRAN checks
&#10;
&#10;
&#10; [![Linux](https://img.shields.io/badge/Linux-FCC624?style=for-the-badge&logo=linux&logoColor=black)](https://cran.r-project.org/web/checks/check_results_SampleCore.html) 
&#10;
+-----------------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| Flavour                           | CRAN check                                                                                                                                                                                                     |
+===================================+================================================================================================================================================================================================================+
| r-devel-linux-x86_64-debian-clang | [![CRAN check -                                                                                                                                                                                                |
|                                   | r-devel-linux-x86_64-debian-clang](https://badges.cranchecks.info/flavor/r-devel-linux-x86_64-debian-clang/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html) |
+-----------------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| r-devel-linux-x86_64-debian-gcc   | [![CRAN check -                                                                                                                                                                                                |
|                                   | r-devel-linux-x86_64-debian-gcc](https://badges.cranchecks.info/flavor/r-devel-linux-x86_64-debian-gcc/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html)     |
+-----------------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| r-devel-linux-x86_64-fedora-clang | [![CRAN check -                                                                                                                                                                                                |
|                                   | r-devel-linux-x86_64-fedora-clang](https://badges.cranchecks.info/flavor/r-devel-linux-x86_64-fedora-clang/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html) |
+-----------------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| r-devel-linux-x86_64-fedora-gcc   | [![CRAN check -                                                                                                                                                                                                |
|                                   | r-devel-linux-x86_64-fedora-gcc](https://badges.cranchecks.info/flavor/r-devel-linux-x86_64-fedora-gcc/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html)     |
+-----------------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| r-patched-linux-x86_64            | [![CRAN check -                                                                                                                                                                                                |
|                                   | r-patched-linux-x86_64](https://badges.cranchecks.info/flavor/r-patched-linux-x86_64/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html)                       |
+-----------------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| r-release-linux-x86_64            | [![CRAN check -                                                                                                                                                                                                |
|                                   | r-release-linux-x86_64](https://badges.cranchecks.info/flavor/r-release-linux-x86_64/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html)                       |
+-----------------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
&#10;
 [![Windows](https://img.shields.io/badge/Windows-0078D6?style=for-the-badge&logo=windows&logoColor=white)](https://cran.r-project.org/web/checks/check_results_SampleCore.html) 
&#10;
+--------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| Flavour                  | CRAN check                                                                                                                                                                                   |
+==========================+==============================================================================================================================================================================================+
| r-devel-windows-x86_64   | [![CRAN check -                                                                                                                                                                              |
|                          | r-devel-windows-x86_64](https://badges.cranchecks.info/flavor/r-devel-windows-x86_64/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html)     |
+--------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| r-release-windows-x86_64 | [![CRAN check -                                                                                                                                                                              |
|                          | r-release-windows-x86_64](https://badges.cranchecks.info/flavor/r-release-windows-x86_64/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html) |
+--------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| r-oldrel-windows-x86_64  | [![CRAN check -                                                                                                                                                                              |
|                          | r-oldrel-windows-x86_64](https://badges.cranchecks.info/flavor/r-oldrel-windows-x86_64/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html)   |
+--------------------------+----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
&#10;
 [![MacOS](https://img.shields.io/badge/mac%20os-000000?style=for-the-badge&logo=apple&logoColor=white)](https://cran.r-project.org/web/checks/check_results_SampleCore.html) 
&#10;
+------------------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| Flavour                | CRAN check                                                                                                                                                                               |
+========================+==========================================================================================================================================================================================+
| r-release-macos-x86_64 | [![CRAN check -                                                                                                                                                                          |
|                        | r-release-macos-x86_64](https://badges.cranchecks.info/flavor/r-release-macos-x86_64/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html) |
+------------------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
| r-oldrel-macos-x86_64  | [![CRAN check -                                                                                                                                                                          |
|                        | r-oldrel-macos-x86_64](https://badges.cranchecks.info/flavor/r-oldrel-macos-x86_64/SampleCore.svg)](https://cran.r-project.org/web/checks/check_results_j.aravind_at_icar.org.in.html)   |
+------------------------+------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------+
-->

## Citing `SampleCore`

To cite the methods in the package use:

``` r
citation("SampleCore")
```

    To cite the R package 'SampleCore' in publications use:

      Aravind, J., Roy, S., and Singh, A. M. ().  SampleCore: Sampling Strategies for Constructing Core Collections. R
      package version 0.1.0, https://aravind-j.github.io/SampleCore/.

    A BibTeX entry for LaTeX users is

      @Manual{,
        title = {SampleCore: Sampling Strategies for Constructing Core Collections},
        author = {J. Aravind and Suman Roy and Anju Mahendru Singh},
        note = {R package version 0.1.0 https://aravind-j.github.io/SampleCore/},
      }

    This free and open-source software implements academic research by the authors and co-workers. If you use it, please
    support the project by citing the package.

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Common checks for all functions
#'
#' Not exported. Strictly internal
#'
#' @keywords internal
#'
#' @template general-arg
#' @template qual-arg
#' @template dist-arg
#' @template log-arg
#' @template sel-arg
#' @template size-arg
#
checks.sample.core <- function(data, names,
                               size, group,
                               qualitative = NULL,
                               dist.mat = NULL,
                               log.base = NULL,
                               alloc,
                               always.selected = NULL,
                               mode = c("alloc", "sel")) {

  # Declare nulls ----

  if (missing(qualitative)) {
    qualitative <- NULL
  }

  if (missing(dist.mat)) {
    dist.mat <- NULL
  }

  if (length(qualitative) == 1) {
    stop("Only one trait specified.")
  }

  # data ----

  # check if 'data' is a data frame object
  if (!is.data.frame(data)) {
    stop('"data" should be a data frame object.')
  }

  if (any(c("tbl_dataf", "tbl") %in% class(data))) {
    warning('"data" is of type tibble\nCoercing to data frame.')
    data <- as.data.frame(data)
  }

  # Argument lengths and type ----

  # check if 'names' argument is character vector of unit length
  if (!(is.character(names) && length(names) == 1)) {
    stop('"names" should be a character vector of unit length.')
  }

  # check if 'group' argument is character vector of unit length
  if (!(is.character(group) && length(group) == 1)) {
    stop('"group" should be a character vector of unit length.')
  }

  # check if 'qualitative' argument is a character vector
  if (!is.null(qualitative)) {
    if (!is.character(qualitative)) {
      stop('"qualitative" should be a character vector.')
    }
  }

  # check if 'group' column is of type factor
  if (!is.factor(data[, group])) {
    stop('"group" column in "data" should be of type factor.')
  }

  # check if 'size' argument is numeric vector of unit length
  if (mode == "alloc") {
    if (!(is.numeric(size) && length(size) == 1)) {
      stop('"size" should be a numeric vector of unit length.')
    }
  }

  # Columns exist in data ----

  # check if 'names' column is present in 'data'
  if (!(names %in% colnames(data))) {
    stop(paste('Column ', names,
               ' specified as the "names" column is not present in "data".',
               sep = ""))
  }

  # check if 'qualitative' columns are present in 'data'
  if (!is.null(qualitative)) {
    if (FALSE %in% (qualitative %in% colnames(data)))  {
      stop(paste('The following column(s) specified in "qualitative" ',
                 'not present in "data":\n',
                 paste(qualitative[!(qualitative %in% colnames(data))],
                       collapse = ", "),
                 sep = ""))
    }
  }

  # check if 'group' column is present in 'data'
  if (!(group %in% colnames(data))) {
    stop(paste('Column ', group,
               ' specified as the "group" column is not present in "data".',
               sep = ""))
  }

  # Check column types ----

  # check if 'names' column is of type character
  if (!is.character(data[, names])) {
    stop('"names" column in "data" should be of type character.')
  }

  # check if 'qualitative' columns are of type factor
  if (!is.null(qualitative)) {
    intqualcols <- unlist(lapply(data[, qualitative],
                                 function(x) is.factor(x)))
    if (FALSE %in% intqualcols) {
      stop(paste('The following "qualitative" column(s) in "data" are not ',
                 'of type factor:\n',
                 paste(names(intqualcols[!intqualcols]), collapse = ", ")))
    }
  }

  # Missing values ----

  # check for missing values
  missvcols <- unlist(lapply(data[, qualitative],
                             function(x) TRUE %in% is.na(x)))
  if (TRUE %in% missvcols) {
    warning(paste('The following column(s) in "data" have missing values:\n',
                  paste(names(missvcols[missvcols]), collapse = ", ")))
  }

  # Duplication ----

  # check for duplication in names
  if (any(duplicated(data[, names]))) {
    stop('Duplicated entries exist in "names" column.')
  }

  # Distance matrix ----

  if (!is.null(dist.mat)) {

    # check if 'dist.mat' is a distance matrix
    if (!inherits(dist.mat, "dist")) {
      stop('"dist.mat" should be a distance matrix of class "dist".')
    }
    dist_labels <- labels(dist.mat) # No dups as it is a distance matrix

    if (is.null(dist_labels)) {
      stop('Labels are missing in "dist.mat".')
    }

    # Check for mismatch between elements of 'data' and 'dist.mat'
    if (!setequal(data[, names], labels(dist.mat))) {
      only_in_data <- setdiff(data[, names], labels(dist.mat))
      only_in_dist <- setdiff(labels(dist.mat), data[, names])

      stop(
        paste0(
          'Mismatch between "dist.mat" labels and entries in "names" column.\n',
          'Only in "data": ', paste(only_in_data, collapse = ', '), '\n',
          'Only in "dist.mat": ', paste(only_in_dist, collapse = ', ')
        )
      )
    }
  }

  # Size ----

  # check if 'size' is a proportion between 0 and 1
  if (mode == "alloc") {
    if (size <= 0 || size >= 1) {
      stop('"size" should be a proportion between 0 and 1.')
    }
  }

  # Log base ----

  if (!is.null(log.base)) {

    if (!is.numeric(log.base) || length(log.base) != 1 || is.na(log.base)) {
      stop('"log.base" must be a single numeric value.')
    }

    if (log.base <= 0) {
      stop('"base" must be positive.')
    }

    if (log.base == 1) {
      stop('"log.base" of 1 is undefined.')
    }

  }

  # Always selected ----

  if (mode == "sel" & !is.null(always.selected)) {
    # check if 'always.selected' is a character vector
    if (!is.character(always.selected)) {
      stop('"always.selected" should be a character vector.')
    }

    # check if always.selected is present in the entire set
    if (any(!(always.selected %in% data[, names]))) {
      alsel_miss <- always.selected[!(always.selected %in% data[, names])]
      stop(paste('The following entry/entries specified in "always.selected" ',
                 'are not present in "data":\n',
                 paste(alsel_miss, collapse = ", "),
                 sep = ""))
    }
  }

  # Allocation vector ----

  if (mode == "sel") {

    nm <- names(alloc)
    lv <- levels(data[, group])

    if (!setequal(nm, lv)) {

      only_in_alloc <- setdiff(nm, lv)
      only_in_group <- setdiff(lv, nm)

      msg <- paste0(
        'Mismatch between "alloc" names and levels of "group" column in data.\n\n',
        if (length(only_in_alloc) > 0)
          paste0(
            'Present in "alloc" but not in "group" column levels: ',
            paste(shQuote(only_in_alloc), collapse = ', '),
            '\n'
          ) else '',
        if (length(only_in_group) > 0)
          paste0(
            'Present in "group" column levels but not in "alloc": ',
            paste(shQuote(only_in_group), collapse = ', '),
            '\n'
          ) else ''
      )

      stop(msg, call. = FALSE)

    }
  }

}

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

wlcm <-
  paste0("\n",
         "------------------------------------------",
         "--------------------------------------\n",
         "Welcome to SampleCore version ",
         utils::packageDescription("SampleCore")$Version, "\n",
         "\n", "\n",
         # "# To know how to use this package type:", "\n",
         # "  browseVignettes(package = 'SampleCore')", "\n",
         # "  for the package vignette.", "\n",
         # "\n",
         "# To know whats new in this version type:", "\n",
         "  news(package='SampleCore')", "\n", "  for the NEWS file.", "\n",
         "\n",
         "# To cite the methods in the package type:", "\n",
         "  citation(package='SampleCore')","\n",
         "\n",
         "# To suppress this message use:", "\n",
         "  suppressPackageStartupMessages(library(SampleCore))", "\n",
         "--------------------------------------------",
         "------------------------------------\n")

.onAttach <- function(lib, pkg,...){
  packageStartupMessage(wlcm)

}

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Selection of Entries from Clusters/Groups on the basis of Genetic Distances
#'
#' Select entries from cluster/groups in the entire collection by genetic
#' distance based sampling according to allocation specified. \loadmathjax
#'
#' For each cluster/group, entries are selected by several methods from
#' within-cluster/group genetic distances between accessions according to the
#' allocation provided (See \strong{Methods}).
#'
#' Entries listed as \code{always.selected} are mandatorily included in the
#' selection. Warnings are issued if requested allocation is smaller than the
#' number of always-selected entries in a cluster/group and/or when the
#' cluster/group does not contain enough remaining entries to fulfill the
#' allocation.
#'
#' @section Methods:
#'
#' \subsection{Centrality Based Methods}{
#'
#'   Selects accessions that are most representative/closest to the
#'   cluster/group center.
#'
#'   \subsection{Medoid-like Representative Sampling by Minimal Mean
#'   Distance}{Selects medoid-like representatives as accessions with the
#'   smallest average distance to all others within the group
#'   \insertCite{kaufman_clustering_1987,kaufman_finding_1990}{SampleCore}.
#'
#'   For each accession \mjseqn{g}, the mean distance to all other accessions
#'   \mjseqn{h} is computed as:
#'
#'   \mjsdeqn{\bar{d}_g = \frac{1}{G} \sum_{h=1}^{G} d_{gh}}
#'
#'   Accessions are ranked by \mjseqn{\bar{d}_g} in ascending order and the top
#'   \mjseqn{n} are selected.
#'
#'   }
#'
#'   \subsection{Medoid-like Representative Sampling by Minimal Median
#'   Distance}{
#'
#'   Selects medoid-like representatives as accessions with the smallest median
#'   distance to all others within the group. This method is less influenced by
#'   outliers
#'   \insertCite{kaufman_clustering_1987,kaufman_finding_1990}{SampleCore}.
#'
#'   For each accession \mjseqn{g}, the median distance to all other accessions
#'   \mjseqn{h} is computed as:
#'
#'   \mjsdeqn{\tilde{d}_g = \text{median}_{h=1,\dots,G}(d_{gh})}
#'
#'   Accessions are ranked by \mjseqn{\tilde{d}_g} in ascending order and the
#'   top \mjseqn{n} are selected.
#'
#'   }
#'
#'   \subsection{Representative Sampling by Proximity to Group
#'   Centroid}{Selects accessions closest to the group centroid in principal
#'   coordinate space, computed via multivariate dispersion analysis using
#'   \code{\link[vegan]{betadisper}}
#'   \insertCite{anderson_distance-based_2006,anderson_multivariate_2006}{SampleCore}.
#'
#'   The distance of each accession \mjseqn{g} to the group centroid \mjseqn{C}
#'   in PCoA space is:
#'
#'   \mjsdeqn{\delta_g = \| \mathbf{p}_g - \mathbf{c} \|}
#'
#'   Where \mjseqn{\mathbf{p}_g} is the PCoA coordinate vector of accession
#'   \mjseqn{g} and \mjseqn{\mathbf{c}} is the group centroid. Accessions are
#'   ranked by \mjseqn{\delta_g} in ascending order and the top \mjseqn{n} are
#'   selected.
#'
#'   }
#'
#'   \subsection{Representative Sampling by Proximity to Group Spatial Median}{
#'   Selects accessions closest to the group spatial median in principal
#'   coordinate space, computed via multivariate dispersion analysis using
#'   \code{\link[vegan]{betadisper}}
#'   \insertCite{oneill_theory_2000}{SampleCore}.
#'
#'   The distance of each accession \mjseqn{g} to the group spatial median
#'   \mjseqn{M} is:
#'
#'   \mjsdeqn{\delta_g^* = \| \mathbf{p}_g - \mathbf{m} \|}
#'
#'   where \mjseqn{\mathbf{m}} is the spatial median of the group in PCoA
#'   space. Accessions are ranked by \mjseqn{\delta_g^*} in ascending order and
#'   the top \mjseqn{n} are selected.
#'
#'   }
#'
#' }
#'
#' \subsection{Peripheral/Extremity Based Methods}{
#'
#'   Selects accessions that are most dissimilar from the rest in a
#'   cluster/group i.e. the accessions which are in the boundary or outliers.
#'
#'   \subsection{Peripheral Sampling by Maximal Mean Distance}{Selects the most
#'   peripheral accessions as those with the largest average distance to all
#'   others within the group
#'   \insertCite{kaufman_clustering_1987,kaufman_finding_1990}{SampleCore}.
#'
#'   \mjsdeqn{\bar{d}_g = \frac{1}{G} \sum_{h=1}^{G} d_{gh}}
#'
#'   Accessions are ranked by \mjseqn{\bar{d}_g} in descending order and the
#'   top \mjseqn{n} are selected.
#'
#'   }
#'
#'   \subsection{Peripheral Sampling by Maximal Median Distance}{Selects the
#'   most peripheral accessions as those with the largest median distance to
#'   all others within the group
#'   \insertCite{kaufman_clustering_1987,kaufman_finding_1990}{SampleCore}.
#'
#'   \mjsdeqn{\tilde{d}_g = \text{median}_{h=1,\dots,G}(d_{gh})}
#'
#'   Accessions are ranked by \mjseqn{\tilde{d}_g} in descending order and the
#'   top \mjseqn{n} are selected.
#'
#'   }
#'
#'   \subsection{Peripheral Sampling by Maximal Eccentricity}{Selects
#'   accessions with the largest eccentricity — the maximum distance to any
#'   other accession in the group
#'   \insertCite{hage_eccentricity_1995}{SampleCore}.
#'
#'   \mjsdeqn{e_g = \max_{h=1,\dots,G} d_{gh}}
#'
#'   Accessions are ranked by \mjseqn{e_g} in descending order and the top
#'   \mjseqn{n} are selected. Eccentricity captures the worst-case
#'   dissimilarity of an accession rather than its average behaviour.
#'
#'   }
#'
#'   \subsection{Peripheral Sampling by Maximal Farness Centrality}{Selects
#'   accessions with the greatest total distance to all others, i.e. those most
#'   remote from the rest of the group
#'   \insertCite{sabidussi_centrality_1966}{SampleCore}.
#'
#'   \mjsdeqn{f_g = \sum_{h=1}^{G} d_{gh}}
#'
#'   Accessions are ranked by \mjseqn{f_g} in descending order and the top
#'   \mjseqn{n} are selected. Farness centrality is proportional to
#'   \mjseqn{\bar{d}_g} and differs from \code{mean.peripheral} only in that it
#'   uses the raw sum rather than the mean, producing identical rankings.
#'
#'   }
#'
#' }
#'
#' \subsection{Space-Filling/Coverage Methods}{
#'
#'   Select accessions that are spread maximally across the feature space in a
#'   cluster/group i.e. diversity sampling.
#'
#'   \subsection{Space-Filling Sampling via the Kennard-Stone
#'   Algorithm}{Selects \mjseqn{n} accessions that maximally and uniformly
#'   cover the distance space via the Kennard-Stone algorithm
#'   \insertCite{kennard_computer_1969}{SampleCore} (See
#'   \code{\link[prospectr]{kenStone}}).
#'
#'   Starting from the pair of accessions with the largest pairwise distance:
#'
#'   \mjsdeqn{\lbrace g_1, g_2 \rbrace = \underset{g,h}{\arg\max}\, d_{gh}}
#'
#'   each subsequent accession \mjseqn{g_k} is selected by maximising its
#'   minimum distance to the already-selected set \mjseqn{S}:
#'
#'   \mjsdeqn{g_k = \underset{g \notin S}{\arg\max} \min_{s \in S} d_{gs}}
#'
#'   This greedy procedure ensures even space coverage without relying on
#'   cluster structure.
#'
#'   }
#'
#'   \subsection{Space-Filling Sampling via the DUPLEX Algorithm}{Extends the
#'   Kennard-Stone algorithm to simultaneously construct a model set and a test
#'   set with similar distributions
#'   \insertCite{kennard_computer_1969,snee_validation_1977}{SampleCore}
#'   (\link[prospectr]{duplex}). Accessions are selected using Mahalanobis
#'   distance:
#'
#'   \mjsdeqn{d_M(g, h) = \sqrt{(\mathbf{x}_g - \mathbf{x}_h)^\top \Sigma^{-1}
#'   (\mathbf{x}_g - \mathbf{x}_h)}}
#'
#'   where \mjseqn{\Sigma} is the covariance matrix. At each step, the pair
#'   maximising \mjseqn{d_M} is split alternately between model and test sets,
#'   ensuring both sets span the full feature space.
#'
#'   }
#'
#'   \subsection{Space-Filling Sampling via the Honigs Algorithm}{Selects
#'   \mjseqn{n} accessions sequentially by maximising dissimilarity to the
#'   already-selected set \insertCite{honigs_unique-sample_1985}{SampleCore}
#'   (\link[prospectr]{honigs})
#'
#'   At each step \mjseqn{k}, the accession \mjseqn{g_k} maximising total
#'   distance to all previously selected accessions \mjseqn{S} is chosen:
#'
#'   \mjsdeqn{g_k = \underset{g \notin S}{\arg\max} \sum_{s \in S} d_{gs}}
#'
#'   This favours accessions that are collectively most dissimilar to the
#'   current selection, producing broad coverage of the distance space.
#'
#'   }.
#'
#'   \subsection{Space-Filling Sampling via Farthest-Point (Max-Min)
#'   Algorithm}{Selects \mjseqn{n} accessions by iteratively maximising the
#'   minimum distance to the current selected set — also known as the
#'   max-min or farthest-point sampling algorithm
#'   \insertCite{gonzalez_clustering_1985,dyer_simple_1985,hochbaum_best_1985}{SampleCore}.
#'
#'   \mjsdeqn{g_k = \underset{g \notin S}{\arg\max} \min_{s \in S} d_{gs}}
#'
#'   This is equivalent to Kennard-Stone but without the symmetric
#'   initialisation step. It provides a deterministic, greedy approximation to
#'   the \mjseqn{k}-centre problem:
#'
#'   \mjsdeqn{\min_{S \subset G,\, |S|=n} \max_{g \in G} \min_{s \in S} d_{gs}}
#'
#'   }
#'
#' }
#'
#' \subsection{Density Based Methods}{
#'
#'   Select points based on local neighbourhood density.
#'
#'   \subsection{Density-Based Sampling by Minimal Nearest-Neighbour
#'   Distance}{Selects accessions residing in the densest regions of the
#'   distance space, identified as those with the smallest nearest-neighbour
#'   distance
#'   \insertCite{cover_nearest_1967,fix_discriminatory_1989}{SampleCore}.
#'
#'   For each accession \mjseqn{g}, the nearest-neighbour distance is:
#'
#'   \mjsdeqn{\text{nn}_g = \min_{h \neq g} d_{gh}}
#'
#'   Accessions are ranked by \mjseqn{\text{nn}_g} in ascending order and the
#'   top \mjseqn{n} are selected. Small \mjseqn{\text{nn}_g} indicates that
#'   \mjseqn{g} resides in a dense cluster; this method preferentially samples
#'   from high-density regions.
#'
#'   }
#'
#' }
#'
#' \subsection{Cluster Based Methods}{
#'
#'   These methods partition the cluster/group space into sub-clusters/groups,
#'   then samples from each one.
#'
#'   \subsection{Globally Optimal Medoid Sampling via Partitioning Around
#'   Medoids (PAM)}{
#'
#'   Selects a set of \mjseqn{n} medoids that jointly minimise the total
#'   distance of every accession to its nearest medoid, via
#'   \code{\link[cluster]{pam}}.
#'
#'   The objective function minimised is:
#'
#'   \mjsdeqn{\min_{S \subset G,\, |S|=n} \sum_{g=1}^{G} \min_{s \in S} d_{gs}}
#'
#'   Unlike \code{"mean.medoid"}, medoids are co-optimised as a set, ensuring
#'   they collectively represent the full distribution of the group rather than
#'   independently scoring each accession.
#'
#'   }
#'
#'   \subsection{Cluster-Based Sampling via K-means (Naes Method)}{Partitions
#'   accessions into \mjseqn{n} clusters via k-means applied to the distance
#'   matrix (See \code{\link[prospectr]{naes}}), then selects the accession
#'   closest to each cluster centre as the representative
#'   \insertCite{naes_design_1987,naes_user-friendly_2017}{SampleCore}.
#'
#'   The k-means objective minimised is:
#'
#'   \mjsdeqn{\min \sum_{k=1}^{n} \sum_{g \in C_k} d_{g, \mu_k}^2}
#'
#'   where \mjseqn{C_k} is the \mjseqn{k}-th cluster and \mjseqn{\mu_k} is its
#'   centre. One representative per cluster is returned, ensuring broad,
#'   partition-aware coverage.
#'
#'   }
#'
#'   \subsection{Cluster-Based Sampling via Hierarchical Clustering with
#'   Random Selection}{Partitions accessions into \mjseqn{n} clusters by
#'   cutting a hierarchical clustering dendrogram at height \mjseqn{k = n},
#'   then randomly samples one accession from each cluster
#'   \insertCite{ward_Hierarchical_1963,li_studies_2002}{SampleCore}.
#'
#'   The dendrogram is built by agglomerative hierarchical clustering using the
#'   linkage criterion specified by \code{\link[stats]{hclust}}. For
#'   clusters \mjseqn{C_1, \dots, C_n}, one accession is drawn uniformly at
#'   random from each:
#'
#'   \mjsdeqn{g_k \sim \text{Uniform}(C_k), \quad k = 1, \dots, n}
#'
#'   This introduces stochasticity within a structured partition, balancing
#'   coverage with randomness.
#'
#'   }
#'
#'   \subsection{Cluster-Based Sampling via Hierarchical Clustering with Medoid
#'   Selection}{Partitions accessions into \mjseqn{n} clusters by cutting a
#'   hierarchical clustering dendrogram at height \mjseqn{k = n}, then selects
#'   the within-cluster medoid as the representative of each cluster
#'   \insertCite{kaufman_clustering_1987,ward_Hierarchical_1963}{SampleCore}.
#'
#'   For each cluster \mjseqn{C_k}, the medoid is the accession minimising
#'   total within-cluster distance:
#'
#'   \mjsdeqn{g_k^* = \underset{g \in C_k}{\arg\min} \sum_{h \in C_k} d_{gh}}
#'
#'   This combines the structured partitioning of hierarchical clustering with
#'   deterministic, centrality-based representative selection.
#'
#'   }
#'
#' }
#'
#' @template general-arg
#' @template sel-arg
#' @template dist-arg
#' @template seldist-arg
#' @param method The method for sampling accessions from each cluster/group.
#'   Either \code{"mean.medoid"}, \code{"median.medoid"},
#'   \code{"nearest.centroid"}, \code{"nearest.median"},
#'   \code{"mean.peripheral"}, \code{"median.peripheral"},
#'   \code{"eccentricity"}, \code{"farness.centrality"}, \code{"kennard.stone"},
#'   \code{"duplex"}, \code{"honigs"}, \code{"farthest.point"},
#'   \code{"nearest.neighbour"}, \code{"naes"}, \code{"optim.medoid"},
#'   \code{"hclust.random"} or \code{"hclust.medoid"}. See \strong{Methods}.
#'
#' @returns A named list where each element contains the selected entry
#'   identifiers for a cluster/group.
#'
#' @importFrom cluster pam
#' @importFrom graphics par title
#' @importFrom grDevices n2mfrow
#' @importFrom MASS isoMDS
#' @importFrom prospectr duplex honigs kenStone naes
#' @importFrom stats cmdscale cutree hclust
#' @importFrom vegan betadisper
#'
#' @export
#'
#' @references
#'
#' \insertAllCited
#'
#' @seealso \code{\link[SampleCore]{select.random}},
#'   \code{\link[SampleCore]{select.diversity}}
#'
#' @examplesIf requireNamespace("cluster", quietly = TRUE) & requireNamespace("ggplot2", quietly = TRUE)
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' library(cluster)
#' library(ggplot2)
#'
#' data(cassava_EC_gp)
#'
#' set.seed(123)
#' data <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]
#'
#' quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW", "AVPW",
#'            "ARSR", "SRDM")
#' qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'           "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'           "PSTR")
#'
#' data[, qual] <- lapply(data[, qual], as.factor)
#'
#' # Get the Gower's distance matrix
#' dist_matrix <- daisy(x = data[, c(qual, quant)],
#'                      metric = "gower")
#'
#'
#' data <- cbind(genotypes = rownames(data), data)
#' row.names(data) <- NULL
#'
#' # Prepare inputs
#' counts <- c(I = 16, II = 15, III = 9, IV = 18, V = 20, VI = 8)
#'
#' mand_accns <-
#'   c("TMe-2018", "TMe-801", "TMe-3191", "TMe-1830", "TMe-1790")
#'
#' gp_vec <- setNames(as.character(data[, "Cluster"]), data[, "genotypes"])
#'
#' \donttest{
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'   # Fetch selected accessions by centrality based methods
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   # Medoid-like Representative Sampling by Minimal Mean Distance
#'   sel_mean_medoid_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "mean.medoid")
#'   sel_mean_medoid_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_mean_medoid_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "mean.medoid")
#'
#'   # Medoid-like Representative Sampling by Minimal Median Distance
#'   sel_median_medoid_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "median.medoid")
#'   sel_median_medoid_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_median_medoid_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "median.medoid")
#'
#'   # Representative Sampling by Proximity to Group Centroid
#'   sel_group_centroid_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "nearest.centroid")
#'   sel_group_centroid_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_group_centroid_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "nearest.centroid")
#'
#'   # Representative Sampling by Proximity to Group Spatial Median
#'   sel_group_median_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "nearest.median")
#'   sel_group_median_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_group_median_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "nearest.median")
#'
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'   # Fetch selected accessions by peripheral/extremity based methods
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   # Peripheral Sampling by Maximal Mean Distance
#'   sel_mean_peripheral_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "mean.peripheral")
#'   sel_mean_peripheral_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_mean_peripheral_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "mean.peripheral")
#'
#'   # Peripheral Sampling by Maximal Median Distance
#'   sel_median_peripheral_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "median.peripheral")
#'   sel_median_peripheral_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_median_peripheral_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "median.peripheral")
#'
#'   # Peripheral Sampling by Maximal Eccentricity
#'   sel_eccentricity_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "eccentricity")
#'   sel_eccentricity_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_eccentricity_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "eccentricity")
#'
#'   # Peripheral Sampling by Maximal Farness Centrality
#'   sel_far_cent_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "farness.centrality")
#'   sel_far_cent_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_far_cent_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "farness.centrality")
#'
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'   # Fetch selected accessions by space-Filling/coverage methods
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   # Space-Filling Sampling via the Kennard-Stone Algorithm
#'   sel_ks_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "kennard.stone")
#'   sel_ks_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_ks_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "kennard.stone")
#'
#'   # Space-Filling Sampling via the DUPLEX Algorithm
#'   sel_duplex_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "duplex")
#'   sel_duplex_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_duplex_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "duplex")
#'
#'   # Space-Filling Sampling via the Honigs Algorithm
#'   sel_honigs_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "honigs")
#'   sel_honigs_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_honigs_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "honigs")
#'
#'   # Space-Filling Sampling via Farthest-Point (Max-Min) Algorithm
#'   sel_far_pt_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "farthest.point")
#'   sel_far_pt_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_far_pt_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "farthest.point")
#'
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'   # Fetch selected accessions by density based methods
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   # Density-Based Sampling by Minimal Nearest-Neighbour Distance
#'   sel_nn_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "nearest.neighbour")
#'   sel_nn_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_nn_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "nearest.neighbour")
#'
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'   # Fetch selected accessions by cluster based methods
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   # Globally Optimal Medoid Sampling via Partitioning Around Medoids (PAM)
#'   sel_pam_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "optim.medoid")
#'   sel_pam_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_pam_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "optim.medoid")
#'
#'   # Cluster-Based Sampling via K-means (Naes Method)
#'   sel_naes_out <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "naes")
#'   sel_naes_out
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_naes_out,
#'                                 use.names = FALSE)) +
#'     labs(title = "naes")
#'
#'   # Cluster-Based Sampling via Hierarchical Clustering with Random Selection
#'
#'   ## UPGMA
#'   sel_hclust_random_out1 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.random",
#'                     hclust.method = "average")
#'   sel_hclust_random_out1
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_random_out1,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.random", subtitle = "average")
#'
#'   ## Single-linkage
#'   sel_hclust_random_out2 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.random",
#'                     hclust.method = "single")
#'   sel_hclust_random_out2
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_random_out2,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.random", subtitle = "single")
#'
#'   ## Complete-linkage
#'   sel_hclust_random_out3 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.random",
#'                     hclust.method = "complete")
#'   sel_hclust_random_out3
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_random_out3,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.random", subtitle = "complete")
#'
#'   ## Ward's D
#'   sel_hclust_random_out4 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.random",
#'                     hclust.method = "ward.D")
#'   sel_hclust_random_out4
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_random_out4,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.random", subtitle = "ward.D")
#'
#'   ## WPGMA
#'   sel_hclust_random_out5 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.random",
#'                     hclust.method = "mcquitty")
#'   sel_hclust_random_out5
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_random_out5,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.random", subtitle = "mcquitty")
#'
#'   ## WPGMC
#'   sel_hclust_random_out6 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.random",
#'                     hclust.method = "median")
#'   sel_hclust_random_out6
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_random_out6,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.random", subtitle = "median")
#'
#'   ## UPGMC
#'   sel_hclust_random_out7 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.random",
#'                     hclust.method = "centroid")
#'   sel_hclust_random_out7
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_random_out7,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.random", subtitle = "centroid")
#'
#'   ## Ward's D2
#'   sel_hclust_random_out8 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.random",
#'                     hclust.method = "ward.D2")
#'   sel_hclust_random_out8
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_random_out8,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.random", subtitle = "ward.D2")
#'
#'   # Cluster-Based Sampling via Hierarchical Clustering with Medoid Selection
#'
#'   ## UPGMA
#'   sel_hclust_medoid_out1 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.medoid",
#'                     hclust.method = "average")
#'   sel_hclust_medoid_out1
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_medoid_out1,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.medoid", subtitle = "average")
#'
#'   ## Single-linkage
#'   sel_hclust_medoid_out2 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.medoid",
#'                     hclust.method = "single")
#'   sel_hclust_medoid_out2
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_medoid_out2,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.medoid", subtitle = "single")
#'
#'   ## Complete-linkage
#'   sel_hclust_medoid_out3 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.medoid",
#'                     hclust.method = "complete")
#'   sel_hclust_medoid_out3
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_medoid_out3,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.medoid", subtitle = "complete")
#'
#'   ## Ward's D
#'   sel_hclust_medoid_out4 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.medoid",
#'                     hclust.method = "ward.D")
#'   sel_hclust_medoid_out4
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_medoid_out4,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.medoid", subtitle = "ward.D")
#'
#'   ## WPGMA
#'   sel_hclust_medoid_out5 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.medoid",
#'                     hclust.method = "mcquitty")
#'   sel_hclust_medoid_out5
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_medoid_out5,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.medoid", subtitle = "mcquitty")
#'
#'   ## WPGMC
#'   sel_hclust_medoid_out6 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.medoid",
#'                     hclust.method = "median")
#'   sel_hclust_medoid_out6
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_medoid_out6,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.medoid", subtitle = "median")
#'
#'   ## UPGMC
#'   sel_hclust_medoid_out7 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.medoid",
#'                     hclust.method = "centroid")
#'   sel_hclust_medoid_out7
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_medoid_out7,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.medoid", subtitle = "centroid")
#'
#'   ## Ward's D2
#'   sel_hclust_medoid_out8 <-
#'     select.distance(data = data, names = "genotypes",
#'                     group = "Cluster", alloc = counts,
#'                     dist.mat = dist_matrix,
#'                     always.selected = mand_accns,
#'                     method = "hclust.medoid",
#'                     hclust.method = "ward.D2")
#'   sel_hclust_medoid_out8
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(sel_hclust_medoid_out8,
#'                                 use.names = FALSE)) +
#'     labs(title = "hclust.medoid", subtitle = "ward.D2")
#' }
#'
select.distance <- function(data, names, group, alloc,
                            dist.mat, always.selected = NULL,
                            method = c("mean.medoid", "median.medoid",
                                       "nearest.centroid", "nearest.median",
                                       "mean.peripheral", "median.peripheral",
                                       "eccentricity", "farness.centrality",
                                       "kennard.stone", "duplex", "honigs",
                                       "farthest.point",
                                       "nearest.neighbour",
                                       "naes", "optim.medoid",
                                       "hclust.random", "hclust.medoid"),
                            hclust.method = c("average", "single",
                                              "complete",
                                              "ward.D", "mcquitty",
                                              "median", "centroid",
                                              "ward.D2")) {

  SampleCore.debug <- getOption("SampleCore.debug", default = FALSE)

  method <- match.arg(method)

  checks.sample.core(data = data, size = NULL,
                     names = names, group = group,
                     dist.mat = dist.mat,
                     qualitative = NULL,
                     log.base = NULL,
                     alloc = alloc,
                     always.selected = always.selected,
                     mode = "sel")

  # Prepare data ----

  dmat <- as.matrix(dist.mat)

  gp_memb <- data[, group]
  dist_labels <- labels(dist.mat)

  # Align labels
  names(gp_memb) <- dist_labels
  gp_memb <- gp_memb[rownames(dmat)]

  if (method == "nearest.centroid") {
    bdout <- vegan::betadisper(d = dist.mat, group = data[, group],
                               type = "centroid", sqrt.dist = TRUE)
    dist_to_centroid <- split(bdout$distances, bdout$group)
  }

  if (method == "nearest.median") {
    bdout <- vegan::betadisper(d = dist.mat, group = data[, group],
                               type = "median", sqrt.dist = TRUE)
    dist_to_median <- split(bdout$distances, bdout$group)
  }

  out <-
    lapply(names(alloc), function(g) {

      group_accns <- data[data[[group]] == g, names]

      # always-selected values belonging to this group
      fixed_accns <- intersect(always.selected, group_accns)

      requested_n <- alloc[[g]]

      # Remaining pool after fixed selections
      rem_accns <- setdiff(group_accns, fixed_accns)
      n_rem <- requested_n - length(fixed_accns)

      if (n_rem < 0) {

        warning(
          sprintf(
            paste0('Group %s: "alloc" (%d) is smaller than number of ',
                   '"always.selected" values (%d). ',
                   'Taking only "always.selected" values.'),
            g, requested_n, length(fixed_accns)
          ),
          call. = FALSE
        )

        sampled_accns <- fixed_accns

      } else {

        idx <- which(labels(dist.mat) %in% rem_accns)

        sub_d <- dmat[idx, idx]

        avg_dist <- rowMeans(as.matrix(sub_d))
        med_dist <- apply(as.matrix(sub_d), 1, median)

        avail_rem <- length(rem_accns)

        if (avail_rem < n_rem) {

          warning(
            sprintf(
              paste0('Group %s has only %d additional accessions but ',
                     '"alloc" requires %d additional entries. ',
                     'Taking all available accessions.'),
              g, avail_rem, n_rem
            ),
            call. = FALSE
          )

          n_rem <- avail_rem
        }

        if (n_rem == 1) {
          sampled_accns <- rem_accns[which.min(avg_dist)]
        } else if (avail_rem < 3 && method %in% c("kennard.stone", "duplex",
                                                  "honigs", "naes")) {
          warning(
            sprintf(
              paste0('Group %s: method "%s" requires at least 3 accessions ',
                     'but only %d available. Falling back to "mean.medoid".'),
              g, method, avail_rem
            ),
            call. = FALSE
          )
          center_idx <- order(avg_dist)[1:n_rem]
          sampled_accns <- rem_accns[center_idx]

        } else { # Method dispatch

          # Centrality Based ----

          ## Mean medoid ----
          if (method == "mean.medoid") {
            center_idx <- order(avg_dist)[1:n_rem]
            sampled_accns <- rem_accns[center_idx]
          }

          ## Median medoid ----
          if (method == "median.medoid") {
            center_idx2 <- order(med_dist)[1:n_rem]
            sampled_accns <- rem_accns[center_idx2]
          }

          ## Nearest to centroid ----
          if (method == "nearest.centroid") {
            dist_to_centroid_rem <- dist_to_centroid[[g]][rem_accns]
            center_idx3 <- order(dist_to_centroid_rem)[1:n_rem]
            sampled_accns <- names(dist_to_centroid[[g]])[center_idx3]
          }

          ## Nearest to centroid ----
          if (method == "nearest.median") {
            dist_to_median_rem <- dist_to_median[[g]][rem_accns]
            center_idx4 <- order(dist_to_median_rem)[1:n_rem]
            sampled_accns <- names(dist_to_median[[g]])[center_idx4]
          }

          # Peripheral / Extremity-Based ----

          ## Mean peripheral -----
          if (method == "mean.peripheral") {
            extreme_idx <- order(avg_dist, decreasing = TRUE)[1:n_rem]
            sampled_accns <- rem_accns[extreme_idx]
          }

          ## Median peripheral ----
          if (method == "median.peripheral") {
            extreme_idx2 <- order(med_dist, decreasing = TRUE)[1:n_rem]
            sampled_accns <- rem_accns[extreme_idx2]
          }

          ## Eccentricity ----
          if (method == "eccentricity") {
            ecc <- apply(as.matrix(sub_d), 1, max)
            ecc_idx <- order(ecc, decreasing = TRUE)[1:n_rem]
            sampled_accns <- rem_accns[ecc_idx]
          }

          ## Farness Centrality ----
          if (method == "farness.centrality") {
            far <- apply(as.matrix(sub_d), 1, sum)
            far_idx <- order(far, decreasing = TRUE)[1:n_rem]
            sampled_accns <- rem_accns[far_idx]
          }

          # Space-Filling / Coverage Methods ----

          ## Kennard-Stone algorithm ----
          if (method == "kennard.stone") {
            kens_res <- prospectr::kenStone(sub_d, k = n_rem)
            sampled_accns <- rem_accns[kens_res$model]
          }

          ## DUPLEX algorithm ----
          if (method == "duplex") {
            dupl_res <- prospectr::duplex(sub_d, k = n_rem, metric = "mahal")
            sampled_accns <- rem_accns[dupl_res$model]
          }

          ## Honigs algorithm ----
          if (method == "honigs") {
            ho_res <- prospectr::honigs(sub_d, k = n_rem)
            sampled_accns <- rem_accns[ho_res$model]
          }

          ## Max-min / Farthest-point sampling ----
          if (method == "farthest.point") {
            farpoint_idx <- farthest_sampling(d = sub_d, n_select = n_rem)
            sampled_accns <- rem_accns[farpoint_idx]
          }

          # Density Based Methods ----

          ## Nearest-neighbour (NN) ----
          if (method == "nearest.neighbour") {
            nn <- apply(as.matrix(sub_d) + diag(Inf, nrow(sub_d)), 1, min)
            nn_idx <- order(nn)[1:n_rem]
            sampled_accns <- rem_accns[nn_idx]
          }

          # Cluster-Based Methods ----

          ## Partitioning around medoids ---
          if (method == "optim.medoid") {
            pam_res <- cluster::pam(sub_d, k = n_rem, diss = TRUE)
            sampled_accns <- pam_res$medoids
          }

          ## K-means sampling (Naes Method) ----
          if (method == "naes") {
            kms_res <- prospectr::naes(sub_d, k = n_rem)
            sampled_accns <- rem_accns[kms_res$model]
          }

          ## Hierarchical Clustering with random selection ----
          if (method %in% c("hclust.random", "hclust.medoid")) {
            tree_out <- hclust(as.dist(sub_d), method = hclust.method)
            k_out <- cutree(tree_out, k = n_rem)
            k_out_list <- split(names(k_out), k_out)
          }

          if (method == "hclust.random") {
            sampled_accns <- sapply(k_out_list,
                                    function(x) {
                                      sample(x, 1)
                                    })
            sampled_accns <- unname(sampled_accns)
          }

          ## Hierarchical Clustering and medoids ----
          if (method == "hclust.medoid") {
            sampled_accns <- sapply(k_out_list, function(members) {
              # cluster submatrix
              k_d <- dmat[members, members, drop = FALSE]
              # total within-cluster distance
              tot_dist <- rowSums(k_d)
              # medoid
              names(which.min(tot_dist))
            })
            sampled_accns <- unname(sampled_accns)
          }
        }
      }

      return(c(sampled_accns, fixed_accns))

    })

  names(out) <- names(alloc)

  if (SampleCore.debug) {

    gp_vec <- setNames(as.character(data[, group]), data[, names])

    viz_out <-
      plot_dist(d = dist.mat, method = "isomds", gp = gp_vec,
                highlight = unlist(out, use.names = FALSE))
    print(viz_out)

  }

  return(out)
}

## Max-min / Farthest-point sampling function ----
farthest_sampling <- function(d, n_select,
                              start = sample(nrow(d), 1)) {

  d <- as.matrix(d)

  selected <- start

  while(length(selected) < n_select) {

    remaining <- setdiff(seq_len(nrow(d)), selected)

    min.dist <- sapply(remaining, function(i) {
      min(d[i, selected])
    })

    next.idx <- remaining[which.max(min.dist)]

    selected <- c(selected, next.idx)
  }

  selected
}

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Selection of Entries from Clusters/Groups on the basis of Optimized Diversity
#'
#' Select entries from cluster/groups in the entire collection which form a
#' subset with the highest trait diversity according to a either pooled or mean
#' diversity index estimate. \loadmathjax
#'
#' To identify subsets with highest diversity estimates, the following
#' strategies are available. These strategies are similar to the "Maximization"
#' or M strategy of \insertCite{schoen_conservation_1993;textual}{SampleCore}.
#'
#'  \subsection{Random search / Monte Carlo Method}{For each cluster/group,
#'  multiple candidate subsets are sampled randomly and the subset with the
#'  highest trait diversity according to either pooled or mean diversity index
#'  estimate is retained. The quality of the solution improves with increasing
#'  \code{n.iter} but is not guaranteed to find the global optimum
#'  \insertCite{anatoly_zhigljavsky_stochastic_2008}{SampleCore}.
#'  }
#'
#'  \subsection{Greedy search with 1-opt}{This method builds a solution
#'  incrementally by adding the accession that maximises the diversity score at
#'  each step, starting from the \code{always.selected} accessions (or a single
#'  randomly drawn accession when there are no accessions specified in
#'  \code{always.selected}) present in the particular cluster/group
#'  \insertCite{nemhauser_analysis_1978,fisher_analysis_1978,cormen_introduction_2022}{SampleCore}.
#'  The 'greedy' solution is then refined by a 1-opt local search controlled by
#'  \code{local.search} and \code{max.iter}
#'  \insertCite{lin_computer_1965}{SampleCore}. Greedy search is deterministic
#'  given a fixed \code{always.selected} set; when there are no accessions
#'  specified in \code{always.selected} present in the particular cluster/group
#'  results may vary across runs due to the random initialisation.
#'
#'  \code{local.search = "best.improvement"} scans all possible single swaps
#'  in each pass and applies the one yielding the greatest improvement before
#'  restarting. his guarantees the steepest ascent at each pass but requires
#'  evaluating all \mjseqn{k \times (n - k)} swap pairs per pass, where
#'  \mjseqn{k} is the number of swappable accessions and \mjseqn{n - k} is the
#'  size of the candidate pool
#'  \insertCite{papadimitriou_combinatorial_1998}{SampleCore}.
#'
#'  \code{local.search = "first.improvement"} applies the first swap that
#'  improves the score and immediately restarts the search. This typically
#'  requires fewer score evaluations per pass and converges faster, but may
#'  find a different local optimum than \code{"best.improvement"}
#'  \insertCite{papadimitriou_combinatorial_1998}{SampleCore}.
#'
#'  Both strategies terminate when no improving swap exists (local optimum)
#'  or when \code{max.iter} passes have been completed.
#'  }
#'
#' Entries listed as \code{always.selected} are mandatorily included in the
#' selection. Warnings are issued if requested allocation is smaller than the
#' number of always-selected entries in a cluster/group and/or when the
#' cluster/group does not contain enough remaining entries to fulfill the
#' allocation.
#'
#' @template general-arg
#' @template sel-arg
#' @template qual-arg
#' @template div-arg
#' @param n.iter Integer specifying the number of random candidate subsets
#'   generated per group to optimze the diversity for random search
#'   (\code{search = "random"}).
#' @param max.iter The maximum number of 1-opt passes for greedy search
#'   (\code{search = "greedy"}).
#' @param search Character string specifying the search strategy used to find
#'   the subset with the highest diversity score. Either \code{"random"}
#'   (default) or \code{"greedy"} (See \strong{Details}).
#' @param local.search Character string specifying the local search strategy
#'   used in the 1-opt improvement phase of the greedy search (\code{search =
#'   "greedy"}). Either \code{"best.improvement"} (default) or
#'   \code{"first.improvement"}. Ignored when \code{search = "random"}.
#'
#' @returns A named list where each element contains the selected entry
#'   identifiers for a cluster/group.
#'
#' @importFrom DiversityStats simpson shannon mcintosh_diversity
#' @export
#'
#' @references
#'
#' \insertAllCited
#'
#' @seealso \code{\link[SampleCore]{select.random}},
#'   \code{\link[SampleCore]{select.distance}}
#'
#' @examples
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' library(cluster)
#' library(ggplot2)
#'
#' data(cassava_EC_gp)
#'
#' set.seed(123)
#' cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]
#'
#' data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)
#'
#' quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW", "AVPW",
#'            "ARSR", "SRDM")
#' qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'           "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'           "PSTR")
#'
#' # Convert qualitative data columns to factor
#' data[, qual] <- lapply(data[, qual], as.factor)
#'
#' # Convert quantitative data columns to qualitative scores
#' quant_to_score5 <- function(x) {
#'
#'   brks <- unique( quantile(x,
#'                            probs = seq(0, 1, 0.2),
#'                            na.rm = TRUE))
#'   cut(x, breaks = brks,
#'       include.lowest = TRUE,
#'       labels = seq_len(length(brks) - 1))
#' }
#'
#' data[, quant] <- lapply(data[, quant], quant_to_score5)
#'
#' traits <- c(quant, qual)
#'
#' # Prepare inputs
#' counts <- c(I = 31, II = 31, III = 18, IV = 35, V = 40, VI = 17)
#'
#' mand_accns <-
#'   c("TMe-2018", "TMe-801", "TMe-3191", "TMe-1830", "TMe-1790")
#'
#' # Get distance matrix - Only for visualization
#'
#' # Convert qualitative data columns to factor
#' cassava_EC_gp[, qual] <- lapply(cassava_EC_gp[, qual], as.factor)
#'
#' # Standardise quantitative data column
#' cassava_EC_gp[, quant] <- lapply(cassava_EC_gp[, quant], function(x) {
#'   scale(x)[, 1]
#' })
#'
#' gp_vec <- setNames(as.character(data[, "Cluster"]), data[, "genotypes"])
#'
#' # Get the Gower's distance matrix
#' dist_matrix <- daisy(x = cassava_EC_gp[, c(qual, quant)],
#'                      metric = "gower")
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Custom Diversity functions
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' div_fun_brillouin <- function(x) {
#'   n <- tabulate(x)
#'   n <- n[n > 0]
#'   N <- sum(n)
#'   if (N <= 1) {
#'     return(0)
#'   }
#'   (lgamma(N + 1) - sum(lgamma(n + 1)))/N
#' }
#'
#' div_fun_margalef <- function(x) {
#'   tab <- tabulate(x)
#'   tab <- tab[tab > 0]
#'   S <- length(tab)
#'   N <- length(x)
#'   if (N <= 1) {
#'     return(0)
#'   }
#'   (S - 1)/log(N)
#' }
#'
#' \donttest{
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'   # Random search
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   # Mean richness
#'   randomsel_mean_richness <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "richness",
#'                      metric = "mean", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_mean_richness
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_mean_richness,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search", subtitle = "Mean richness")
#'
#'   # Pooled richness
#'   randomsel_sum_richness <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "richness",
#'                      metric = "pooled", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_sum_richness
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_sum_richness,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search", subtitle = "Pooled richness")
#'
#'   # Mean Shannon-Weaver diversity index
#'   randomsel_mean_shannon <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "shannon",
#'                      metric = "mean", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_mean_shannon
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_mean_shannon,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Mean Shannon-Weaver diversity index")
#'
#'   # Pooled Shannon-Weaver diversity index
#'   randomsel_sum_shannon <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "shannon",
#'                      metric = "pooled", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_sum_shannon
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_sum_shannon,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Pooled Shannon-Weaver diversity index")
#'
#'   # Mean Gini-Simpson diversity index
#'   randomsel_mean_simpson <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "simpson",
#'                      metric = "mean", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_mean_simpson
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_mean_simpson,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Mean Gini-Simpson diversity index")
#'
#'   # Pooled Gini-Simpson diversity index
#'   randomsel_sum_simpson <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "simpson",
#'                      metric = "pooled", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_sum_simpson
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_sum_simpson,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Pooled Gini-Simpson diversity index")
#'
#'   # Mean McIntosh diversity index
#'   randomsel_mean_mcintosh <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "mcintosh",
#'                      metric = "pooled", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_mean_mcintosh
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_mean_mcintosh,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Mean McIntosh diversity index")
#'
#'   # Pooled McIntosh diversity index
#'   randomsel_sum_mcintosh <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "mcintosh",
#'                      metric = "pooled", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_sum_mcintosh
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_sum_mcintosh,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Pooled McIntosh diversity index")
#'
#'   # Mean Brillouin diversity index
#'   randomsel_mean_brillouin <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.fun = div_fun_brillouin,
#'                      metric = "mean", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_mean_brillouin
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_mean_brillouin,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Mean Brillouin diversity index")
#'
#'   # Pooled Brillouin diversity index
#'   randomsel_sum_brillouin <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.fun = div_fun_brillouin,
#'                      metric = "pooled", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_sum_brillouin
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_sum_brillouin,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Pooled Brillouin diversity index")
#'
#'
#'   # Mean Margalef's richness index
#'   randomsel_mean_margalef <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.fun = div_fun_margalef,
#'                      metric = "mean", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_mean_margalef
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_mean_margalef,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Mean Margalef's diversity index")
#'
#'   # Pooled Margalef's richness index
#'   randomsel_sum_margalef <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.fun = div_fun_margalef,
#'                      metric = "pooled", search = "random", local.search = NULL,
#'                      n.iter = 50)
#'   randomsel_sum_margalef
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(randomsel_sum_margalef,
#'                                 use.names = FALSE)) +
#'     labs(title = "Random search",
#'          subtitle = "Pooled Margalef's diversity index")
#'
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'   # Greedy search with 1-opt best improvement
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   # Mean richness
#'   greedysel_best_mean_richness <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "richness",
#'                      metric = "mean", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_mean_richness
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_mean_richness,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Mean richness")
#'
#'   # Pooled richness
#'   greedysel_best_sum_richness <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "richness",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_sum_richness
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_sum_richness,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Pooled richness")
#'
#'   # Mean Shannon-Weaver diversity index
#'   greedysel_best_mean_shannon <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "shannon",
#'                      metric = "mean", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_mean_shannon
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_mean_shannon,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Mean Shannon-Weaver diversity index")
#'
#'   # Pooled Shannon-Weaver diversity index
#'   greedysel_best_sum_shannon <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "shannon",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_sum_shannon
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_sum_shannon,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Pooled Shannon-Weaver diversity index")
#'
#'   # Mean Gini-Simpson diversity index
#'   greedysel_best_mean_simpson <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "simpson",
#'                      metric = "mean", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_mean_simpson
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_mean_simpson,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Mean Gini-Simpson diversity index")
#'
#'   # Pooled Gini-Simpson diversity index
#'   greedysel_best_sum_simpson <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "simpson",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_sum_simpson
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_sum_simpson,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Pooled Gini-Simpson diversity index")
#'
#'   # Mean McIntosh diversity index
#'   greedysel_best_mean_mcintosh <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "mcintosh",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_mean_mcintosh
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_mean_mcintosh,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Mean McIntosh diversity index")
#'
#'   # Pooled McIntosh diversity index
#'   greedysel_best_sum_mcintosh <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "mcintosh",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_sum_mcintosh
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_sum_mcintosh,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Pooled McIntosh diversity index")
#'
#'   # Mean Brillouin diversity index
#'   greedysel_best_mean_brillouin <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns,
#'                      div.fun = div_fun_brillouin,
#'                      metric = "mean", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_mean_brillouin
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_mean_brillouin,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Mean Brillouin diversity index")
#'
#'   # Pooled Brillouin diversity index
#'   greedysel_best_sum_brillouin <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns,
#'                      div.fun = div_fun_brillouin,
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_sum_brillouin
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_sum_brillouin,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Pooled Brillouin diversity index")
#'
#'   # Mean Margalef's richness index
#'   greedysel_best_mean_margalef <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns,
#'                      div.fun = div_fun_margalef,
#'                      metric = "mean", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_mean_margalef
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_mean_margalef,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Mean Margalef's diversity index")
#'
#'   # Pooled Margalef's richness index
#'   greedysel_best_sum_margalef <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns,
#'                      div.fun = div_fun_margalef,
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "best.improvement",max.iter = 3)
#'   greedysel_best_sum_margalef
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_best_sum_margalef,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt best improvement",
#'          subtitle = "Pooled Margalef's diversity index")
#'
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'   # Greedy search with 1-opt first improvement
#'   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#'   # Mean richness
#'   greedysel_first_mean_richness <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "richness",
#'                      metric = "mean", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_mean_richness
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_mean_richness,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Mean richness")
#'
#'   # Pooled richness
#'   greedysel_first_sum_richness <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "richness",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_sum_richness
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_sum_richness,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Pooled richness")
#'
#'   # Mean Shannon-Weaver diversity index
#'   greedysel_first_mean_shannon <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "shannon",
#'                      metric = "mean", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_mean_shannon
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_mean_shannon,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Mean Shannon-Weaver diversity index")
#'
#'   # Pooled Shannon-Weaver diversity index
#'   greedysel_first_sum_shannon <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "shannon",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_sum_shannon
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_sum_shannon,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Pooled Shannon-Weaver diversity index")
#'
#'   # Mean Gini-Simpson diversity index
#'   greedysel_first_mean_simpson <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "simpson",
#'                      metric = "mean", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_mean_simpson
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_mean_simpson,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Mean Gini-Simpson diversity index")
#'
#'   # Pooled Gini-Simpson diversity index
#'   greedysel_first_sum_simpson <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "simpson",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_sum_simpson
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_sum_simpson,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Pooled Gini-Simpson diversity index")
#'
#'   # Mean McIntosh diversity index
#'   greedysel_first_mean_mcintosh <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "mcintosh",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_mean_mcintosh
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_mean_mcintosh,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Mean McIntosh diversity index")
#'
#'   # Pooled McIntosh diversity index
#'   greedysel_first_sum_mcintosh <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns, div.index = "mcintosh",
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_sum_mcintosh
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_sum_mcintosh,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Pooled McIntosh diversity index")
#'
#'   # Mean Brillouin diversity index
#'   greedysel_first_mean_brillouin <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns,
#'                      div.fun = div_fun_brillouin,
#'                      metric = "mean", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_mean_brillouin
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_mean_brillouin,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Mean Brillouin diversity index")
#'
#'   # Pooled Brillouin diversity index
#'   greedysel_first_sum_brillouin <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns,
#'                      div.fun = div_fun_brillouin,
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_sum_brillouin
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_sum_brillouin,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Pooled Brillouin diversity index")
#'
#'   # Mean Margalef's richness index
#'   greedysel_first_mean_margalef <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns,
#'                      div.fun = div_fun_margalef,
#'                      metric = "mean", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_mean_margalef
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_mean_margalef,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Mean Margalef's richness index")
#'
#'   # Pooled Margalef's richness index
#'   greedysel_first_sum_margalef <-
#'     select.diversity(data = data, names = "genotypes", group = "Cluster",
#'                      alloc = counts, qualitative = traits,
#'                      always.selected = mand_accns,
#'                      div.fun = div_fun_margalef,
#'                      metric = "pooled", search = "greedy",
#'                      local.search = "first.improvement",max.iter = 3)
#'   greedysel_first_sum_margalef
#'
#'   plot_dist(d = dist_matrix, method = "isomds",
#'             gp = gp_vec,
#'             highlight =  unlist(greedysel_first_sum_margalef,
#'                                 use.names = FALSE)) +
#'     labs(title = "Greed search | 1-opt first improvement",
#'          subtitle = "Pooled Margalef's richness index")
#' }
#'
select.diversity <- function(data, names, group, alloc,
                             qualitative,
                             always.selected = NULL,
                             div.index = c("richness", "shannon",
                                           "simpson", "mcintosh"),
                             shannon.base = exp(1),
                             div.fun = NULL,
                             metric = c("mean", "pooled"),
                             search = c("random", "greedy"),
                             local.search = c("best.improvement",
                                              "first.improvement"),
                             n.iter = 1000,
                             max.iter = 30) {

  div.index <- match.arg(div.index)
  metric <- match.arg(metric)
  search <- match.arg(search)
  if (search == "greedy") {
    local.search <- match.arg(local.search)
  }

  if (search == "random" && !is.null(local.search)) {
    warning('"local.search" is ignored when search = "random"',
            call. = FALSE)
  }

  checks.sample.core(data = data, size = NULL,
                     names = names, group = group,
                     dist.mat = NULL,
                     qualitative = qualitative,
                     log.base = NULL,
                     alloc = alloc,
                     always.selected = always.selected,
                     mode = "sel")

  SampleCore.debug <- getOption("SampleCore.debug", default = FALSE)

  # Prepare data ----

  traits <- c(qualitative)

  data[, group] <- droplevels(data[, group])

  # gp_memb <- data[, group]

  # Split data once
  grouped_data <- split(data, data[[group]])

  # Choose diversity function once ----
  div_fun_internal <-
    switch(
      div.index,
      richness = function(x) length(unique(x)),
      shannon  = function(x) DiversityStats::shannon(x, base = shannon.base),
      simpson  = DiversityStats::gini_simpson,
      mcintosh = DiversityStats::mcintosh_diversity
    )

  if (!is.null(div.fun)) {
    div_fun_internal <- div.fun
  }

  # Main group-wise selection ----

  ## Random search ----

  out <-
    lapply(names(alloc), function(g) {

      sub_df <- grouped_data[[g]]

      if (is.null(sub_df) || nrow(sub_df) == 0) {
        warning(
          sprintf("Group %s has no rows in data.", g),
          call. = FALSE
        )
        return(character(0))
      }

      group_accns <- sub_df[[names]]

      # Always selected accessions

      fixed_accns <-
        intersect(always.selected, group_accns)

      alloc_n <- alloc[[g]]

      rem_accns <-
        setdiff(group_accns, fixed_accns)

      n_rem <- alloc_n - length(fixed_accns)

      # Handle alloc smaller than fixed set
      if (n_rem < 0) {

        warning(
          sprintf(
            paste0('Group %s: "alloc" (%d) is smaller than number of ',
                   '"always.selected" values (%d). ',
                   'Taking only "always.selected" values.'),
            g, alloc_n, length(fixed_accns)
          ),
          call. = FALSE
        )
        return(fixed_accns)
      }


      # Handle insufficient remaining pool
      avail_rem <- length(rem_accns)

      if (avail_rem < n_rem) {

        warning(
          sprintf(
            paste0('Group %s has only %d additional accessions but "alloc" ',
                   'requires %d additional entries. ',
                   'Taking all available accessions.'),
            g, avail_rem, n_rem
          ),
          call. = FALSE
        )
        n_rem <- avail_rem

      }

      # If no remaining sampling needed
      if (n_rem == 0) {
        return(fixed_accns)
      }

      # sub_df -> traits_mat
      traits_mat <- do.call(cbind, lapply(traits, function(tr) {
        x <- sub_df[[tr]]
        if (is.factor(x)) as.double(as.integer(x)) else as.double(x)
      }))
      colnames(traits_mat) <- traits

      ## Random search ----
      if (search ==  "random") {

        # Candidate subsets fetched randomly
        candidate_subsets <-
          replicate(n.iter, c(fixed_accns,
                              sample(rem_accns, n_rem, replace = FALSE)),
                    simplify = FALSE)

        # Get candidate subset scores
        candidate_scores <-
          vapply(candidate_subsets,
                 function(x) {
                   idx <- match(x, group_accns)
                   compute_score(idx = idx,
                                 traits_mat = traits_mat,
                                 div_fun = div_fun_internal,
                                 metric = metric)
                 }, numeric(1))

        if (SampleCore.debug) {
          message(paste(range(candidate_scores)))
        }

        best_subset <- candidate_subsets[[which.max(candidate_scores)]]

      } else if (search == "greedy") { ## Greedy search ----

        ### Greedy initialization ----

        if (SampleCore.debug) {
          message("--Greedy initialization started.---------------------\n\n")
        }

        # Ignores max.iter

        # when fixed_accns is NULL
        if (length(fixed_accns) == 0L) {
          seed_acc <- sample(rem_accns, 1L)
          selected <- seed_acc # start from always-selected set
          pool <- setdiff(rem_accns, seed_acc) # remaining candidates
          n_to_add <- max(0L, n_rem - 1L)
        } else {
          selected <- fixed_accns
          pool <- rem_accns
          n_to_add <- n_rem
        }

        idx_lookup   <- setNames(seq_len(nrow(sub_df)), group_accns)
        selected_idx <- idx_lookup[selected]
        pool_idx     <- idx_lookup[pool]

        for (i in seq_len(n_to_add)) {
          scores <- vapply(pool_idx, function(cand_i) {
            compute_score(c(selected_idx, cand_i), traits_mat,
                          div_fun_internal, metric)
          }, numeric(1))

          best_pos     <- which.max(scores)
          selected_idx <- c(selected_idx, pool_idx[best_pos])
          pool_idx     <- pool_idx[-best_pos]   # integer remove by position — faster than setdiff
        }

        ### 1-opt local search ----

        if (SampleCore.debug) {
          message("--Local search started.------------------------------\n\n")
        }

        current_idx <- selected_idx
        fixed_idx <- idx_lookup[fixed_accns]
        rem_idx <- idx_lookup[rem_accns]
        current_score <- compute_score(current_idx, traits_mat,
                                       div_fun_internal, metric)

        # Initialize indices ONCE
        swappable_idx <- setdiff(current_idx, fixed_idx)
        candidate_idx <- setdiff(rem_idx, current_idx)

        iter_1opt <- 0L
        repeat {
          if (iter_1opt >= max.iter) break # cap check

          iter_1opt <- iter_1opt + 1L

          # Exit if no swaps are possible
          if (length(swappable_idx) == 0L || length(candidate_idx) == 0L) break

          improved  <- FALSE

          #### Best-Improvement Strategy ----
          if (local.search == "best.improvement") {
            best_overall_score <- current_score
            best_out_val <- NULL
            best_in_val  <- NULL
            best_out_pos_in_swappable <- NULL
            best_in_pos_in_candidate  <- NULL

            # Nested Loops: Scanning all possible swaps
            for (i in seq_along(swappable_idx)) {
              out_val <- swappable_idx[i]
              # Pre-calculate the subset excluding the 'out' candidate
              subset_minus_out <- current_idx[current_idx != out_val]

              for (j in seq_along(candidate_idx)) {
                in_val      <- candidate_idx[j]
                trial_score <-
                  compute_score(idx = c(subset_minus_out, in_val),
                                traits_mat = traits_mat,
                                div_fun =div_fun_internal,
                                metric = metric)

                # Track the best improvement found so far
                if (trial_score > best_overall_score) {
                  best_overall_score <- trial_score
                  best_out_val <- out_val
                  best_in_val <- in_val
                  best_out_pos_in_swappable <- i
                  best_in_pos_in_candidate  <- j
                }
              }
            }

            # Check if an improvement was actually found in this pass
            if (!is.null(best_out_val)) {
              # Update current collection
              current_idx[match(best_out_val, current_idx)] <- best_in_val
              current_score <- best_overall_score

              # Update indices in-place
              swappable_idx[best_out_pos_in_swappable] <- best_in_val
              candidate_idx[best_in_pos_in_candidate] <- best_out_val
              improved <- TRUE

              if (SampleCore.debug) {
                message(sprintf("Best-improvement | Iter %d: Swapped out %d for %d. New score: %f",
                                iter_1opt, best_out_val, best_in_val,
                                current_score))
              }
            }

          } else {

            ### First-improvement strategy ----

            # Nested loops
            for (i in seq_along(swappable_idx)) {
              out_val <- swappable_idx[i]
              subset_minus_out <- current_idx[current_idx != out_val]

              for (j in seq_along(candidate_idx)) {
                in_val <- candidate_idx[j]
                trial_score <- compute_score(c(subset_minus_out, in_val),
                                             traits_mat, div_fun_internal,
                                             metric)

                if (trial_score > current_score) {
                  # First improvement found - Apply swap immediately
                  current_idx[match(out_val, current_idx)] <- in_val
                  current_score <- trial_score

                  # swap in-place
                  swappable_idx[i] <- in_val # best_in enters swappable pool
                  candidate_idx[j] <- out_val # best_out enters candidate pool
                  improved <- TRUE

                  if (SampleCore.debug) {
                    message(sprintf("First-improvement | Iter %d: Swapped out %d for %d. New score: %f",
                                    iter_1opt, out_val, in_val, current_score))
                  }

                  break # Break inner loop
                }
              }
              if (improved) break # Break outer loop to restart 1-opt with new current_idx
            }
          }

          if (!improved) break # Local optimum reached
        }

        best_subset <- group_accns[current_idx]

      }
    })

  return(out)

}


compute_score <- function(idx, traits_mat,
                          div_fun, metric) {
  trait_div <- vapply(seq_len(ncol(traits_mat)), function(t) {
    div_fun(traits_mat[idx, t, drop = TRUE])
  }, numeric(1))
  switch(metric,
         mean   = mean(trait_div,  na.rm = TRUE),
         pooled = sum(trait_div,   na.rm = TRUE)
  )
}

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Selection of Entries from Clusters/Groups by Random Sampling
#'
#' Select entries from cluster/groups in the entire collection by random
#' sampling according to allocation specified.
#'
#' For each cluster/group entries are selected randomly according to the
#' allocation provided
#' \insertCite{brown_core_1989,van_hintum_core_2000}{SampleCore}. Entries listed
#' as \code{always.selected} are mandatorily included in the selection. Warnings
#' are issued if requested allocation is smaller than the number of
#' always-selected entries in a cluster/group and/or when the cluster/group does
#' not contain enough remaining entries to fulfill the allocation.
#'
#' @template general-arg
#' @template sel-arg
#'
#' @returns A named list where each element contains the selected entry
#'   identifiers for a cluster/group.
#' @export
#'
#' @references
#'
#' \insertAllCited
#'
#' @seealso \code{\link[SampleCore]{select.distance}},
#'   \code{\link[SampleCore]{select.diversity}}
#'
#' @examplesIf requireNamespace("cluster", quietly = TRUE)
#'
#' library(cluster)
#'
#' # Get data
#' data(cassava_EC_gp)
#'
#' set.seed(123)
#' cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]
#'
#' data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)
#' row.names(data) <- NULL
#'
#' # Prepare inputs
#' counts <- c(I = 31, II = 31, III = 18, IV = 35, V = 40, VI = 17)
#'
#' mand_accns <-
#'   c("TMe-2018", "TMe-801", "TMe-3191", "TMe-1830", "TMe-1790")
#'
#' # Specify the seed
#' set.seed(123)
#'
#' # Fetch selected accessions
#' sel_random_out <-
#'   select.random(data = data, names = "genotypes",
#'                 group = "Cluster", alloc = counts,
#'                 always.selected = mand_accns)
#'
#' sel_random_out
#'
#' # Get distance matrix - Only for visualization
#' quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
#'            "AVPW", "ARSR", "SRDM")
#' qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'           "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'           "PSTR")
#'
#' # Convert qualitative data columns to factor
#' cassava_EC_gp[, qual] <- lapply(cassava_EC_gp[, qual], as.factor)
#'
#' # Standardise quantitative data column
#' cassava_EC_gp[, quant] <- lapply(cassava_EC_gp[, quant], function(x) {
#'   scale(x)[, 1]
#' })
#'
#' gp_vec <- setNames(as.character(data[, "Cluster"]), data[, "genotypes"])
#'
#' # Get the Gower's distance matrix
#' dist_matrix <- daisy(x = cassava_EC_gp[, c(qual, quant)],
#'                      metric = "gower")
#'
#' plot_dist(d = dist_matrix, method = "isomds",
#'           gp = gp_vec,
#'           highlight =  unlist(sel_random_out, use.names = FALSE))
#'
select.random <- function(data, names, group, alloc,
                          always.selected = NULL) {

  checks.sample.core(data = data, size = NULL,
                     names = names, group = group,
                     dist.mat = NULL,
                     qualitative = NULL,
                     log.base = NULL,
                     alloc = alloc,
                     always.selected = always.selected,
                     mode = "sel")

  out <-
    lapply(names(alloc), function(g) {

      group_accns <- data[data[[group]] == g, names]

      # always-selected values belonging to this group
      fixed_accns <- intersect(always.selected, group_accns)

      requested_n <- alloc[[g]]

      # Remaining pool after fixed selections
      rem_accns <- setdiff(group_accns, fixed_accns)

      n_rem <- requested_n - length(fixed_accns)

      # Handle alloc smaller than fixed set
      if (n_rem < 0) {

        warning(
          sprintf(
            paste0('Group %s: "alloc" (%d) is smaller than number of ',
                   '"always.selected" values (%d). ',
                   'Taking only "always.selected" values.'),
            g, requested_n, length(fixed_accns)
          ),
          call. = FALSE
        )

        sampled_accns <- fixed_accns
      } else {

        avail_rem <- length(rem_accns)

        # Handle insufficient remaining pool
        if (avail_rem < n_rem) {

          warning(
            sprintf(
              paste0('Group %s has only %d additional accessions but "alloc" ',
                     'requires %d additional entries. ',
                     'Taking all available accessions.'),
              g, avail_rem, n_rem
            ),
            call. = FALSE
          )

          n_rem <- avail_rem
        }

        if (n_rem == 0) {

          sampled_accns <- fixed_accns

        } else {

        sampled_accns <- c(fixed_accns,
                           sample(rem_accns, n_rem, replace = FALSE))

        }

      }

      sampled_accns

    })

  names(out) <- names(alloc)

  return(out)

}

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Allocation of Entries to be Selected from Clusters/Groups based on
#' Distance-based Diversity Metrics for Core Collection Development
#'
#' Estimate the number of entries to be allocated from each cluster/group in
#' the entire collection to construct a core collection on the basis of
#' different metrics computed from within cluster/group distances. The following
#' strategies are implemented. \loadmathjax
#' \itemize{
#' \item{Diversity (Distance based)}
#' \item{Diversity (Distance based) & Proportional}
#' \item{Diversity (Distance based) & Logarithmic}
#' \item{Diversity (Distance based) & Square root}}
#'
#' @section Details:
#'
#'   The number of entries to be chosen from each cluster is estimated either on
#'   the basis of diversity of entries within that cluster/group alone or in
#'   combination with the size of the cluster/group (See \strong{Methods}).
#'
#'   The within-cluster/group diversity is estimated as several metrics from the
#'   within cluster/group genetic distances between accessions (See
#'   \strong{Metrics}).
#'
#'   \insertCite{franco_sampling_2005;textual}{SampleCore} proposed a method
#'   based on mean Gower's distance \insertCite{gower_general_1971}{SampleCore}
#'   which was also extended to other distance measure averages named D
#'   Allocation strategy \insertCite{franco_sampling_2006}{SampleCore}. These
#'   methods were also combined with the proportional and logarithmic methods.
#'   For example, the GP and GL strategy of
#'   \insertCite{bisht_assessment_1999;textual}{SampleCore} and
#'   \insertCite{mahajan_sampling_1999;textual}{SampleCore} as well as the NY
#'   and LD allocation methods of
#'   \insertCite{franco_sampling_2005;textual}{SampleCore}.
#'
#' @template divmethods-section
#'
#' @section Metrics:
#'
#'   \subsection{Summary/Decriptive statistics}{These include mean, median,
#'   maximum and range of genetic distances between entries in a cluster.}
#'
#'   \subsection{Mean nearest-neighbour distance (\mjseqn{MNND})}{It is the
#'   average, across all entries, of the distance to each entry’s
#'   closest other entry (\mjseqn{d_{g_{min}}}), based on a genetic given
#'   distance matrix \insertCite{clark_distance_1954}{SampleCore}.
#'
#'   For each entry, the nearest-neighbour distance (\mjseqn{d_{g_{min}}})
#'   is the smallest non-zero distance with any other entry.
#'
#'   \mjsdeqn{d_{g_{min}} = \min_{h \ne g} d_{gh}}
#'
#'   The Mean nearest-neighbour distance (\mjseqn{MNND}) can then be computed
#'   as:
#'
#'   \mjsdeqn{\textrm{MNND} = \frac{1}{G} \sum_{g=1}^{G} d_g}
#'
#'   Where, (\mjseqn{g}) is the index of an entry in a genetic distance
#'   matrix, \mjseqn{h} is the index of all other genotypes and \mjseqn{G} is
#'   the total number of genotypes in a cluster/group.
#'
#'   }
#'
#'   \subsection{Minimum spanning tree length (\mjseqn{MSTL})}{It is defined as
#'   the sum of edge weights in the minimum spanning tree constructed from the
#'   genetic distance matrix of entries within a cluster/group.  A minimum
#'   spanning tree (MST) connects all entries such that the total distance
#'   is minimized and no cycles are formed. It represents the most efficient
#'   way to connect all entries based on pairwise genetic distances
#'   \insertCite{gower_minimum_1969}{SampleCore}.
#'
#'   For genetic distance \mjseqn{d_{gh}} between entries \mjseqn{g} and
#'   \mjseqn{h}, the MST is a subset of edges that connects all
#'   \mjseqn{G} entries with exactly \mjseqn{G - 1} edges and minimum total
#'   weight. The MST length (\mjseqn{MSTL}) can then be computed as:
#'
#'   \mjsdeqn{\textrm{MSTL} = \sum_{(g,h) \in \mathcal{T}} d_{gh}}
#'
#'   Where \mjseqn{\mathcal{T}} denotes the set of edges in the MST.
#'
#'   }
#'
#'   \subsection{Mean distance to centroid and median (\mjseqn{MDC},
#'   \mjseqn{MDM})}{These quantify the average dispersion of entries within a
#'   cluster/group relative to a central point in multivariate space derived
#'   from the genetic distance matrix.
#'
#'   The centroid represents the multivariate mean position of all entries
#'   in a cluster
#'   \insertCite{sokal_principles_1963,sneath_numerical_1973}{SampleCore}.,
#'   whereas the median (spatial median) provides a robust central location
#'   that is less influenced by extreme values
#'   \insertCite{bradley_constrained_1999}{SampleCore}.
#'
#'   For \mjseqn{d_{gC}} and \mjseqn{d_{gM}} distances of entry
#'   \mjseqn{g} from the centroid \mjseqn{C} and median \mjseqn{M},
#'   respectively. These measures are computed as:
#'
#'   \mjsdeqn{\textrm{MDC} = \frac{1}{G} \sum_{g=1}^{G} d_{gC}}
#'
#'   \mjsdeqn{\textrm{MDM} = \frac{1}{G} \sum_{g=1}^{G} d_{gM}}
#'
#'   Where \mjseqn{G} is the total number of entries in the cluster/group.
#'
#'   }
#'
#'   \subsection{Number of clusters}{\insertCite{diwan_core_1994}{SampleCore}
#'   proposed the number of clusters produced by a multivariate cluster
#'   analysis at a specific distance threshold as an estimate of the diversity.
#'
#'   }
#'
#' @template general-arg
#' @template log-arg
#' @template size-arg
#' @template dist-arg
#' @param method The allocation method. Either \code{"dist"} for constant or
#'   \code{"dist.prop"} for proportional or \code{"dist.log"} for logarithmic or
#'   \code{"dist.sqrt"} for square root allocation. See \strong{Methods}.
#' @param metric The metric to be computed from the distance matrix. Either
#'   \code{"mean"}, \code{"median"}, \code{"max"}, \code{"range"},
#'   \code{"mnnd"}, \code{"mdc"}, \code{"mdm"}, \code{"mstl"}, or
#'   \code{"nclust"}. See \strong{Metrics}.
#' @param clust.fun A function to generate clusters from a distance matrix and
#'   return the number of clusters.
#'
#' @template alloc-returns
#'
#' @importFrom vegan betadisper
#' @importFrom igraph E graph_from_adjacency_matrix mst
#' @importFrom stats as.dist setNames median
#' @export
#'
#' @references
#'
#' \insertAllCited
#'
#' @seealso \code{\link[SampleCore]{allocate.basic}},
#'   \code{\link[SampleCore]{allocate.diversity}}
#'
#' @examplesIf requireNamespace("cluster", quietly = TRUE) & requireNamespace("fastcluster", quietly = TRUE) & requireNamespace("dbscan", quietly = TRUE) & requireNamespace("biotools", quietly = TRUE)
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' library(cluster)
#'
#' # Get distance matrix
#' data("cassava_EC_gp")
#'
#' set.seed(123)
#' cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]
#'
#' quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
#'            "AVPW", "ARSR", "SRDM")
#' qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'           "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'           "PSTR")
#'
#' data <- cassava_EC_gp
#'
#' # Convert qualitative data columns to factor
#' data[, qual] <- lapply(data[, qual], as.factor)
#'
#' # Standardise quantitative data column
#' data[, quant] <- lapply(data[, quant], function(x) {
#'   scale(x)[, 1]
#' })
#'
#' # Get the Gower's distance matrix
#' dist_matrix <- daisy(x = data[, c(qual, quant)],
#'                      metric = "gower")
#'
#' # Get data
#' data <- cassava_EC_gp
#' data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)
#' row.names(data) <- NULL
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Custom clustering functions
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' # UPGMA with hclust
#' clust_fun_upgma <- function(x) {
#'   # Tree
#'   tree_out <- hclust(x, method = "average")
#'   # Clusters
#'   cutree(tree_out, h = 0.2)
#' }
#'
#' if (requireNamespace('fastcluster', quietly = TRUE)) {
#'   # Ward's minimum variance with fastcluster
#'   clust_fun_ward <- function(x) {
#'     # Tree
#'     tree_out <- fastcluster::hclust(x, method = "ward.D2")
#'     # Clusters
#'     cutree(tree_out, h = 0.2)
#'   }
#' }
#'
#' if (requireNamespace('dbscan', quietly = TRUE)) {
#'   # Density-based clustering with dbscan
#'   clust_fun_dbscan <- function(x) {
#'     clust_out <- dbscan::dbscan(x, eps = 0.25)
#'     # remove noise: TODO
#'     setNames(clust_out$cluster, labels(x))
#'   }
#' }
#'
#' if (requireNamespace('biotools', quietly = TRUE)) {
#'   # Tocher's sequential clustering
#'   clust_fun_tocher <- function(x) {
#'     clust_out <- biotools::tocher(x, algorithm = "sequential")
#'     setNames(clust_out$class, labels(x))
#'   }
#' }
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Diversity (Distance based) allocation
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' ## Mean
#' dist_out_mean <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist",
#'                     dist.mat = dist_matrix, metric = "mean",
#'                     size = 0.2)
#' dist_out_mean
#'
#' ## Median
#' dist_out_median <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist",
#'                     dist.mat = dist_matrix, metric = "median",
#'                     size = 0.2)
#' dist_out_median
#'
#' ## Maximum
#' dist_out_max <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist",
#'                     dist.mat = dist_matrix, metric = "max",
#'                     size = 0.2)
#' dist_out_max
#'
#' ## Range
#' dist_out_range <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist",
#'                     dist.mat = dist_matrix, metric = "range",
#'                     size = 0.2)
#' dist_out_range
#'
#' ## Mean nearest-neighbour distance
#' dist_out_mnnd <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist",
#'                     dist.mat = dist_matrix, metric = "mnnd",
#'                     size = 0.2)
#' dist_out_mnnd
#'
#' ## Minimum spanning tree length
#' dist_out_mstl <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist",
#'                     dist.mat = dist_matrix, metric = "mstl",
#'                     size = 0.2)
#' dist_out_mstl
#'
#' \donttest{
#'   ## Mean distance to centroid
#'   dist_out_mdc <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist",
#'                       dist.mat = dist_matrix, metric = "mdc",
#'                       size = 0.2)
#'   dist_out_mdc
#'
#'   ## Mean distance to median
#'   dist_out_mdm <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist",
#'                       dist.mat = dist_matrix, metric = "mdm",
#'                       size = 0.2)
#'   dist_out_mdm
#' }
#'
#' ## Number of clusters
#'
#' ### UPGMA with hclust
#' dist_out_nclust1 <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist",
#'                     dist.mat = dist_matrix, metric = "nclust",
#'                     clust.fun = clust_fun_upgma,
#'                     size = 0.2)
#' dist_out_nclust1
#'
#' # Ward's minimum variance with fastcluster
#' if (requireNamespace('fastcluster', quietly = TRUE)) {
#'   dist_out_nclust2 <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist",
#'                       dist.mat = dist_matrix, metric = "nclust",
#'                       clust.fun = clust_fun_ward,
#'                       size = 0.2)
#'   dist_out_nclust2
#' }
#'
#'
#' # Density-based clustering with dbscan
#' if (requireNamespace('dbscan', quietly = TRUE)) {
#'   dist_out_nclust3 <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist",
#'                       dist.mat = dist_matrix, metric = "nclust",
#'                       clust.fun = clust_fun_dbscan,
#'                       size = 0.2)
#'   dist_out_nclust3
#' }
#'
#' \donttest{
#'   if (requireNamespace('biotools', quietly = TRUE)) {
#'     # Tocher's sequential clustering
#'     dist_out_nclust4 <-
#'       allocate.distance(data = data, names = "genotypes",
#'                         group = "Cluster", method = "dist",
#'                         dist.mat = dist_matrix, metric = "nclust",
#'                         clust.fun = clust_fun_tocher,
#'                         size = 0.2)
#'     dist_out_nclust4
#'   }
#' }
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Diversity (Distance based) & Proportional
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' ## Mean
#' dist_prop_out_mean <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.prop",
#'                     dist.mat = dist_matrix, metric = "mean",
#'                     size = 0.2)
#' dist_prop_out_mean
#'
#' ## Median
#' dist_prop_out_median <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.prop",
#'                     dist.mat = dist_matrix, metric = "median",
#'                     size = 0.2)
#' dist_prop_out_median
#'
#' ## Maximum
#' dist_prop_out_max <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.prop",
#'                     dist.mat = dist_matrix, metric = "max",
#'                     size = 0.2)
#' dist_prop_out_max
#'
#' ## Range
#' dist_prop_out_range <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.prop",
#'                     dist.mat = dist_matrix, metric = "range",
#'                     size = 0.2)
#' dist_prop_out_range
#'
#' ## Mean nearest-neighbour distance
#' dist_prop_out_mnnd <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.prop",
#'                     dist.mat = dist_matrix, metric = "mnnd",
#'                     size = 0.2)
#' dist_prop_out_mnnd
#'
#' ## Minimum spanning tree length
#' dist_prop_out_mstl <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.prop",
#'                     dist.mat = dist_matrix, metric = "mstl",
#'                     size = 0.2)
#' dist_prop_out_mstl
#'
#' \donttest{
#'   ## Mean distance to centroid
#'   dist_prop_out_mdc <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.prop",
#'                       dist.mat = dist_matrix, metric = "mdc",
#'                       size = 0.2)
#'   dist_prop_out_mdc
#'
#'   ## Mean distance to median
#'   dist_prop_out_mdm <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.prop",
#'                       dist.mat = dist_matrix, metric = "mdm",
#'                       size = 0.2)
#'   dist_prop_out_mdm
#' }
#'
#' ## Number of clusters
#'
#' ### UPGMA with hclust
#' dist_prop_out_nclust1 <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.prop",
#'                     dist.mat = dist_matrix, metric = "nclust",
#'                     clust.fun = clust_fun_upgma,
#'                     size = 0.2)
#' dist_prop_out_nclust1
#'
#' # Ward's minimum variance with fastcluster
#' if (requireNamespace('fastcluster', quietly = TRUE)) {
#'   dist_prop_out_nclust2 <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.prop",
#'                       dist.mat = dist_matrix, metric = "nclust",
#'                       clust.fun = clust_fun_ward,
#'                       size = 0.2)
#'   dist_prop_out_nclust2
#' }
#'
#' # Density-based clustering with dbscan
#' if (requireNamespace('dbscan', quietly = TRUE)) {
#'   dist_prop_out_nclust3 <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.prop",
#'                       dist.mat = dist_matrix, metric = "nclust",
#'                       clust.fun = clust_fun_dbscan,
#'                       size = 0.2)
#'   dist_prop_out_nclust3
#' }
#'
#' \donttest{
#'   if (requireNamespace('biotools', quietly = TRUE)) {
#'     # Tocher's sequential clustering
#'     dist_prop_out_nclust4 <-
#'       allocate.distance(data = data, names = "genotypes",
#'                         group = "Cluster", method = "dist.prop",
#'                         dist.mat = dist_matrix, metric = "nclust",
#'                         clust.fun = clust_fun_tocher,
#'                         size = 0.2)
#'     dist_prop_out_nclust4
#'   }
#' }
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Diversity (Distance based) & Logarithmic
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' ## Mean
#' dist_log_out_mean <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.log",
#'                     dist.mat = dist_matrix, metric = "mean",
#'                     size = 0.2)
#' dist_log_out_mean
#'
#' ## Median
#' dist_log_out_median <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.log",
#'                     dist.mat = dist_matrix, metric = "median",
#'                     size = 0.2)
#' dist_log_out_median
#'
#' ## Maximum
#' dist_log_out_max <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.log",
#'                     dist.mat = dist_matrix, metric = "max",
#'                     size = 0.2)
#' dist_log_out_max
#'
#' ## Range
#' dist_log_out_range <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.log",
#'                     dist.mat = dist_matrix, metric = "range",
#'                     size = 0.2)
#' dist_log_out_range
#'
#' ## Mean nearest-neighbour distance
#' dist_log_out_mnnd <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.log",
#'                     dist.mat = dist_matrix, metric = "mnnd",
#'                     size = 0.2)
#' dist_log_out_mnnd
#'
#' ## Minimum spanning tree length
#' dist_log_out_mstl <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.log",
#'                     dist.mat = dist_matrix, metric = "mstl",
#'                     size = 0.2)
#' dist_log_out_mstl
#'
#' \donttest{
#'   ## Mean distance to centroid
#'   dist_log_out_mdc <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.log",
#'                       dist.mat = dist_matrix, metric = "mdc",
#'                       size = 0.2)
#'   dist_log_out_mdc
#'
#'   ## Mean distance to median
#'   dist_log_out_mdm <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.log",
#'                       dist.mat = dist_matrix, metric = "mdm",
#'                       size = 0.2)
#'   dist_log_out_mdm
#' }
#'
#' ## Number of clusters
#'
#' ### UPGMA with hclust
#' dist_log_out_nclust1 <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.log",
#'                     dist.mat = dist_matrix, metric = "nclust",
#'                     clust.fun = clust_fun_upgma,
#'                     size = 0.2)
#' dist_log_out_nclust1
#'
#' # Ward's minimum variance with fastcluster
#' if (requireNamespace('fastcluster', quietly = TRUE)) {
#'   dist_log_out_nclust2 <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.log",
#'                       dist.mat = dist_matrix, metric = "nclust",
#'                       clust.fun = clust_fun_ward,
#'                       size = 0.2)
#'   dist_log_out_nclust2
#' }
#'
#' if (requireNamespace('dbscan', quietly = TRUE)) {
#'   # Density-based clustering with dbscan
#'   dist_log_out_nclust3 <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.log",
#'                       dist.mat = dist_matrix, metric = "nclust",
#'                       clust.fun = clust_fun_dbscan,
#'                       size = 0.2)
#'   dist_log_out_nclust3
#' }
#'
#' \donttest{
#'   if (requireNamespace('biotools', quietly = TRUE)) {
#'     # Tocher's sequential clustering
#'     dist_log_out_nclust4 <-
#'       allocate.distance(data = data, names = "genotypes",
#'                         group = "Cluster", method = "dist.log",
#'                         dist.mat = dist_matrix, metric = "nclust",
#'                         clust.fun = clust_fun_tocher,
#'                         size = 0.2)
#'     dist_log_out_nclust4
#'   }
#' }
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Diversity (Distance based) & Square root
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' ## Mean
#' dist_sqrt_out_mean <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.sqrt",
#'                     dist.mat = dist_matrix, metric = "mean",
#'                     size = 0.2)
#' dist_sqrt_out_mean
#'
#' ## Median
#' dist_sqrt_out_median <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.sqrt",
#'                     dist.mat = dist_matrix, metric = "median",
#'                     size = 0.2)
#' dist_sqrt_out_median
#'
#' ## Maximum
#' dist_sqrt_out_max <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.sqrt",
#'                     dist.mat = dist_matrix, metric = "max",
#'                     size = 0.2)
#' dist_sqrt_out_max
#'
#' ## Range
#' dist_sqrt_out_range <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.sqrt",
#'                     dist.mat = dist_matrix, metric = "range",
#'                     size = 0.2)
#' dist_sqrt_out_range
#'
#' ## Mean nearest-neighbour distance
#' dist_sqrt_out_mnnd <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.sqrt",
#'                     dist.mat = dist_matrix, metric = "mnnd",
#'                     size = 0.2)
#' dist_sqrt_out_mnnd
#'
#' ## Minimum spanning tree length
#' dist_sqrt_out_mstl <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.sqrt",
#'                     dist.mat = dist_matrix, metric = "mstl",
#'                     size = 0.2)
#' dist_sqrt_out_mstl
#'
#' \donttest{
#'   ## Mean distance to centroid
#'   dist_sqrt_out_mdc <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.sqrt",
#'                       dist.mat = dist_matrix, metric = "mdc",
#'                       size = 0.2)
#'   dist_sqrt_out_mdc
#'
#'   ## Mean distance to median
#'   dist_sqrt_out_mdm <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.sqrt",
#'                       dist.mat = dist_matrix, metric = "mdm",
#'                       size = 0.2)
#'   dist_sqrt_out_mdm
#' }
#'
#' ## Number of clusters
#'
#' ### UPGMA with hclust
#' dist_sqrt_out_nclust1 <-
#'   allocate.distance(data = data, names = "genotypes",
#'                     group = "Cluster", method = "dist.sqrt",
#'                     dist.mat = dist_matrix, metric = "nclust",
#'                     clust.fun = clust_fun_upgma,
#'                     size = 0.2)
#' dist_sqrt_out_nclust1
#'
#' # Ward's minimum variance with fastcluster
#' if (requireNamespace('fastcluster', quietly = TRUE)) {
#'   dist_sqrt_out_nclust2 <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.sqrt",
#'                       dist.mat = dist_matrix, metric = "nclust",
#'                       clust.fun = clust_fun_ward,
#'                       size = 0.2)
#'   dist_sqrt_out_nclust2
#' }
#'
#' if (requireNamespace('dbscan', quietly = TRUE)) {
#'   # Density-based clustering with dbscan
#'   dist_sqrt_out_nclust3 <-
#'     allocate.distance(data = data, names = "genotypes",
#'                       group = "Cluster", method = "dist.sqrt",
#'                       dist.mat = dist_matrix, metric = "nclust",
#'                       clust.fun = clust_fun_dbscan,
#'                       size = 0.2)
#'   dist_sqrt_out_nclust3
#' }
#'
#' \donttest{
#'   if (requireNamespace('biotools', quietly = TRUE)) {
#'     # Tocher's sequential clustering
#'     dist_sqrt_out_nclust4 <-
#'       allocate.distance(data = data, names = "genotypes",
#'                         group = "Cluster", method = "dist.sqrt",
#'                         dist.mat = dist_matrix, metric = "nclust",
#'                         clust.fun = clust_fun_tocher,
#'                         size = 0.2)
#'     dist_sqrt_out_nclust4
#'   }
#' }
#'
allocate.distance <- function(data, names, group,
                              dist.mat,
                              method = c("dist", "dist.prop",
                                         "dist.log", "dist.sqrt"),
                              metric = c("mean", "median", "max",
                                         "range", "mnnd", "mdc",
                                         "mdm", "mstl", "nclust"),
                              clust.fun = NULL,
                              log.base = exp(1),
                              size) {

  # Checks ----

  checks.sample.core(data = data, size = size,
                     names = names, group = group,
                     dist.mat = dist.mat,
                     qualitative = NULL,
                     log.base = log.base,
                     mode = "alloc")

  method <- match.arg(method)
  metric <- match.arg(metric)

  if (!is.null(clust.fun)) {
    if (!is.function(clust.fun)) {
      stop('"clust.fun" must be a function.')
    }

    clust_res <- clust.fun(dist.mat)
    if (!is.vector(clust_res) && !is.factor(clust_res)) {
      stop("clust.fun must return a vector or factor of cluster assignments.")
    }

    if (length(clust_res) != nrow(as.matrix(dist.mat))) {
      stop("clust.fun must return one cluster label per observation in x.")
    }
  }

  # Prepare data ----

  data[, group] <- droplevels(data[, group])

  dmat <- as.matrix(dist.mat)

  gp_memb <- data[, group]
  dist_labels <- labels(dist.mat)

  # mean_dist <- mean(dist.mat[upper.tri(dist.mat)], na.rm = TRUE)

  # Align labels
  names(gp_memb) <- dist_labels
  gp_memb <- gp_memb[rownames(dmat)]

  # Basic group stats ----

  gps <- levels(data[, group])
  gpsize <- summary(data[, group])
  gpcount <- length(gps)

  tcount <- nrow(data)

  # Compute group-wise distance metrics ----

  if (metric %in% c("mean", "median", "max",
                    "range", "mnnd", "mstl", "nclust") ) {

    group_dist_metric <- sapply(unique(gp_memb), function(g) {
      idx <- which(gp_memb == g)

      # extract submatrix for the group
      sub_d <- dmat[idx, idx]

      ## Mean/Average distance ----
      if (metric == "mean") {
        # take only upper triangle
        # avoid duplicates + diagonal
        out <- mean(sub_d[upper.tri(sub_d)], na.rm = TRUE)
      }

      ## Median distance ----
      if (metric == "median") {
        out <- median(sub_d[upper.tri(sub_d)])
      }

      ## Maximum distance ----
      if (metric == "max") {
        out <- max(sub_d[upper.tri(sub_d)])
      }

      ## Range ----
      if (metric == "range") {
        out <- diff(range(sub_d[upper.tri(sub_d)]))
      }

      ## Mean nearest-neighbour distance (MNND) ----
      # Captures clustering vs dispersion
      if (metric == "mnnd") {
        vals <- sub_d
        diag(vals) <- Inf # To exclude self-distances
        out <- mean(apply(vals, 1, min))
      }

      ## Minimum spanning tree (MSTL) length ----
      if (metric == "mstl") {
        # mst_mat <- ape::mst(sub_d)
        # mat <- as.matrix(sub_d)
        # sum(mst_mat * mat) / 2

        g <- igraph::graph_from_adjacency_matrix(
          as.matrix(sub_d),
          mode = "undirected",
          weighted = TRUE,
          diag = FALSE
        )

        mst_g <- igraph::mst(g)
        out <- sum(igraph::E(mst_g)$weight)
      }

      ## Diwan et al., 1994 (No. of clusters )----
      if (metric == "nclust") {
        # Handle groups with only 1 entry (cannot cluster)
        if (length(idx) == 1) {
          return(1)
        }

        # Return cluster membership
        clusters <- clust.fun(as.dist(sub_d))

        # count of unique clusters
        out <- length(unique(clusters))
      }

      return(out)

    })
  }

  ## Mean distance to centroid ----
  if (metric == "mdc") {
    # only for euclidean dist
    # coords <- cmdscale(sub_d, k = nrow(sub_d) - 1)
    # centroid <- colMeans(coords)
    # mean(sqrt(rowSums((coords - centroid) ^ 2)))

    bdout <- vegan::betadisper(d = dist.mat, group = data[, group],
                               type = "centroid")
    group_dist_metric <- bdout$group.distances
  }

  ## Mean distance to median ----
  if (metric == "mdm") {
    bdout <- vegan::betadisper(d = dist.mat, group = data[, group],
                               type = "median")
    group_dist_metric <- bdout$group.distances
  }

  # Freq estimation ----

  ## "dist" ----
  if (method == "dist") {
    freq <- group_dist_metric / sum(group_dist_metric)
  }

  ## "dist.prop" ----
  if (method == "dist.prop") {
    freq <- (group_dist_metric * gpsize) /
      sum(group_dist_metric * gpsize)
  }

  ## "dist.log" ----
  if (method == "dist.log") {
    log_gpsize <- log(gpsize, base = log.base)
    freq <- (group_dist_metric * log_gpsize) /
      sum(group_dist_metric * log_gpsize)
  }

  ## "dist.sqrt" ----
  if (method == "dist.sqrt") {
    freq <- (group_dist_metric * sqrt(gpsize)) /
      sum(group_dist_metric * sqrt(gpsize))
  }

  # Get output ---
  out <- setNames(round(freq * size * tcount), names(gpsize))

  return(out)

}

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Plot a distance matrix as a 2D projection
#'
#' Reduces a distance matrix to two dimensions using Classical MDS, Isotonic
#' MDS, or t-SNE, and returns a \code{ggplot2} scatter plot in which proximity
#' reflects similarity. Points can optionally be highlighted or split into facet
#' panels by group.
#'
#' @param d A distance matrix of class \code{\link[stats]{dist}}. Labels must be
#'   set (i.e. \code{labels(d)} must not be \code{NULL}). Duplicate labels are
#'   not permitted.
#' @param method Character string specifying the dimensionality-reduction
#'   method. One of:
#'   \describe{
#'     \item{\code{"cmds"}}{Classical (metric) Multidimensional Scaling via
#'       \code{\link[stats]{cmdscale}}. This is the default.}
#'     \item{\code{"isomds"}}{Non-metric (isotonic) MDS via
#'       \code{\link[MASS]{isoMDS}}. Automatically falls back to
#'       \code{"cmds"} with a message when \code{n < 3}.}
#'     \item{\code{"tsne"}}{t-distributed Stochastic Neighbour Embedding via
#'       \code{\link[Rtsne]{Rtsne}}. Perplexity is set automatically to
#'       \code{min(30, floor((n - 1) / 3))}.}
#'   }
#' @param highlight Optional character vector of labels to highlight in the
#'   plot. Matching identifiers are plotted in \strong{red}; all others in
#'   black. \code{NULL} (default) disables highlighting. Every value must be
#'   present in \code{labels(d)}.
#' @param gp Optional named character vector mapping labels to group names
#'   (\code{names(gp)} = labels, values = group names). When supplied, the plot
#'   is split into one facet panel per group via
#'   \code{\link[ggplot2]{facet_wrap}}. The set of names must match
#'   \code{labels(d)} exactly. \code{NULL} (default) produces a single panel.
#' @param point.alpha Alpha transparency value for points.
#'
#' @return A \code{\link[ggplot2]{ggplot}} object. The plot can be further
#'   customised with standard \pkg{ggplot2} additions before printing or saving.
#'
#' @seealso \code{\link[stats]{cmdscale}}, \code{\link[MASS]{isoMDS}},
#' \code{\link[Rtsne]{Rtsne}}, \code{\link[ggplot2]{ggplot}}
#'
#' @importFrom ggplot2 ggplot aes geom_point scale_color_manual labs theme_bw theme element_rect facet_wrap
#' @importFrom MASS isoMDS
#' @importFrom Rtsne Rtsne
#' @export
#'
#' @examples
#' # Basic usage with the built-in eurodist dataset
#' plot_dist(eurodist)
#'
#' # Non-metric MDS with two highlighted cities
#' plot_dist(eurodist, method = "isomds",
#'           highlight = c("Madrid", "Rome"))
#'
#' # Classical MDS split by a user-defined grouping
#' regions <-
#'   c(Athens = "South",  Barcelona = "South", Brussels = "North",
#'     Calais = "North",  Cherbourg = "North", Cologne = "North",
#'     Copenhagen = "North", Geneva = "South", Gibraltar = "South",
#'     Hamburg = "North", `Hook of Holland` = "North", Lisbon = "South",
#'     Lyons = "South",  Madrid = "South",  Marseilles = "South",
#'     Milan = "South",  Munich = "North",  Paris = "North",
#'     Rome = "South",   Stockholm = "North", Vienna = "North")
#'
#' plot_dist(eurodist, method = "cmds", gp = regions,
#'           highlight = c("Madrid", "Cherbourg", "Rome", "Brussels"))
#'
plot_dist <- function(d,
                      method = c("cmds", "isomds", "tsne"),
                      highlight = NULL,
                      gp = NULL, point.alpha = 0.8) {

  # Checks ----
  method <- match.arg(method)

  d_labs <- attr(d, "Labels")

  if (!is.null(gp)) {
    missing <- setdiff(d_labs, names(gp))
    if (length(missing) > 0) {
      warning('Some objects in distance matrix "d" have no group assignment: ',
              paste(missing, collapse = ", "))
    }
  }

  # check if 'd' is a distance matrix
  if (!inherits(d, "dist")) {
    stop('"d" should be a distance matrix of class "dist".')
  }
  dist_labels <- d_labs # No dups as it is a distance matrix

  if (is.null(dist_labels)) {
    stop('Labels are missing in distance matrix "d".')
  }

  if (!is.null(highlight)) {
    # check if 'highlight' is a character vector
    if (!is.character(highlight)) {
      stop('"highlight" should be a character vector.')
    }

    # check if highlight is present in the entire set
    if (any(!(highlight %in% d_labs))) {
      alsel_miss <- highlight[!(highlight %in% d_labs)]
      stop(paste('The following entry/entries specified in "highlight" ',
                 'are not present in "data":\n',
                 paste(alsel_miss, collapse = ", "),
                 sep = ""))
    }
  }

  # check point.alpha argument is numeric vector of unit length
  if (!(is.numeric(point.alpha) && length(point.alpha) == 1)) {
    stop('"point.alpha" should be a numeric vector of unit length.')
  }

  n <- attr(d, "Size")

  # fallback for small matrices
  if (method == "isomds" && n < 3) {
    message("isoMDS requires n >= 3; falling back to cmds.")
    method <- "cmds"
  }

  if (method == "tsne" && n < 4) {
    message("tsne requires n >= 4; falling back to cmds.")
    method <- "cmds"
  }

  # Dimensional reduction ----
  if (method == "cmds") {
    fit <- cmdscale(d, k = 2)
  }

  if (method == "isomds") {
    fit <- MASS::isoMDS(d, trace = FALSE)$points
  }

  if (method == "tsne") {

    prplx <- max(1, min(30, floor((n - 1) / 3)))

    fit <- Rtsne::Rtsne(as.matrix(d),
      is_distance = TRUE, perplexity = prplx)$Y

    rownames(fit) <- d_labs
  }

  # Labels ----
  labs <- d_labs

  # Plotting dataframe ----
  df <- data.frame(sample = labs,
                   X = fit[, 1],
                   Y = fit[, 2])

  # Highlighting ----
  df$highlight <- if (!is.null(highlight) &&
                      length(highlight) > 0) {
    ifelse(df$sample %in% highlight, "highlight", "other")
  } else {
    "other"
  }

  # Ensure highlighted points are plotted last
  df$highlight <- factor(df$highlight,
                         levels = c("other", "highlight"))

  df <- df[order(df$highlight), ]

  # Groups ----
  if (!is.null(gp)) {

    # gp should be a named vector:
    # names(gp) = sample labels
    # values(gp) = group labels

    if (is.null(names(gp))) {
      stop("gp must be a named vector with sample labels as names.")
    }

    df$group <- gp[df$sample]

  } else {
    df$group <- "all"
  }

  # ggplot ----

  axis_label <- switch(method, cmds = "PCoA", isomds = "MDS", tsne = "t-SNE")

  p <- ggplot2::ggplot(df,
                       ggplot2::aes(x = X, y = Y,
                                    color = highlight)) +
    ggplot2::geom_point(size = 3, alpha = point.alpha) +
    ggplot2::scale_color_manual(values = c(other = "black",
        highlight = "red")) +
    ggplot2::labs(
      x = paste0(axis_label, " 1"), y = paste0(axis_label, " 2"),
      color = NULL,
      title = paste("Distance projection:", method)
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(legend.position = "none",
      strip.background = ggplot2::element_rect(fill = "grey95"))

  # Faceting ----
  if (!is.null(gp)) {
    p <- p +
      ggplot2::facet_wrap(~group)
  }

  return(p)
}

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

if (getRversion() >= "2.15.1")  {
  utils::globalVariables(
    c(
      "nclust", "idx", "sub_d", "dist.mat", "cassava_EC_gp",
      "X", "Y", "label"
    )
  )
}

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Allocation of Entries to be Selected from Clusters/Groups based on Diversity
#' Index Estimates for Core Collection Development
#'
#' Estimate the number of entries to be allocated from each cluster/group in the
#' entire collection to construct a core collection on the basis of different
#' metrics computed from within cluster/group diversity index estimates. The
#' following strategies are implemented. \loadmathjax
#' \itemize{
#' \item{Diversity}
#' \item{Diversity & Proportional}
#' \item{Diversity & Logarithmic}
#' \item{Diversity & Square root}}
#'
#' @section Details:
#'
#'   The number of entries to be chosen from each cluster is estimated either on
#'   the basis of diversity of entries within that cluster/group alone or in
#'   combination with the size of the cluster/group (See
#'   \strong{\code{Methods}}).
#'
#'   There are several methods proposed on the basis of diversity indices such
#'   as genetic multiplicity (G) dependent method based on the range of genetic
#'   diversity \insertCite{yonezawa_sampling_1995}{SampleCore}, H strategy based
#'   on Nei's gene diversity \insertCite{nei_analysis_1973}{SampleCore} and a
#'   method based on the pooled Shannon diversity index
#'   \insertCite{bisht_assessment_1999,mahajan_sampling_1999}{SampleCore}.
#'   Similarly, measures such as expected proportion of heterozygous loci per
#'   individual and effective number of alleles have also been employed as a
#'   diversity measure for determining sample size
#'   \insertCite{franco_sampling_2006}{SampleCore}.
#'
#'   The within-cluster/group diversity is estimated as either pooled or mean
#'   value of cluster/group-wise diversity indices. The following diversity
#'   indices are implemented in this function.
#'
#'   \itemize{
#'    \item{Shannon or Shannon-Weaver or Shannon-Wiener Diversity Index or
#'    Shannon entropy
#'    (\mjseqn{H}) \insertCite{shannon_mathematical_1949,peet_measurement_1974}{SampleCore}}
#'    \item{Simpson's Index of Diversity or Gini's Diversity Index or
#'    Gini-Simpson Index or Nei's Diversity Index or Nei's Variation Index
#'    (\mjseqn{D}) or Hurlbert’s probability of interspecific encounter
#'    (\mjseqn{PIE}) \insertCite{gini_variabilita_1912,gini_variabilita_1912-1,greenberg_measurement_1956,berger_diversity_1970,hurlbert_nonconcept_1971,nei_analysis_1973,peet_measurement_1974}{SampleCore}}
#'    \item{McIntosh Diversity Index (\mjseqn{D_{Mc}})
#'    \insertCite{mcintosh_index_1967,peet_measurement_1974}{DiversityStats}}}
#'
#' @template divmethods-section
#'
#' @template general-arg
#' @template size-arg
#' @template log-arg
#' @template qual-arg
#' @param method The allocation method. Either \code{"div"} for constant or
#'   \code{"div.prop"} for proportional or \code{"div.log"} for logarithmic or
#'   \code{"div.sqrt"} for square root allocation.
#' @template div-arg
#'
#' @template alloc-returns
#'
#' @importFrom DiversityStats simpson shannon mcintosh_diversity
#' @export
#'
#' @references
#'
#' \insertAllCited
#'
#' @seealso \code{\link[SampleCore]{allocate.basic}},
#'   \code{\link[SampleCore]{allocate.distance}}
#'
#' @examples
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Prepare example data
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' # Get data
#' data("cassava_EC_gp")
#'
#' set.seed(123)
#' cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]
#'
#' data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)
#' row.names(data) <- NULL
#'
#' # Column names of traits
#' quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
#'            "AVPW", "ARSR", "SRDM")
#' qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
#'           "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
#'           "PSTR")
#'
#' # Convert qualitative data columns to factor
#' data[, qual] <- lapply(data[, qual], as.factor)
#'
#' # Convert quantitative data columns to qualitative scores
#' quant_to_score5 <- function(x) {
#'
#'   brks <- unique( quantile(x,
#'                            probs = seq(0, 1, 0.2),
#'                            na.rm = TRUE))
#'   cut(x, breaks = brks,
#'       include.lowest = TRUE,
#'       labels = seq_len(length(brks) - 1))
#' }
#'
#' data[, quant] <- lapply(data[, quant], quant_to_score5)
#'
#' traits <- c(quant, qual)
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Custom diversity index functions
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' div_fun_brillouin <- function(x) {
#'   n <- tabulate(x)
#'   n <- n[n > 0]
#'   N <- sum(n)
#'   if (N <= 1) {
#'     return(0)
#'   }
#'   (lgamma(N + 1) - sum(lgamma(n + 1)))/N
#' }
#'
#' div_fun_margalef <- function(x) {
#'   tab <- tabulate(x)
#'   tab <- tab[tab > 0]
#'   S <- length(tab)
#'   N <- length(x)
#'   if (N <= 1) {
#'     return(0)
#'   }
#'   (S - 1)/log(N)
#' }
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Diversity allocation
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' ## Shannon-Weaver Diversity Index
#' div_out_shannon1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                     group = "Cluster",
#'                     qualitative = traits,
#'                     method = "div",
#'                     div.index = "shannon", metric = "pooled",
#'                     size = 0.2)
#' div_out_shannon1
#'
#' div_out_shannon2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.index = "shannon", metric = "mean",
#'                      size = 0.2)
#' div_out_shannon2
#'
#' ##  Gini-Simpson Index
#' div_out_simpson1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.index = "simpson", metric = "pooled",
#'                      size = 0.2)
#' div_out_simpson1
#'
#' div_out_simpson2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.index = "simpson", metric = "mean",
#'                      size = 0.2)
#' div_out_simpson2
#'
#' ## McIntosh Diversity Index
#' div_out_mcintosh1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.index = "mcintosh", metric = "pooled",
#'                      size = 0.2)
#' div_out_mcintosh1
#'
#' div_out_mcintosh2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.index = "mcintosh", metric = "mean",
#'                      size = 0.2)
#' div_out_mcintosh2
#'
#' ## Richness
#' div_out_richness1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.index = "richness", metric = "pooled",
#'                      size = 0.2)
#' div_out_richness1
#'
#' div_out_richness2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.index = "richness", metric = "mean",
#'                      size = 0.2)
#' div_out_richness2
#'
#' ## Brillouin Diversity Index
#' div_out_brillouin1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.fun = div_fun_brillouin, metric = "pooled",
#'                      size = 0.2)
#' div_out_brillouin1
#'
#' div_out_brillouin2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.fun = div_fun_brillouin, metric = "mean",
#'                      size = 0.2)
#' div_out_brillouin2
#'
#' ## Margalef's richness Index
#' div_out_margalef1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.fun = div_fun_margalef, metric = "pooled",
#'                      size = 0.2)
#' div_out_margalef1
#'
#' div_out_margalef2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div",
#'                      div.fun = div_fun_margalef, metric = "mean",
#'                      size = 0.2)
#' div_out_margalef2
#'
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Diversity allocation & Proportional
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' ## Shannon-Weaver Diversity Index
#' dist_prop_out_shannon1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.index = "shannon", metric = "pooled",
#'                      size = 0.2)
#' dist_prop_out_shannon1
#'
#' dist_prop_out_shannon2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.index = "shannon", metric = "mean",
#'                      size = 0.2)
#' dist_prop_out_shannon2
#'
#' ##  Gini-Simpson Index
#' dist_prop_out_simpson1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.index = "simpson", metric = "pooled",
#'                      size = 0.2)
#' dist_prop_out_simpson1
#'
#' dist_prop_out_simpson2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.index = "simpson", metric = "mean",
#'                      size = 0.2)
#' dist_prop_out_simpson2
#'
#' ## McIntosh Diversity Index
#' dist_prop_out_mcintosh1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.index = "mcintosh", metric = "pooled",
#'                      size = 0.2)
#' dist_prop_out_mcintosh1
#'
#' dist_prop_out_mcintosh2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.index = "mcintosh", metric = "mean",
#'                      size = 0.2)
#' dist_prop_out_mcintosh2
#'
#' ## Richness
#' div_out_richness1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "richness", metric = "pooled",
#'                      size = 0.2)
#' div_out_richness1
#'
#' div_out_richness2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "richness", metric = "mean",
#'                      size = 0.2)
#' div_out_richness2
#'
#' ## Brillouin Diversity Index
#' div_out_brillouin1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.fun = div_fun_brillouin, metric = "pooled",
#'                      size = 0.2)
#' div_out_brillouin1
#'
#' div_out_brillouin2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.fun = div_fun_brillouin, metric = "mean",
#'                      size = 0.2)
#' div_out_brillouin2
#'
#' ## Margalef's richness Index
#' div_out_margalef1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.fun = div_fun_margalef, metric = "pooled",
#'                      size = 0.2)
#' div_out_margalef1
#'
#' div_out_margalef2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.prop",
#'                      div.fun = div_fun_margalef, metric = "mean",
#'                      size = 0.2)
#' div_out_margalef2
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Diversity allocation & Logarithmic
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' ## Shannon-Weaver Diversity Index
#' dist_log_out_shannon1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "shannon", metric = "pooled",
#'                      size = 0.2)
#' dist_log_out_shannon1
#'
#' dist_log_out_shannon2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "shannon", metric = "mean",
#'                      size = 0.2)
#' dist_log_out_shannon2
#'
#' ##  Gini-Simpson Index
#' dist_log_out_simpson1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "simpson", metric = "pooled",
#'                      size = 0.2)
#' dist_log_out_simpson1
#'
#' dist_log_out_simpson2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "simpson", metric = "mean",
#'                      size = 0.2)
#' dist_log_out_simpson2
#'
#' ## McIntosh Diversity Index
#' dist_log_out_mcintosh1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "mcintosh", metric = "pooled",
#'                      size = 0.2)
#' dist_log_out_mcintosh1
#'
#' dist_log_out_mcintosh2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "mcintosh", metric = "mean",
#'                      size = 0.2)
#' dist_log_out_mcintosh2
#'
#' ## Richness
#' div_out_richness1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "richness", metric = "pooled",
#'                      size = 0.2)
#' div_out_richness1
#'
#' div_out_richness2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.index = "richness", metric = "mean",
#'                      size = 0.2)
#' div_out_richness2
#'
#' ## Brillouin Diversity Index
#' div_out_brillouin1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.fun = div_fun_brillouin, metric = "pooled",
#'                      size = 0.2)
#' div_out_brillouin1
#'
#' div_out_brillouin2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.fun = div_fun_brillouin, metric = "mean",
#'                      size = 0.2)
#' div_out_brillouin2
#'
#' ## Margalef's richness Index
#' div_out_margalef1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.fun = div_fun_margalef, metric = "pooled",
#'                      size = 0.2)
#' div_out_margalef1
#'
#' div_out_margalef2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.log",
#'                      div.fun = div_fun_margalef, metric = "mean",
#'                      size = 0.2)
#' div_out_margalef2
#'
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#' # Diversity allocation & Square root
#' #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#'
#' ## Shannon-Weaver Diversity Index
#' dist_sqrt_out_shannon1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.index = "shannon", metric = "pooled",
#'                      size = 0.2)
#' dist_sqrt_out_shannon1
#'
#' dist_sqrt_out_shannon2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.index = "shannon", metric = "mean",
#'                      size = 0.2)
#' dist_sqrt_out_shannon2
#'
#' ##  Gini-Simpson Index
#' dist_sqrt_out_simpson1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.index = "simpson", metric = "pooled",
#'                      size = 0.2)
#' dist_sqrt_out_simpson1
#'
#' dist_sqrt_out_simpson2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.index = "simpson", metric = "mean",
#'                      size = 0.2)
#' dist_sqrt_out_simpson2
#'
#' ## McIntosh Diversity Index
#' dist_sqrt_out_mcintosh1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.index = "mcintosh", metric = "pooled",
#'                      size = 0.2)
#' dist_sqrt_out_mcintosh1
#'
#' dist_sqrt_out_mcintosh2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.index = "mcintosh", metric = "mean",
#'                      size = 0.2)
#' dist_sqrt_out_mcintosh2
#'
#' ## Richness
#' div_out_richness1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.index = "richness", metric = "pooled",
#'                      size = 0.2)
#' div_out_richness1
#'
#' div_out_richness2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.index = "richness", metric = "mean",
#'                      size = 0.2)
#' div_out_richness2
#'
#' ## Brillouin Diversity Index
#' div_out_brillouin1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.fun = div_fun_brillouin, metric = "pooled",
#'                      size = 0.2)
#' div_out_brillouin1
#'
#' div_out_brillouin2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.fun = div_fun_brillouin, metric = "mean",
#'                      size = 0.2)
#' div_out_brillouin2
#'
#' ## Margalef's richness Index
#' div_out_margalef1 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.fun = div_fun_margalef, metric = "pooled",
#'                      size = 0.2)
#' div_out_margalef1
#'
#' div_out_margalef2 <-
#'   allocate.diversity(data = data, names = "genotypes",
#'                      group = "Cluster",
#'                      qualitative = traits,
#'                      method = "div.sqrt",
#'                      div.fun = div_fun_margalef, metric = "mean",
#'                      size = 0.2)
#' div_out_margalef2
#'
allocate.diversity <- function(data, names, group,
                               qualitative,
                               method = c("div", "div.prop",
                                          "div.sqrt", "div.log"),
                               div.index = c("richness", "shannon",
                                             "simpson", "mcintosh"),
                               shannon.base = exp(1),
                               div.fun = NULL,
                               log.base = exp(1),
                               metric = c("pooled", "mean"),
                               size) {

  # Check stats ----

  checks.sample.core(data = data, size = size,
                     names = names, group = group,
                     dist.mat = NULL,
                     qualitative = qualitative,
                     log.base = log.base,
                     mode = "alloc")

  method <- match.arg(method)
  metric <- match.arg(metric)

  if (!is.null(div.fun)) {
    div.index <- NULL
  }

  if (!is.null(div.index)) {

    div.index <- match.arg(div.index)

  } else {

    if (!is.function(div.fun)) {
      stop('"div.fun" must be a function.')
    }

    div_res <- div.fun(as.factor(cassava_EC_gp$CUAL))
    if (!(is.numeric(div_res) && length(div_res) == 1)) {
      stop('"div.fun" should return a numeric vector of unit length.')
    }
  }

  # Prepare data ----

  data[, group] <- droplevels(data[, group])

  gp_memb <- data[, group]

  if (method != "nclust") {
    # Convert quantitative to qualitative
  }

  # traits <- c(qualitative, quantitative)
  traits <- c(qualitative)

  # Basic group stats ----

  gps <- levels(data[, group])
  gpsize <- summary(data[, group])
  gpcount <- length(gps)

  tcount <- nrow(data)


  # Compute group-wise diversity metrics ----

    group_dist_metric <- sapply(unique(gp_memb), function(g) {
      idx <- which(gp_memb == g)

      # Get trait-wise diversity indices
      trait_wise_div <- sapply(traits, function(trt) {

        trt_x <- droplevels(data[idx, trt])

        if (is.null(div.fun)) {
          ## Richness ----
          if (div.index == "richness") {
            out <- length(unique(trt_x))
          }

          ## Shannon ----
          if (div.index == "shannon") {
            out <- DiversityStats::shannon(trt_x, base = shannon.base)
          }

          ## Simpson ----
          if (div.index == "simpson") {
            out <- DiversityStats::gini_simpson(trt_x)
          }

          ## McIntosh ----
          if (div.index == "mcintosh") {
            out <- DiversityStats::mcintosh_diversity(trt_x)
          }
        } else {
          ## Custom with div.fun ----
          out <- div.fun(trt_x)
        }

        return(out)

      })

      ## Pooled diversity ----
      if (metric == "pooled") {
        metric_out <- sum(trait_wise_div)
      }

      ## Mean/Average diversity ----
      if (metric == "mean") {
        metric_out <- mean(trait_wise_div)
      }

      return(metric_out)

    })

  # Freq estimation ----

  ## "div" ----
  if (method == "div") {
    freq <- group_dist_metric / sum(group_dist_metric)
  }

  ## "div.prop" ----
  if (method == "div.prop") {
    freq <- (group_dist_metric * gpsize) /
      sum(group_dist_metric * gpsize)
  }

  ## "div.log" ----
  if (method == "div.log") {
    log_gpsize <- log(gpsize, base = log.base)
    freq <- (group_dist_metric * log_gpsize) /
      sum(group_dist_metric * log_gpsize)
  }

  ## "div.sqrt" ----
  if (method == "div.sqrt") {
    freq <- (group_dist_metric * sqrt(gpsize)) /
      sum(group_dist_metric * sqrt(gpsize))
  }

  # Get output ---
  out <- setNames(round(freq * size * tcount), names(gpsize))

  return(out)

}

### This file is part of 'SampleCore' package for R.

### Copyright (C) 2024-2026, ICAR-NBPGR.
#
# SampleCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# SampleCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  https://www.r-project.org/Licenses/

#' Allocation of Entries to be Selected from Clusters/Groups based on Size for
#' Core Collection Development
#'
#' Estimate the number of entries to be allocated from each cluster/group in the
#' entire collection to construct a core collection on the basis of
#' cluster/group size. The following strategies are implemented. \loadmathjax
#' \itemize{
#' \item{Constant}
#' \item{Proportional}
#' \item{Logarithmic}
#' \item{Square root}}
#' The different methods to determine the number of entries from each group or
#' clusters implemented in \code{allocate.basic} are as follows.
#'
#' These are different methods which estimate the number of entries only on the
#' basis of total number of entries in each cluster/group.
#'
#' \insertCite{brown_core_1989;textual}{SampleCore} proposed the constant (C),
#' proportional (P) and logarithmic (L) methods and later a similar square root
#' method was proposed by
#' \insertCite{huaman_selecting_1999;textual}{SampleCore}.
#'
#'   \subsection{Constant method}{From an entire collection of size \mjseqn{N},
#'   to construct a core set of sample size \mjseqn{n}, the number of entries to
#'   be selected from the \mjseqn{i}th group among \mjseqn{1 \cdots g} groups
#'   (\mjseqn{n_{i}}) is estimated as below.
#'
#'   \mjsdeqn{n_{i} = \frac{n}{g} \times N}
#'
#'   }
#'
#'   \subsection{Proportional method}{Here the number of entries to be selected
#'   is proportional to the cluster/group size (\mjseqn{N_{i}}) as
#'   below.
#'
#'   \mjsdeqn{n_{i} = n \times \frac{N_{i}}{\sum_{i=1}^{g}N_{i}}}
#'
#'   \mjsdeqn{n_{i} = n \times \frac{N_{i}}{N}}
#'
#'   }
#'
#'   \subsection{Logarithmic method}{Here the number of entries to be selected
#'   is proportional to the logarithm of the cluster/group size
#'   (\mjseqn{N_{i}}) as below.
#'
#'   \mjsdeqn{n_{i} = n \times
#'   \frac{\log{(N_{i})}}{\sum_{i=1}^{g}\log{(N_{i})}}}
#'
#'   }
#'
#'   \subsection{Square root method}{Here the number of entries to
#'   be selected is proportional to the square root of the cluster/group size
#'   (\mjseqn{N_{i}}) as below.
#'
#'   \mjsdeqn{n_{i} = n \times \frac{\sqrt{N_{i}}}{\sum_{i=1}^{g}\sqrt{N_{i}}}}
#'
#'   }
#'
#' @template general-arg
#' @template size-arg
#' @template log-arg
#' @param method The allocation method. Either \code{"const"} for constant or
#'   \code{"prop"} for proportional or \code{"log"} for logarithmic or
#'   \code{"sqrt"} for square root allocation.
#'
#' @template alloc-returns
#'
#' @import mathjaxr
#' @importFrom Rdpack reprompt
#' @export
#'
#' @references
#'
#' \insertAllCited
#'
#' @seealso \code{\link[SampleCore]{allocate.distance}},
#'   \code{\link[SampleCore]{allocate.diversity}}
#'
#' @examples
#' # Get data
#' data("cassava_EC_gp")
#'
#' set.seed(123)
#' cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]
#'
#' data <- cassava_EC_gp
#'
#' data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)
#' row.names(data) <- NULL
#'
#' # Constant allocation
#' const_out <-
#'   allocate.basic(data = data, names = "genotypes",
#'                  group = "Cluster", method = "const",
#'                  size = 0.2)
#' const_out
#'
#' # Proportional allocation
#' prop_out <-
#'   allocate.basic(data = data, names = "genotypes",
#'                  group = "Cluster", method = "prop",
#'                  size = 0.2)
#' prop_out
#'
#' # Logarithmic allocation
#' log_out <-
#'   allocate.basic(data = data, names = "genotypes",
#'                  group = "Cluster", method = "log",
#'                  size = 0.2)
#' log_out
#'
#' # Square root allocation
#' sqrt_out <-
#'   allocate.basic(data = data, names = "genotypes",
#'                  group = "Cluster", method = "sqrt",
#'                  size = 0.2)
#' sqrt_out
#'
allocate.basic <- function(data,
                           names, group,
                           method = c("const", "prop", "log", "sqrt"),
                           log.base = exp(1),
                           size) {

  # Check stats ----

  checks.sample.core(data = data, size = size,
                     names = names, group = group,
                     dist.mat = NULL,
                     qualitative = NULL,
                     log.base = log.base,
                     mode = "alloc")

  method <- match.arg(method)

  # Prepare data ----

  data[, group] <- droplevels(data[, group])

  # Basic group stats ----

  gps <- levels(data[, group])
  gpsize <- summary(data[, group])
  gpcount <- length(gps)

  tcount <- nrow(data)

  # Constant allocation ----

  if (method == "const") {

    # Candidate sample sizes
    smpsize_c <- ceiling((size * tcount) / gpcount)
    smpsize_f <- floor((size * tcount) / gpcount)

    # Choose value closer to target total sample size
    if (abs((smpsize_c * gpcount) - (size * tcount)) >=
        abs((smpsize_f * gpcount) - (size * tcount))) {
      smpsize <- smpsize_f
    } else {
      smpsize <- smpsize_c
    }

    # Assign sample size to groups
    out <- rep(smpsize, length(gps))
    names(out) <- names(gpsize)

    # Check if sample size exceeds the group size
    # ie. group is too small to support the required sample size
    if (any(gpsize < smpsize)) {
      # Problem groups
      sml_gpsize <- gpsize[gpsize < smpsize]

      sml_gpsize <- paste(names(sml_gpsize), " (=", sml_gpsize,")",
                          sep = "", collapse = ", ")

      warning('For the following groups in "', group,
              '" the sampling count estimated with "size" = ', size,
              " (", smpsize, ")", " exceeds the group count.\n", sml_gpsize,
              "\nRestricting the sampling count to group counts",
              " for these groups.")

      out <- ifelse(gpsize < smpsize, gpsize, smpsize)
    }

  }

  if (method != "const") {

  # Freq estimation ----

  ## "prop" ----
  if (method == "prop") {
    freq <- gpsize / sum(gpsize)
  }

  ## "log" ----
  if (method == "log") {
    log_gpsize <- log(gpsize, base = log.base)
    freq <- log_gpsize / sum(log_gpsize)
  }

  ## "sqrt" ----
  if (method == "sqrt") {
    freq <- sqrt(gpsize) / sum(sqrt(gpsize))
  }

  # Get output ---
    out <- setNames(round(freq * size * tcount), names(gpsize))
  }

  return(out)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checks.sample.core.R
\name{checks.sample.core}
\alias{checks.sample.core}
\title{Common checks for all functions}
\usage{
checks.sample.core(
  data,
  names,
  size,
  group,
  qualitative = NULL,
  dist.mat = NULL,
  log.base = NULL,
  alloc,
  always.selected = NULL,
  mode = c("alloc", "sel")
)
}
\arguments{
\item{data}{The data as a data frame object. The data frame should possess
one row per individual and columns with the individual names and multiple
trait/character data.}

\item{names}{Name of column with the accession names as a character string.}

\item{size}{The desired core set size proportion.}

\item{group}{Name of column with the accession group/cluster names as a
character string.}

\item{qualitative}{Name of columns with the qualitative traits as a character
vector.}

\item{dist.mat}{A precomputed distance matrix of distance measures between the
accessions in \code{data}.}

\item{log.base}{The logarithm base to be used for logarithmic method of
sampling. Default is \code{exp(1)}.}

\item{alloc}{A named numeric vector specifying the number of entries to be
selected. Names should correspond to the levels of the "\code{"group"}
column, and values indicate the number of elements to be selected from each
level.}

\item{always.selected}{Names of accessions to be always included in the core
set as a character vector.}
}
\description{
Not exported. Strictly internal
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allocate.distance.R
\name{allocate.distance}
\alias{allocate.distance}
\title{Allocation of Entries to be Selected from Clusters/Groups based on
Distance-based Diversity Metrics for Core Collection Development}
\usage{
allocate.distance(
  data,
  names,
  group,
  dist.mat,
  method = c("dist", "dist.prop", "dist.log", "dist.sqrt"),
  metric = c("mean", "median", "max", "range", "mnnd", "mdc", "mdm", "mstl", "nclust"),
  clust.fun = NULL,
  log.base = exp(1),
  size
)
}
\arguments{
\item{data}{The data as a data frame object. The data frame should possess
one row per individual and columns with the individual names and multiple
trait/character data.}

\item{names}{Name of column with the accession names as a character string.}

\item{group}{Name of column with the accession group/cluster names as a
character string.}

\item{dist.mat}{A precomputed distance matrix of distance measures between the
accessions in \code{data}.}

\item{method}{The allocation method. Either \code{"dist"} for constant or
\code{"dist.prop"} for proportional or \code{"dist.log"} for logarithmic or
\code{"dist.sqrt"} for square root allocation. See \strong{Methods}.}

\item{metric}{The metric to be computed from the distance matrix. Either
\code{"mean"}, \code{"median"}, \code{"max"}, \code{"range"},
\code{"mnnd"}, \code{"mdc"}, \code{"mdm"}, \code{"mstl"}, or
\code{"nclust"}. See \strong{Metrics}.}

\item{clust.fun}{A function to generate clusters from a distance matrix and
return the number of clusters.}

\item{log.base}{The logarithm base to be used for logarithmic method of
sampling. Default is \code{exp(1)}.}

\item{size}{The desired core set size proportion.}
}
\value{
A named numeric vector specifying the number of entries to be
  selected from each cluster/group. The vector names correspond to the levels
  of the "\code{"group"} column, and values indicate the number of elements
  to be selected from each level.
}
\description{
Estimate the number of entries to be allocated from each cluster/group in
the entire collection to construct a core collection on the basis of
different metrics computed from within cluster/group distances. The following
strategies are implemented. \loadmathjax
\itemize{
\item{Diversity (Distance based)}
\item{Diversity (Distance based) & Proportional}
\item{Diversity (Distance based) & Logarithmic}
\item{Diversity (Distance based) & Square root}}
}
\section{Details}{


  The number of entries to be chosen from each cluster is estimated either on
  the basis of diversity of entries within that cluster/group alone or in
  combination with the size of the cluster/group (See \strong{Methods}).

  The within-cluster/group diversity is estimated as several metrics from the
  within cluster/group genetic distances between accessions (See
  \strong{Metrics}).

  \insertCite{franco_sampling_2005;textual}{SampleCore} proposed a method
  based on mean Gower's distance \insertCite{gower_general_1971}{SampleCore}
  which was also extended to other distance measure averages named D
  Allocation strategy \insertCite{franco_sampling_2006}{SampleCore}. These
  methods were also combined with the proportional and logarithmic methods.
  For example, the GP and GL strategy of
  \insertCite{bisht_assessment_1999;textual}{SampleCore} and
  \insertCite{mahajan_sampling_1999;textual}{SampleCore} as well as the NY
  and LD allocation methods of
  \insertCite{franco_sampling_2005;textual}{SampleCore}.
}

\section{Methods}{


  \subsection{Diversity method}{From an entire collection of size \mjseqn{N},
  to construct a core set of sample size \mjseqn{n}, the number of entries to
  be selected from the \mjseqn{i}th group among \mjseqn{1 \cdots g} groups
  (\mjseqn{n_{i}}) is estimated as below.

   \mjsdeqn{n_{i} = n \times \frac{D_{i}}{\sum_{i=1}^{g}D_{i}}}

   Where, \mjseqn{D_{i}} is a measure of the extent of diversity present in
   the \mjseqn{i}th cluster.

   }

   \subsection{Diversity and proportional method}{Here the number of entries
   to be selected is proportional to the diversity of the cluster/group
   (\mjseqn{D_{i}}) weighted by the the cluster/group size (\mjseqn{N_{i}}).

    \mjsdeqn{n_{i} = n \times \frac{N_{i}D_{i}}{\sum_{i=1}^{g}N_{i}D_{i}}}

   }

   \subsection{Diversity and logarithmic method}{Here the number of entries
   to be selected is proportional to the diversity of the cluster/group
   (\mjseqn{D_{i}}) weighted by the logarithm of the cluster/group size
   (\mjseqn{N_{i}}).

   \mjsdeqn{n_{i} = n \times
   \frac{\log(N_{i})D_{i}}{\sum_{i=1}^{g}\log(N_{i})D_{i}}}

   }

   \subsection{Diversity and square root method}{Here the number of entries
   to be selected is proportional to the diversity of the cluster/group
   (\mjseqn{D_{i}}) weighted by the square root of the cluster/group size
   (\mjseqn{N_{i}}).

   \mjsdeqn{n_{i} = n \times
   \frac{\sqrt{N_{i}}D_{i}}{\sum_{i=1}^{g}\sqrt{N_{i}}D_{i}}}

   }
}

\section{Metrics}{


  \subsection{Summary/Decriptive statistics}{These include mean, median,
  maximum and range of genetic distances between entries in a cluster.}

  \subsection{Mean nearest-neighbour distance (\mjseqn{MNND})}{It is the
  average, across all entries, of the distance to each entry’s
  closest other entry (\mjseqn{d_{g_{min}}}), based on a genetic given
  distance matrix \insertCite{clark_distance_1954}{SampleCore}.

  For each entry, the nearest-neighbour distance (\mjseqn{d_{g_{min}}})
  is the smallest non-zero distance with any other entry.

  \mjsdeqn{d_{g_{min}} = \min_{h \ne g} d_{gh}}

  The Mean nearest-neighbour distance (\mjseqn{MNND}) can then be computed
  as:

  \mjsdeqn{\textrm{MNND} = \frac{1}{G} \sum_{g=1}^{G} d_g}

  Where, (\mjseqn{g}) is the index of an entry in a genetic distance
  matrix, \mjseqn{h} is the index of all other genotypes and \mjseqn{G} is
  the total number of genotypes in a cluster/group.

  }

  \subsection{Minimum spanning tree length (\mjseqn{MSTL})}{It is defined as
  the sum of edge weights in the minimum spanning tree constructed from the
  genetic distance matrix of entries within a cluster/group.  A minimum
  spanning tree (MST) connects all entries such that the total distance
  is minimized and no cycles are formed. It represents the most efficient
  way to connect all entries based on pairwise genetic distances
  \insertCite{gower_minimum_1969}{SampleCore}.

  For genetic distance \mjseqn{d_{gh}} between entries \mjseqn{g} and
  \mjseqn{h}, the MST is a subset of edges that connects all
  \mjseqn{G} entries with exactly \mjseqn{G - 1} edges and minimum total
  weight. The MST length (\mjseqn{MSTL}) can then be computed as:

  \mjsdeqn{\textrm{MSTL} = \sum_{(g,h) \in \mathcal{T}} d_{gh}}

  Where \mjseqn{\mathcal{T}} denotes the set of edges in the MST.

  }

  \subsection{Mean distance to centroid and median (\mjseqn{MDC},
  \mjseqn{MDM})}{These quantify the average dispersion of entries within a
  cluster/group relative to a central point in multivariate space derived
  from the genetic distance matrix.

  The centroid represents the multivariate mean position of all entries
  in a cluster
  \insertCite{sokal_principles_1963,sneath_numerical_1973}{SampleCore}.,
  whereas the median (spatial median) provides a robust central location
  that is less influenced by extreme values
  \insertCite{bradley_constrained_1999}{SampleCore}.

  For \mjseqn{d_{gC}} and \mjseqn{d_{gM}} distances of entry
  \mjseqn{g} from the centroid \mjseqn{C} and median \mjseqn{M},
  respectively. These measures are computed as:

  \mjsdeqn{\textrm{MDC} = \frac{1}{G} \sum_{g=1}^{G} d_{gC}}

  \mjsdeqn{\textrm{MDM} = \frac{1}{G} \sum_{g=1}^{G} d_{gM}}

  Where \mjseqn{G} is the total number of entries in the cluster/group.

  }

  \subsection{Number of clusters}{\insertCite{diwan_core_1994}{SampleCore}
  proposed the number of clusters produced by a multivariate cluster
  analysis at a specific distance threshold as an estimate of the diversity.

  }
}

\examples{
\dontshow{if (requireNamespace("cluster", quietly = TRUE) & requireNamespace("fastcluster", quietly = TRUE) & requireNamespace("dbscan", quietly = TRUE) & requireNamespace("biotools", quietly = TRUE)) withAutoprint(\{ # examplesIf}
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

library(cluster)

# Get distance matrix
data("cassava_EC_gp")

set.seed(123)
cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]

quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
           "AVPW", "ARSR", "SRDM")
qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
          "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
          "PSTR")

data <- cassava_EC_gp

# Convert qualitative data columns to factor
data[, qual] <- lapply(data[, qual], as.factor)

# Standardise quantitative data column
data[, quant] <- lapply(data[, quant], function(x) {
  scale(x)[, 1]
})

# Get the Gower's distance matrix
dist_matrix <- daisy(x = data[, c(qual, quant)],
                     metric = "gower")

# Get data
data <- cassava_EC_gp
data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)
row.names(data) <- NULL

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Custom clustering functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# UPGMA with hclust
clust_fun_upgma <- function(x) {
  # Tree
  tree_out <- hclust(x, method = "average")
  # Clusters
  cutree(tree_out, h = 0.2)
}

if (requireNamespace('fastcluster', quietly = TRUE)) {
  # Ward's minimum variance with fastcluster
  clust_fun_ward <- function(x) {
    # Tree
    tree_out <- fastcluster::hclust(x, method = "ward.D2")
    # Clusters
    cutree(tree_out, h = 0.2)
  }
}

if (requireNamespace('dbscan', quietly = TRUE)) {
  # Density-based clustering with dbscan
  clust_fun_dbscan <- function(x) {
    clust_out <- dbscan::dbscan(x, eps = 0.25)
    # remove noise: TODO
    setNames(clust_out$cluster, labels(x))
  }
}

if (requireNamespace('biotools', quietly = TRUE)) {
  # Tocher's sequential clustering
  clust_fun_tocher <- function(x) {
    clust_out <- biotools::tocher(x, algorithm = "sequential")
    setNames(clust_out$class, labels(x))
  }
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Diversity (Distance based) allocation
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

## Mean
dist_out_mean <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist",
                    dist.mat = dist_matrix, metric = "mean",
                    size = 0.2)
dist_out_mean

## Median
dist_out_median <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist",
                    dist.mat = dist_matrix, metric = "median",
                    size = 0.2)
dist_out_median

## Maximum
dist_out_max <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist",
                    dist.mat = dist_matrix, metric = "max",
                    size = 0.2)
dist_out_max

## Range
dist_out_range <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist",
                    dist.mat = dist_matrix, metric = "range",
                    size = 0.2)
dist_out_range

## Mean nearest-neighbour distance
dist_out_mnnd <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist",
                    dist.mat = dist_matrix, metric = "mnnd",
                    size = 0.2)
dist_out_mnnd

## Minimum spanning tree length
dist_out_mstl <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist",
                    dist.mat = dist_matrix, metric = "mstl",
                    size = 0.2)
dist_out_mstl

\donttest{
  ## Mean distance to centroid
  dist_out_mdc <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist",
                      dist.mat = dist_matrix, metric = "mdc",
                      size = 0.2)
  dist_out_mdc

  ## Mean distance to median
  dist_out_mdm <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist",
                      dist.mat = dist_matrix, metric = "mdm",
                      size = 0.2)
  dist_out_mdm
}

## Number of clusters

### UPGMA with hclust
dist_out_nclust1 <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist",
                    dist.mat = dist_matrix, metric = "nclust",
                    clust.fun = clust_fun_upgma,
                    size = 0.2)
dist_out_nclust1

# Ward's minimum variance with fastcluster
if (requireNamespace('fastcluster', quietly = TRUE)) {
  dist_out_nclust2 <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist",
                      dist.mat = dist_matrix, metric = "nclust",
                      clust.fun = clust_fun_ward,
                      size = 0.2)
  dist_out_nclust2
}


# Density-based clustering with dbscan
if (requireNamespace('dbscan', quietly = TRUE)) {
  dist_out_nclust3 <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist",
                      dist.mat = dist_matrix, metric = "nclust",
                      clust.fun = clust_fun_dbscan,
                      size = 0.2)
  dist_out_nclust3
}

\donttest{
  if (requireNamespace('biotools', quietly = TRUE)) {
    # Tocher's sequential clustering
    dist_out_nclust4 <-
      allocate.distance(data = data, names = "genotypes",
                        group = "Cluster", method = "dist",
                        dist.mat = dist_matrix, metric = "nclust",
                        clust.fun = clust_fun_tocher,
                        size = 0.2)
    dist_out_nclust4
  }
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Diversity (Distance based) & Proportional
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

## Mean
dist_prop_out_mean <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.prop",
                    dist.mat = dist_matrix, metric = "mean",
                    size = 0.2)
dist_prop_out_mean

## Median
dist_prop_out_median <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.prop",
                    dist.mat = dist_matrix, metric = "median",
                    size = 0.2)
dist_prop_out_median

## Maximum
dist_prop_out_max <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.prop",
                    dist.mat = dist_matrix, metric = "max",
                    size = 0.2)
dist_prop_out_max

## Range
dist_prop_out_range <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.prop",
                    dist.mat = dist_matrix, metric = "range",
                    size = 0.2)
dist_prop_out_range

## Mean nearest-neighbour distance
dist_prop_out_mnnd <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.prop",
                    dist.mat = dist_matrix, metric = "mnnd",
                    size = 0.2)
dist_prop_out_mnnd

## Minimum spanning tree length
dist_prop_out_mstl <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.prop",
                    dist.mat = dist_matrix, metric = "mstl",
                    size = 0.2)
dist_prop_out_mstl

\donttest{
  ## Mean distance to centroid
  dist_prop_out_mdc <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.prop",
                      dist.mat = dist_matrix, metric = "mdc",
                      size = 0.2)
  dist_prop_out_mdc

  ## Mean distance to median
  dist_prop_out_mdm <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.prop",
                      dist.mat = dist_matrix, metric = "mdm",
                      size = 0.2)
  dist_prop_out_mdm
}

## Number of clusters

### UPGMA with hclust
dist_prop_out_nclust1 <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.prop",
                    dist.mat = dist_matrix, metric = "nclust",
                    clust.fun = clust_fun_upgma,
                    size = 0.2)
dist_prop_out_nclust1

# Ward's minimum variance with fastcluster
if (requireNamespace('fastcluster', quietly = TRUE)) {
  dist_prop_out_nclust2 <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.prop",
                      dist.mat = dist_matrix, metric = "nclust",
                      clust.fun = clust_fun_ward,
                      size = 0.2)
  dist_prop_out_nclust2
}

# Density-based clustering with dbscan
if (requireNamespace('dbscan', quietly = TRUE)) {
  dist_prop_out_nclust3 <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.prop",
                      dist.mat = dist_matrix, metric = "nclust",
                      clust.fun = clust_fun_dbscan,
                      size = 0.2)
  dist_prop_out_nclust3
}

\donttest{
  if (requireNamespace('biotools', quietly = TRUE)) {
    # Tocher's sequential clustering
    dist_prop_out_nclust4 <-
      allocate.distance(data = data, names = "genotypes",
                        group = "Cluster", method = "dist.prop",
                        dist.mat = dist_matrix, metric = "nclust",
                        clust.fun = clust_fun_tocher,
                        size = 0.2)
    dist_prop_out_nclust4
  }
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Diversity (Distance based) & Logarithmic
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

## Mean
dist_log_out_mean <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.log",
                    dist.mat = dist_matrix, metric = "mean",
                    size = 0.2)
dist_log_out_mean

## Median
dist_log_out_median <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.log",
                    dist.mat = dist_matrix, metric = "median",
                    size = 0.2)
dist_log_out_median

## Maximum
dist_log_out_max <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.log",
                    dist.mat = dist_matrix, metric = "max",
                    size = 0.2)
dist_log_out_max

## Range
dist_log_out_range <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.log",
                    dist.mat = dist_matrix, metric = "range",
                    size = 0.2)
dist_log_out_range

## Mean nearest-neighbour distance
dist_log_out_mnnd <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.log",
                    dist.mat = dist_matrix, metric = "mnnd",
                    size = 0.2)
dist_log_out_mnnd

## Minimum spanning tree length
dist_log_out_mstl <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.log",
                    dist.mat = dist_matrix, metric = "mstl",
                    size = 0.2)
dist_log_out_mstl

\donttest{
  ## Mean distance to centroid
  dist_log_out_mdc <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.log",
                      dist.mat = dist_matrix, metric = "mdc",
                      size = 0.2)
  dist_log_out_mdc

  ## Mean distance to median
  dist_log_out_mdm <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.log",
                      dist.mat = dist_matrix, metric = "mdm",
                      size = 0.2)
  dist_log_out_mdm
}

## Number of clusters

### UPGMA with hclust
dist_log_out_nclust1 <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.log",
                    dist.mat = dist_matrix, metric = "nclust",
                    clust.fun = clust_fun_upgma,
                    size = 0.2)
dist_log_out_nclust1

# Ward's minimum variance with fastcluster
if (requireNamespace('fastcluster', quietly = TRUE)) {
  dist_log_out_nclust2 <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.log",
                      dist.mat = dist_matrix, metric = "nclust",
                      clust.fun = clust_fun_ward,
                      size = 0.2)
  dist_log_out_nclust2
}

if (requireNamespace('dbscan', quietly = TRUE)) {
  # Density-based clustering with dbscan
  dist_log_out_nclust3 <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.log",
                      dist.mat = dist_matrix, metric = "nclust",
                      clust.fun = clust_fun_dbscan,
                      size = 0.2)
  dist_log_out_nclust3
}

\donttest{
  if (requireNamespace('biotools', quietly = TRUE)) {
    # Tocher's sequential clustering
    dist_log_out_nclust4 <-
      allocate.distance(data = data, names = "genotypes",
                        group = "Cluster", method = "dist.log",
                        dist.mat = dist_matrix, metric = "nclust",
                        clust.fun = clust_fun_tocher,
                        size = 0.2)
    dist_log_out_nclust4
  }
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Diversity (Distance based) & Square root
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

## Mean
dist_sqrt_out_mean <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.sqrt",
                    dist.mat = dist_matrix, metric = "mean",
                    size = 0.2)
dist_sqrt_out_mean

## Median
dist_sqrt_out_median <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.sqrt",
                    dist.mat = dist_matrix, metric = "median",
                    size = 0.2)
dist_sqrt_out_median

## Maximum
dist_sqrt_out_max <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.sqrt",
                    dist.mat = dist_matrix, metric = "max",
                    size = 0.2)
dist_sqrt_out_max

## Range
dist_sqrt_out_range <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.sqrt",
                    dist.mat = dist_matrix, metric = "range",
                    size = 0.2)
dist_sqrt_out_range

## Mean nearest-neighbour distance
dist_sqrt_out_mnnd <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.sqrt",
                    dist.mat = dist_matrix, metric = "mnnd",
                    size = 0.2)
dist_sqrt_out_mnnd

## Minimum spanning tree length
dist_sqrt_out_mstl <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.sqrt",
                    dist.mat = dist_matrix, metric = "mstl",
                    size = 0.2)
dist_sqrt_out_mstl

\donttest{
  ## Mean distance to centroid
  dist_sqrt_out_mdc <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.sqrt",
                      dist.mat = dist_matrix, metric = "mdc",
                      size = 0.2)
  dist_sqrt_out_mdc

  ## Mean distance to median
  dist_sqrt_out_mdm <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.sqrt",
                      dist.mat = dist_matrix, metric = "mdm",
                      size = 0.2)
  dist_sqrt_out_mdm
}

## Number of clusters

### UPGMA with hclust
dist_sqrt_out_nclust1 <-
  allocate.distance(data = data, names = "genotypes",
                    group = "Cluster", method = "dist.sqrt",
                    dist.mat = dist_matrix, metric = "nclust",
                    clust.fun = clust_fun_upgma,
                    size = 0.2)
dist_sqrt_out_nclust1

# Ward's minimum variance with fastcluster
if (requireNamespace('fastcluster', quietly = TRUE)) {
  dist_sqrt_out_nclust2 <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.sqrt",
                      dist.mat = dist_matrix, metric = "nclust",
                      clust.fun = clust_fun_ward,
                      size = 0.2)
  dist_sqrt_out_nclust2
}

if (requireNamespace('dbscan', quietly = TRUE)) {
  # Density-based clustering with dbscan
  dist_sqrt_out_nclust3 <-
    allocate.distance(data = data, names = "genotypes",
                      group = "Cluster", method = "dist.sqrt",
                      dist.mat = dist_matrix, metric = "nclust",
                      clust.fun = clust_fun_dbscan,
                      size = 0.2)
  dist_sqrt_out_nclust3
}

\donttest{
  if (requireNamespace('biotools', quietly = TRUE)) {
    # Tocher's sequential clustering
    dist_sqrt_out_nclust4 <-
      allocate.distance(data = data, names = "genotypes",
                        group = "Cluster", method = "dist.sqrt",
                        dist.mat = dist_matrix, metric = "nclust",
                        clust.fun = clust_fun_tocher,
                        size = 0.2)
    dist_sqrt_out_nclust4
  }
}
\dontshow{\}) # examplesIf}
}
\references{
\insertAllCited
}
\seealso{
\code{\link[SampleCore]{allocate.basic}},
  \code{\link[SampleCore]{allocate.diversity}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select.random.R
\name{select.random}
\alias{select.random}
\title{Selection of Entries from Clusters/Groups by Random Sampling}
\usage{
select.random(data, names, group, alloc, always.selected = NULL)
}
\arguments{
\item{data}{The data as a data frame object. The data frame should possess
one row per individual and columns with the individual names and multiple
trait/character data.}

\item{names}{Name of column with the accession names as a character string.}

\item{group}{Name of column with the accession group/cluster names as a
character string.}

\item{alloc}{A named numeric vector specifying the number of entries to be
selected. Names should correspond to the levels of the "\code{"group"}
column, and values indicate the number of elements to be selected from each
level.}

\item{always.selected}{Names of accessions to be always included in the core
set as a character vector.}
}
\value{
A named list where each element contains the selected entry
  identifiers for a cluster/group.
}
\description{
Select entries from cluster/groups in the entire collection by random
sampling according to allocation specified.
}
\details{
For each cluster/group entries are selected randomly according to the
allocation provided
\insertCite{brown_core_1989,van_hintum_core_2000}{SampleCore}. Entries listed
as \code{always.selected} are mandatorily included in the selection. Warnings
are issued if requested allocation is smaller than the number of
always-selected entries in a cluster/group and/or when the cluster/group does
not contain enough remaining entries to fulfill the allocation.
}
\examples{
\dontshow{if (requireNamespace("cluster", quietly = TRUE)) withAutoprint(\{ # examplesIf}

library(cluster)

# Get data
data(cassava_EC_gp)

set.seed(123)
cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]

data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)
row.names(data) <- NULL

# Prepare inputs
counts <- c(I = 31, II = 31, III = 18, IV = 35, V = 40, VI = 17)

mand_accns <-
  c("TMe-2018", "TMe-801", "TMe-3191", "TMe-1830", "TMe-1790")

# Specify the seed
set.seed(123)

# Fetch selected accessions
sel_random_out <-
  select.random(data = data, names = "genotypes",
                group = "Cluster", alloc = counts,
                always.selected = mand_accns)

sel_random_out

# Get distance matrix - Only for visualization
quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
           "AVPW", "ARSR", "SRDM")
qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
          "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
          "PSTR")

# Convert qualitative data columns to factor
cassava_EC_gp[, qual] <- lapply(cassava_EC_gp[, qual], as.factor)

# Standardise quantitative data column
cassava_EC_gp[, quant] <- lapply(cassava_EC_gp[, quant], function(x) {
  scale(x)[, 1]
})

gp_vec <- setNames(as.character(data[, "Cluster"]), data[, "genotypes"])

# Get the Gower's distance matrix
dist_matrix <- daisy(x = cassava_EC_gp[, c(qual, quant)],
                     metric = "gower")

plot_dist(d = dist_matrix, method = "isomds",
          gp = gp_vec,
          highlight =  unlist(sel_random_out, use.names = FALSE))
\dontshow{\}) # examplesIf}
}
\references{
\insertAllCited
}
\seealso{
\code{\link[SampleCore]{select.distance}},
  \code{\link[SampleCore]{select.diversity}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allocate.basic.R
\name{allocate.basic}
\alias{allocate.basic}
\title{Allocation of Entries to be Selected from Clusters/Groups based on Size for
Core Collection Development}
\usage{
allocate.basic(
  data,
  names,
  group,
  method = c("const", "prop", "log", "sqrt"),
  log.base = exp(1),
  size
)
}
\arguments{
\item{data}{The data as a data frame object. The data frame should possess
one row per individual and columns with the individual names and multiple
trait/character data.}

\item{names}{Name of column with the accession names as a character string.}

\item{group}{Name of column with the accession group/cluster names as a
character string.}

\item{method}{The allocation method. Either \code{"const"} for constant or
\code{"prop"} for proportional or \code{"log"} for logarithmic or
\code{"sqrt"} for square root allocation.}

\item{log.base}{The logarithm base to be used for logarithmic method of
sampling. Default is \code{exp(1)}.}

\item{size}{The desired core set size proportion.}
}
\value{
A named numeric vector specifying the number of entries to be
  selected from each cluster/group. The vector names correspond to the levels
  of the "\code{"group"} column, and values indicate the number of elements
  to be selected from each level.
}
\description{
Estimate the number of entries to be allocated from each cluster/group in the
entire collection to construct a core collection on the basis of
cluster/group size. The following strategies are implemented. \loadmathjax
\itemize{
\item{Constant}
\item{Proportional}
\item{Logarithmic}
\item{Square root}}
The different methods to determine the number of entries from each group or
clusters implemented in \code{allocate.basic} are as follows.
}
\details{
These are different methods which estimate the number of entries only on the
basis of total number of entries in each cluster/group.

\insertCite{brown_core_1989;textual}{SampleCore} proposed the constant (C),
proportional (P) and logarithmic (L) methods and later a similar square root
method was proposed by
\insertCite{huaman_selecting_1999;textual}{SampleCore}.

  \subsection{Constant method}{From an entire collection of size \mjseqn{N},
  to construct a core set of sample size \mjseqn{n}, the number of entries to
  be selected from the \mjseqn{i}th group among \mjseqn{1 \cdots g} groups
  (\mjseqn{n_{i}}) is estimated as below.

  \mjsdeqn{n_{i} = \frac{n}{g} \times N}

  }

  \subsection{Proportional method}{Here the number of entries to be selected
  is proportional to the cluster/group size (\mjseqn{N_{i}}) as
  below.

  \mjsdeqn{n_{i} = n \times \frac{N_{i}}{\sum_{i=1}^{g}N_{i}}}

  \mjsdeqn{n_{i} = n \times \frac{N_{i}}{N}}

  }

  \subsection{Logarithmic method}{Here the number of entries to be selected
  is proportional to the logarithm of the cluster/group size
  (\mjseqn{N_{i}}) as below.

  \mjsdeqn{n_{i} = n \times
  \frac{\log{(N_{i})}}{\sum_{i=1}^{g}\log{(N_{i})}}}

  }

  \subsection{Square root method}{Here the number of entries to
  be selected is proportional to the square root of the cluster/group size
  (\mjseqn{N_{i}}) as below.

  \mjsdeqn{n_{i} = n \times \frac{\sqrt{N_{i}}}{\sum_{i=1}^{g}\sqrt{N_{i}}}}

  }
}
\examples{
# Get data
data("cassava_EC_gp")

set.seed(123)
cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]

data <- cassava_EC_gp

data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)
row.names(data) <- NULL

# Constant allocation
const_out <-
  allocate.basic(data = data, names = "genotypes",
                 group = "Cluster", method = "const",
                 size = 0.2)
const_out

# Proportional allocation
prop_out <-
  allocate.basic(data = data, names = "genotypes",
                 group = "Cluster", method = "prop",
                 size = 0.2)
prop_out

# Logarithmic allocation
log_out <-
  allocate.basic(data = data, names = "genotypes",
                 group = "Cluster", method = "log",
                 size = 0.2)
log_out

# Square root allocation
sqrt_out <-
  allocate.basic(data = data, names = "genotypes",
                 group = "Cluster", method = "sqrt",
                 size = 0.2)
sqrt_out

}
\references{
\insertAllCited
}
\seealso{
\code{\link[SampleCore]{allocate.distance}},
  \code{\link[SampleCore]{allocate.diversity}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select.diversity.R
\name{select.diversity}
\alias{select.diversity}
\title{Selection of Entries from Clusters/Groups on the basis of Optimized Diversity}
\usage{
select.diversity(
  data,
  names,
  group,
  alloc,
  qualitative,
  always.selected = NULL,
  div.index = c("richness", "shannon", "simpson", "mcintosh"),
  shannon.base = exp(1),
  div.fun = NULL,
  metric = c("mean", "pooled"),
  search = c("random", "greedy"),
  local.search = c("best.improvement", "first.improvement"),
  n.iter = 1000,
  max.iter = 30
)
}
\arguments{
\item{data}{The data as a data frame object. The data frame should possess
one row per individual and columns with the individual names and multiple
trait/character data.}

\item{names}{Name of column with the accession names as a character string.}

\item{group}{Name of column with the accession group/cluster names as a
character string.}

\item{alloc}{A named numeric vector specifying the number of entries to be
selected. Names should correspond to the levels of the "\code{"group"}
column, and values indicate the number of elements to be selected from each
level.}

\item{qualitative}{Name of columns with the qualitative traits as a character
vector.}

\item{always.selected}{Names of accessions to be always included in the core
set as a character vector.}

\item{div.index}{The diversity index to be used to estimate within
cluster/group diversity.}

\item{shannon.base}{The logarithm base to be used for estimation of Shannon
diversity index. Default is \code{exp(1)}.}

\item{div.fun}{A function to estimate diversity index from a factor vector of
qualitative trait data.}

\item{metric}{The metric to be computed from the diversity index. Either
\code{"pooled"} or \code{"mean"}.}

\item{search}{Character string specifying the search strategy used to find
the subset with the highest diversity score. Either \code{"random"}
(default) or \code{"greedy"} (See \strong{Details}).}

\item{local.search}{Character string specifying the local search strategy
used in the 1-opt improvement phase of the greedy search (\code{search =
"greedy"}). Either \code{"best.improvement"} (default) or
\code{"first.improvement"}. Ignored when \code{search = "random"}.}

\item{n.iter}{Integer specifying the number of random candidate subsets
generated per group to optimze the diversity for random search
(\code{search = "random"}).}

\item{max.iter}{The maximum number of 1-opt passes for greedy search
(\code{search = "greedy"}).}
}
\value{
A named list where each element contains the selected entry
  identifiers for a cluster/group.
}
\description{
Select entries from cluster/groups in the entire collection which form a
subset with the highest trait diversity according to a either pooled or mean
diversity index estimate. \loadmathjax
}
\details{
To identify subsets with highest diversity estimates, the following
strategies are available. These strategies are similar to the "Maximization"
or M strategy of \insertCite{schoen_conservation_1993;textual}{SampleCore}.

 \subsection{Random search / Monte Carlo Method}{For each cluster/group,
 multiple candidate subsets are sampled randomly and the subset with the
 highest trait diversity according to either pooled or mean diversity index
 estimate is retained. The quality of the solution improves with increasing
 \code{n.iter} but is not guaranteed to find the global optimum
 \insertCite{anatoly_zhigljavsky_stochastic_2008}{SampleCore}.
 }

 \subsection{Greedy search with 1-opt}{This method builds a solution
 incrementally by adding the accession that maximises the diversity score at
 each step, starting from the \code{always.selected} accessions (or a single
 randomly drawn accession when there are no accessions specified in
 \code{always.selected}) present in the particular cluster/group
 \insertCite{nemhauser_analysis_1978,fisher_analysis_1978,cormen_introduction_2022}{SampleCore}.
 The 'greedy' solution is then refined by a 1-opt local search controlled by
 \code{local.search} and \code{max.iter}
 \insertCite{lin_computer_1965}{SampleCore}. Greedy search is deterministic
 given a fixed \code{always.selected} set; when there are no accessions
 specified in \code{always.selected} present in the particular cluster/group
 results may vary across runs due to the random initialisation.

 \code{local.search = "best.improvement"} scans all possible single swaps
 in each pass and applies the one yielding the greatest improvement before
 restarting. his guarantees the steepest ascent at each pass but requires
 evaluating all \mjseqn{k \times (n - k)} swap pairs per pass, where
 \mjseqn{k} is the number of swappable accessions and \mjseqn{n - k} is the
 size of the candidate pool
 \insertCite{papadimitriou_combinatorial_1998}{SampleCore}.

 \code{local.search = "first.improvement"} applies the first swap that
 improves the score and immediately restarts the search. This typically
 requires fewer score evaluations per pass and converges faster, but may
 find a different local optimum than \code{"best.improvement"}
 \insertCite{papadimitriou_combinatorial_1998}{SampleCore}.

 Both strategies terminate when no improving swap exists (local optimum)
 or when \code{max.iter} passes have been completed.
 }

Entries listed as \code{always.selected} are mandatorily included in the
selection. Warnings are issued if requested allocation is smaller than the
number of always-selected entries in a cluster/group and/or when the
cluster/group does not contain enough remaining entries to fulfill the
allocation.
}
\examples{

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

library(cluster)
library(ggplot2)

data(cassava_EC_gp)

set.seed(123)
cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]

data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)

quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW", "AVPW",
           "ARSR", "SRDM")
qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
          "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
          "PSTR")

# Convert qualitative data columns to factor
data[, qual] <- lapply(data[, qual], as.factor)

# Convert quantitative data columns to qualitative scores
quant_to_score5 <- function(x) {

  brks <- unique( quantile(x,
                           probs = seq(0, 1, 0.2),
                           na.rm = TRUE))
  cut(x, breaks = brks,
      include.lowest = TRUE,
      labels = seq_len(length(brks) - 1))
}

data[, quant] <- lapply(data[, quant], quant_to_score5)

traits <- c(quant, qual)

# Prepare inputs
counts <- c(I = 31, II = 31, III = 18, IV = 35, V = 40, VI = 17)

mand_accns <-
  c("TMe-2018", "TMe-801", "TMe-3191", "TMe-1830", "TMe-1790")

# Get distance matrix - Only for visualization

# Convert qualitative data columns to factor
cassava_EC_gp[, qual] <- lapply(cassava_EC_gp[, qual], as.factor)

# Standardise quantitative data column
cassava_EC_gp[, quant] <- lapply(cassava_EC_gp[, quant], function(x) {
  scale(x)[, 1]
})

gp_vec <- setNames(as.character(data[, "Cluster"]), data[, "genotypes"])

# Get the Gower's distance matrix
dist_matrix <- daisy(x = cassava_EC_gp[, c(qual, quant)],
                     metric = "gower")

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Custom Diversity functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

div_fun_brillouin <- function(x) {
  n <- tabulate(x)
  n <- n[n > 0]
  N <- sum(n)
  if (N <= 1) {
    return(0)
  }
  (lgamma(N + 1) - sum(lgamma(n + 1)))/N
}

div_fun_margalef <- function(x) {
  tab <- tabulate(x)
  tab <- tab[tab > 0]
  S <- length(tab)
  N <- length(x)
  if (N <= 1) {
    return(0)
  }
  (S - 1)/log(N)
}

\donttest{
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Random search
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  # Mean richness
  randomsel_mean_richness <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "richness",
                     metric = "mean", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_mean_richness

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_mean_richness,
                                use.names = FALSE)) +
    labs(title = "Random search", subtitle = "Mean richness")

  # Pooled richness
  randomsel_sum_richness <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "richness",
                     metric = "pooled", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_sum_richness

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_sum_richness,
                                use.names = FALSE)) +
    labs(title = "Random search", subtitle = "Pooled richness")

  # Mean Shannon-Weaver diversity index
  randomsel_mean_shannon <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "shannon",
                     metric = "mean", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_mean_shannon

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_mean_shannon,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Mean Shannon-Weaver diversity index")

  # Pooled Shannon-Weaver diversity index
  randomsel_sum_shannon <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "shannon",
                     metric = "pooled", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_sum_shannon

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_sum_shannon,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Pooled Shannon-Weaver diversity index")

  # Mean Gini-Simpson diversity index
  randomsel_mean_simpson <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "simpson",
                     metric = "mean", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_mean_simpson

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_mean_simpson,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Mean Gini-Simpson diversity index")

  # Pooled Gini-Simpson diversity index
  randomsel_sum_simpson <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "simpson",
                     metric = "pooled", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_sum_simpson

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_sum_simpson,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Pooled Gini-Simpson diversity index")

  # Mean McIntosh diversity index
  randomsel_mean_mcintosh <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "mcintosh",
                     metric = "pooled", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_mean_mcintosh

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_mean_mcintosh,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Mean McIntosh diversity index")

  # Pooled McIntosh diversity index
  randomsel_sum_mcintosh <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "mcintosh",
                     metric = "pooled", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_sum_mcintosh

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_sum_mcintosh,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Pooled McIntosh diversity index")

  # Mean Brillouin diversity index
  randomsel_mean_brillouin <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.fun = div_fun_brillouin,
                     metric = "mean", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_mean_brillouin

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_mean_brillouin,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Mean Brillouin diversity index")

  # Pooled Brillouin diversity index
  randomsel_sum_brillouin <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.fun = div_fun_brillouin,
                     metric = "pooled", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_sum_brillouin

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_sum_brillouin,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Pooled Brillouin diversity index")


  # Mean Margalef's richness index
  randomsel_mean_margalef <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.fun = div_fun_margalef,
                     metric = "mean", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_mean_margalef

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_mean_margalef,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Mean Margalef's diversity index")

  # Pooled Margalef's richness index
  randomsel_sum_margalef <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.fun = div_fun_margalef,
                     metric = "pooled", search = "random", local.search = NULL,
                     n.iter = 50)
  randomsel_sum_margalef

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(randomsel_sum_margalef,
                                use.names = FALSE)) +
    labs(title = "Random search",
         subtitle = "Pooled Margalef's diversity index")

  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Greedy search with 1-opt best improvement
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  # Mean richness
  greedysel_best_mean_richness <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "richness",
                     metric = "mean", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_mean_richness

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_mean_richness,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Mean richness")

  # Pooled richness
  greedysel_best_sum_richness <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "richness",
                     metric = "pooled", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_sum_richness

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_sum_richness,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Pooled richness")

  # Mean Shannon-Weaver diversity index
  greedysel_best_mean_shannon <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "shannon",
                     metric = "mean", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_mean_shannon

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_mean_shannon,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Mean Shannon-Weaver diversity index")

  # Pooled Shannon-Weaver diversity index
  greedysel_best_sum_shannon <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "shannon",
                     metric = "pooled", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_sum_shannon

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_sum_shannon,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Pooled Shannon-Weaver diversity index")

  # Mean Gini-Simpson diversity index
  greedysel_best_mean_simpson <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "simpson",
                     metric = "mean", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_mean_simpson

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_mean_simpson,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Mean Gini-Simpson diversity index")

  # Pooled Gini-Simpson diversity index
  greedysel_best_sum_simpson <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "simpson",
                     metric = "pooled", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_sum_simpson

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_sum_simpson,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Pooled Gini-Simpson diversity index")

  # Mean McIntosh diversity index
  greedysel_best_mean_mcintosh <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "mcintosh",
                     metric = "pooled", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_mean_mcintosh

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_mean_mcintosh,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Mean McIntosh diversity index")

  # Pooled McIntosh diversity index
  greedysel_best_sum_mcintosh <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "mcintosh",
                     metric = "pooled", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_sum_mcintosh

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_sum_mcintosh,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Pooled McIntosh diversity index")

  # Mean Brillouin diversity index
  greedysel_best_mean_brillouin <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns,
                     div.fun = div_fun_brillouin,
                     metric = "mean", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_mean_brillouin

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_mean_brillouin,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Mean Brillouin diversity index")

  # Pooled Brillouin diversity index
  greedysel_best_sum_brillouin <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns,
                     div.fun = div_fun_brillouin,
                     metric = "pooled", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_sum_brillouin

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_sum_brillouin,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Pooled Brillouin diversity index")

  # Mean Margalef's richness index
  greedysel_best_mean_margalef <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns,
                     div.fun = div_fun_margalef,
                     metric = "mean", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_mean_margalef

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_mean_margalef,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Mean Margalef's diversity index")

  # Pooled Margalef's richness index
  greedysel_best_sum_margalef <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns,
                     div.fun = div_fun_margalef,
                     metric = "pooled", search = "greedy",
                     local.search = "best.improvement",max.iter = 3)
  greedysel_best_sum_margalef

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_best_sum_margalef,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt best improvement",
         subtitle = "Pooled Margalef's diversity index")

  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Greedy search with 1-opt first improvement
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  # Mean richness
  greedysel_first_mean_richness <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "richness",
                     metric = "mean", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_mean_richness

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_mean_richness,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Mean richness")

  # Pooled richness
  greedysel_first_sum_richness <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "richness",
                     metric = "pooled", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_sum_richness

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_sum_richness,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Pooled richness")

  # Mean Shannon-Weaver diversity index
  greedysel_first_mean_shannon <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "shannon",
                     metric = "mean", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_mean_shannon

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_mean_shannon,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Mean Shannon-Weaver diversity index")

  # Pooled Shannon-Weaver diversity index
  greedysel_first_sum_shannon <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "shannon",
                     metric = "pooled", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_sum_shannon

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_sum_shannon,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Pooled Shannon-Weaver diversity index")

  # Mean Gini-Simpson diversity index
  greedysel_first_mean_simpson <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "simpson",
                     metric = "mean", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_mean_simpson

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_mean_simpson,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Mean Gini-Simpson diversity index")

  # Pooled Gini-Simpson diversity index
  greedysel_first_sum_simpson <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "simpson",
                     metric = "pooled", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_sum_simpson

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_sum_simpson,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Pooled Gini-Simpson diversity index")

  # Mean McIntosh diversity index
  greedysel_first_mean_mcintosh <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "mcintosh",
                     metric = "pooled", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_mean_mcintosh

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_mean_mcintosh,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Mean McIntosh diversity index")

  # Pooled McIntosh diversity index
  greedysel_first_sum_mcintosh <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns, div.index = "mcintosh",
                     metric = "pooled", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_sum_mcintosh

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_sum_mcintosh,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Pooled McIntosh diversity index")

  # Mean Brillouin diversity index
  greedysel_first_mean_brillouin <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns,
                     div.fun = div_fun_brillouin,
                     metric = "mean", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_mean_brillouin

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_mean_brillouin,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Mean Brillouin diversity index")

  # Pooled Brillouin diversity index
  greedysel_first_sum_brillouin <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns,
                     div.fun = div_fun_brillouin,
                     metric = "pooled", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_sum_brillouin

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_sum_brillouin,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Pooled Brillouin diversity index")

  # Mean Margalef's richness index
  greedysel_first_mean_margalef <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns,
                     div.fun = div_fun_margalef,
                     metric = "mean", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_mean_margalef

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_mean_margalef,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Mean Margalef's richness index")

  # Pooled Margalef's richness index
  greedysel_first_sum_margalef <-
    select.diversity(data = data, names = "genotypes", group = "Cluster",
                     alloc = counts, qualitative = traits,
                     always.selected = mand_accns,
                     div.fun = div_fun_margalef,
                     metric = "pooled", search = "greedy",
                     local.search = "first.improvement",max.iter = 3)
  greedysel_first_sum_margalef

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(greedysel_first_sum_margalef,
                                use.names = FALSE)) +
    labs(title = "Greed search | 1-opt first improvement",
         subtitle = "Pooled Margalef's richness index")
}

}
\references{
\insertAllCited
}
\seealso{
\code{\link[SampleCore]{select.random}},
  \code{\link[SampleCore]{select.distance}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_dist.R
\name{plot_dist}
\alias{plot_dist}
\title{Plot a distance matrix as a 2D projection}
\usage{
plot_dist(
  d,
  method = c("cmds", "isomds", "tsne"),
  highlight = NULL,
  gp = NULL,
  point.alpha = 0.8
)
}
\arguments{
\item{d}{A distance matrix of class \code{\link[stats]{dist}}. Labels must be
set (i.e. \code{labels(d)} must not be \code{NULL}). Duplicate labels are
not permitted.}

\item{method}{Character string specifying the dimensionality-reduction
method. One of:
\describe{
  \item{\code{"cmds"}}{Classical (metric) Multidimensional Scaling via
    \code{\link[stats]{cmdscale}}. This is the default.}
  \item{\code{"isomds"}}{Non-metric (isotonic) MDS via
    \code{\link[MASS]{isoMDS}}. Automatically falls back to
    \code{"cmds"} with a message when \code{n < 3}.}
  \item{\code{"tsne"}}{t-distributed Stochastic Neighbour Embedding via
    \code{\link[Rtsne]{Rtsne}}. Perplexity is set automatically to
    \code{min(30, floor((n - 1) / 3))}.}
}}

\item{highlight}{Optional character vector of labels to highlight in the
plot. Matching identifiers are plotted in \strong{red}; all others in
black. \code{NULL} (default) disables highlighting. Every value must be
present in \code{labels(d)}.}

\item{gp}{Optional named character vector mapping labels to group names
(\code{names(gp)} = labels, values = group names). When supplied, the plot
is split into one facet panel per group via
\code{\link[ggplot2]{facet_wrap}}. The set of names must match
\code{labels(d)} exactly. \code{NULL} (default) produces a single panel.}

\item{point.alpha}{Alpha transparency value for points.}
}
\value{
A \code{\link[ggplot2]{ggplot}} object. The plot can be further
  customised with standard \pkg{ggplot2} additions before printing or saving.
}
\description{
Reduces a distance matrix to two dimensions using Classical MDS, Isotonic
MDS, or t-SNE, and returns a \code{ggplot2} scatter plot in which proximity
reflects similarity. Points can optionally be highlighted or split into facet
panels by group.
}
\examples{
# Basic usage with the built-in eurodist dataset
plot_dist(eurodist)

# Non-metric MDS with two highlighted cities
plot_dist(eurodist, method = "isomds",
          highlight = c("Madrid", "Rome"))

# Classical MDS split by a user-defined grouping
regions <-
  c(Athens = "South",  Barcelona = "South", Brussels = "North",
    Calais = "North",  Cherbourg = "North", Cologne = "North",
    Copenhagen = "North", Geneva = "South", Gibraltar = "South",
    Hamburg = "North", `Hook of Holland` = "North", Lisbon = "South",
    Lyons = "South",  Madrid = "South",  Marseilles = "South",
    Milan = "South",  Munich = "North",  Paris = "North",
    Rome = "South",   Stockholm = "North", Vienna = "North")

plot_dist(eurodist, method = "cmds", gp = regions,
          highlight = c("Madrid", "Cherbourg", "Rome", "Brussels"))

}
\seealso{
\code{\link[stats]{cmdscale}}, \code{\link[MASS]{isoMDS}},
\code{\link[Rtsne]{Rtsne}}, \code{\link[ggplot2]{ggplot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allocate.diversity.R
\name{allocate.diversity}
\alias{allocate.diversity}
\title{Allocation of Entries to be Selected from Clusters/Groups based on Diversity
Index Estimates for Core Collection Development}
\usage{
allocate.diversity(
  data,
  names,
  group,
  qualitative,
  method = c("div", "div.prop", "div.sqrt", "div.log"),
  div.index = c("richness", "shannon", "simpson", "mcintosh"),
  shannon.base = exp(1),
  div.fun = NULL,
  log.base = exp(1),
  metric = c("pooled", "mean"),
  size
)
}
\arguments{
\item{data}{The data as a data frame object. The data frame should possess
one row per individual and columns with the individual names and multiple
trait/character data.}

\item{names}{Name of column with the accession names as a character string.}

\item{group}{Name of column with the accession group/cluster names as a
character string.}

\item{qualitative}{Name of columns with the qualitative traits as a character
vector.}

\item{method}{The allocation method. Either \code{"div"} for constant or
\code{"div.prop"} for proportional or \code{"div.log"} for logarithmic or
\code{"div.sqrt"} for square root allocation.}

\item{div.index}{The diversity index to be used to estimate within
cluster/group diversity.}

\item{shannon.base}{The logarithm base to be used for estimation of Shannon
diversity index. Default is \code{exp(1)}.}

\item{div.fun}{A function to estimate diversity index from a factor vector of
qualitative trait data.}

\item{log.base}{The logarithm base to be used for logarithmic method of
sampling. Default is \code{exp(1)}.}

\item{metric}{The metric to be computed from the diversity index. Either
\code{"pooled"} or \code{"mean"}.}

\item{size}{The desired core set size proportion.}
}
\value{
A named numeric vector specifying the number of entries to be
  selected from each cluster/group. The vector names correspond to the levels
  of the "\code{"group"} column, and values indicate the number of elements
  to be selected from each level.
}
\description{
Estimate the number of entries to be allocated from each cluster/group in the
entire collection to construct a core collection on the basis of different
metrics computed from within cluster/group diversity index estimates. The
following strategies are implemented. \loadmathjax
\itemize{
\item{Diversity}
\item{Diversity & Proportional}
\item{Diversity & Logarithmic}
\item{Diversity & Square root}}
}
\section{Details}{


  The number of entries to be chosen from each cluster is estimated either on
  the basis of diversity of entries within that cluster/group alone or in
  combination with the size of the cluster/group (See
  \strong{\code{Methods}}).

  There are several methods proposed on the basis of diversity indices such
  as genetic multiplicity (G) dependent method based on the range of genetic
  diversity \insertCite{yonezawa_sampling_1995}{SampleCore}, H strategy based
  on Nei's gene diversity \insertCite{nei_analysis_1973}{SampleCore} and a
  method based on the pooled Shannon diversity index
  \insertCite{bisht_assessment_1999,mahajan_sampling_1999}{SampleCore}.
  Similarly, measures such as expected proportion of heterozygous loci per
  individual and effective number of alleles have also been employed as a
  diversity measure for determining sample size
  \insertCite{franco_sampling_2006}{SampleCore}.

  The within-cluster/group diversity is estimated as either pooled or mean
  value of cluster/group-wise diversity indices. The following diversity
  indices are implemented in this function.

  \itemize{
   \item{Shannon or Shannon-Weaver or Shannon-Wiener Diversity Index or
   Shannon entropy
   (\mjseqn{H}) \insertCite{shannon_mathematical_1949,peet_measurement_1974}{SampleCore}}
   \item{Simpson's Index of Diversity or Gini's Diversity Index or
   Gini-Simpson Index or Nei's Diversity Index or Nei's Variation Index
   (\mjseqn{D}) or Hurlbert’s probability of interspecific encounter
   (\mjseqn{PIE}) \insertCite{gini_variabilita_1912,gini_variabilita_1912-1,greenberg_measurement_1956,berger_diversity_1970,hurlbert_nonconcept_1971,nei_analysis_1973,peet_measurement_1974}{SampleCore}}
   \item{McIntosh Diversity Index (\mjseqn{D_{Mc}})
   \insertCite{mcintosh_index_1967,peet_measurement_1974}{DiversityStats}}}
}

\section{Methods}{


  \subsection{Diversity method}{From an entire collection of size \mjseqn{N},
  to construct a core set of sample size \mjseqn{n}, the number of entries to
  be selected from the \mjseqn{i}th group among \mjseqn{1 \cdots g} groups
  (\mjseqn{n_{i}}) is estimated as below.

   \mjsdeqn{n_{i} = n \times \frac{D_{i}}{\sum_{i=1}^{g}D_{i}}}

   Where, \mjseqn{D_{i}} is a measure of the extent of diversity present in
   the \mjseqn{i}th cluster.

   }

   \subsection{Diversity and proportional method}{Here the number of entries
   to be selected is proportional to the diversity of the cluster/group
   (\mjseqn{D_{i}}) weighted by the the cluster/group size (\mjseqn{N_{i}}).

    \mjsdeqn{n_{i} = n \times \frac{N_{i}D_{i}}{\sum_{i=1}^{g}N_{i}D_{i}}}

   }

   \subsection{Diversity and logarithmic method}{Here the number of entries
   to be selected is proportional to the diversity of the cluster/group
   (\mjseqn{D_{i}}) weighted by the logarithm of the cluster/group size
   (\mjseqn{N_{i}}).

   \mjsdeqn{n_{i} = n \times
   \frac{\log(N_{i})D_{i}}{\sum_{i=1}^{g}\log(N_{i})D_{i}}}

   }

   \subsection{Diversity and square root method}{Here the number of entries
   to be selected is proportional to the diversity of the cluster/group
   (\mjseqn{D_{i}}) weighted by the square root of the cluster/group size
   (\mjseqn{N_{i}}).

   \mjsdeqn{n_{i} = n \times
   \frac{\sqrt{N_{i}}D_{i}}{\sum_{i=1}^{g}\sqrt{N_{i}}D_{i}}}

   }
}

\examples{
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# Get data
data("cassava_EC_gp")

set.seed(123)
cassava_EC_gp <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]

data <- cbind(genotypes = rownames(cassava_EC_gp), cassava_EC_gp)
row.names(data) <- NULL

# Column names of traits
quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW",
           "AVPW", "ARSR", "SRDM")
qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
          "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
          "PSTR")

# Convert qualitative data columns to factor
data[, qual] <- lapply(data[, qual], as.factor)

# Convert quantitative data columns to qualitative scores
quant_to_score5 <- function(x) {

  brks <- unique( quantile(x,
                           probs = seq(0, 1, 0.2),
                           na.rm = TRUE))
  cut(x, breaks = brks,
      include.lowest = TRUE,
      labels = seq_len(length(brks) - 1))
}

data[, quant] <- lapply(data[, quant], quant_to_score5)

traits <- c(quant, qual)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Custom diversity index functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

div_fun_brillouin <- function(x) {
  n <- tabulate(x)
  n <- n[n > 0]
  N <- sum(n)
  if (N <= 1) {
    return(0)
  }
  (lgamma(N + 1) - sum(lgamma(n + 1)))/N
}

div_fun_margalef <- function(x) {
  tab <- tabulate(x)
  tab <- tab[tab > 0]
  S <- length(tab)
  N <- length(x)
  if (N <= 1) {
    return(0)
  }
  (S - 1)/log(N)
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Diversity allocation
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

## Shannon-Weaver Diversity Index
div_out_shannon1 <-
  allocate.diversity(data = data, names = "genotypes",
                    group = "Cluster",
                    qualitative = traits,
                    method = "div",
                    div.index = "shannon", metric = "pooled",
                    size = 0.2)
div_out_shannon1

div_out_shannon2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.index = "shannon", metric = "mean",
                     size = 0.2)
div_out_shannon2

##  Gini-Simpson Index
div_out_simpson1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.index = "simpson", metric = "pooled",
                     size = 0.2)
div_out_simpson1

div_out_simpson2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.index = "simpson", metric = "mean",
                     size = 0.2)
div_out_simpson2

## McIntosh Diversity Index
div_out_mcintosh1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.index = "mcintosh", metric = "pooled",
                     size = 0.2)
div_out_mcintosh1

div_out_mcintosh2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.index = "mcintosh", metric = "mean",
                     size = 0.2)
div_out_mcintosh2

## Richness
div_out_richness1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.index = "richness", metric = "pooled",
                     size = 0.2)
div_out_richness1

div_out_richness2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.index = "richness", metric = "mean",
                     size = 0.2)
div_out_richness2

## Brillouin Diversity Index
div_out_brillouin1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.fun = div_fun_brillouin, metric = "pooled",
                     size = 0.2)
div_out_brillouin1

div_out_brillouin2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.fun = div_fun_brillouin, metric = "mean",
                     size = 0.2)
div_out_brillouin2

## Margalef's richness Index
div_out_margalef1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.fun = div_fun_margalef, metric = "pooled",
                     size = 0.2)
div_out_margalef1

div_out_margalef2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div",
                     div.fun = div_fun_margalef, metric = "mean",
                     size = 0.2)
div_out_margalef2


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Diversity allocation & Proportional
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

## Shannon-Weaver Diversity Index
dist_prop_out_shannon1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.index = "shannon", metric = "pooled",
                     size = 0.2)
dist_prop_out_shannon1

dist_prop_out_shannon2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.index = "shannon", metric = "mean",
                     size = 0.2)
dist_prop_out_shannon2

##  Gini-Simpson Index
dist_prop_out_simpson1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.index = "simpson", metric = "pooled",
                     size = 0.2)
dist_prop_out_simpson1

dist_prop_out_simpson2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.index = "simpson", metric = "mean",
                     size = 0.2)
dist_prop_out_simpson2

## McIntosh Diversity Index
dist_prop_out_mcintosh1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.index = "mcintosh", metric = "pooled",
                     size = 0.2)
dist_prop_out_mcintosh1

dist_prop_out_mcintosh2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.index = "mcintosh", metric = "mean",
                     size = 0.2)
dist_prop_out_mcintosh2

## Richness
div_out_richness1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "richness", metric = "pooled",
                     size = 0.2)
div_out_richness1

div_out_richness2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "richness", metric = "mean",
                     size = 0.2)
div_out_richness2

## Brillouin Diversity Index
div_out_brillouin1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.fun = div_fun_brillouin, metric = "pooled",
                     size = 0.2)
div_out_brillouin1

div_out_brillouin2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.fun = div_fun_brillouin, metric = "mean",
                     size = 0.2)
div_out_brillouin2

## Margalef's richness Index
div_out_margalef1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.fun = div_fun_margalef, metric = "pooled",
                     size = 0.2)
div_out_margalef1

div_out_margalef2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.prop",
                     div.fun = div_fun_margalef, metric = "mean",
                     size = 0.2)
div_out_margalef2

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Diversity allocation & Logarithmic
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

## Shannon-Weaver Diversity Index
dist_log_out_shannon1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "shannon", metric = "pooled",
                     size = 0.2)
dist_log_out_shannon1

dist_log_out_shannon2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "shannon", metric = "mean",
                     size = 0.2)
dist_log_out_shannon2

##  Gini-Simpson Index
dist_log_out_simpson1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "simpson", metric = "pooled",
                     size = 0.2)
dist_log_out_simpson1

dist_log_out_simpson2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "simpson", metric = "mean",
                     size = 0.2)
dist_log_out_simpson2

## McIntosh Diversity Index
dist_log_out_mcintosh1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "mcintosh", metric = "pooled",
                     size = 0.2)
dist_log_out_mcintosh1

dist_log_out_mcintosh2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "mcintosh", metric = "mean",
                     size = 0.2)
dist_log_out_mcintosh2

## Richness
div_out_richness1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "richness", metric = "pooled",
                     size = 0.2)
div_out_richness1

div_out_richness2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.index = "richness", metric = "mean",
                     size = 0.2)
div_out_richness2

## Brillouin Diversity Index
div_out_brillouin1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.fun = div_fun_brillouin, metric = "pooled",
                     size = 0.2)
div_out_brillouin1

div_out_brillouin2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.fun = div_fun_brillouin, metric = "mean",
                     size = 0.2)
div_out_brillouin2

## Margalef's richness Index
div_out_margalef1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.fun = div_fun_margalef, metric = "pooled",
                     size = 0.2)
div_out_margalef1

div_out_margalef2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.log",
                     div.fun = div_fun_margalef, metric = "mean",
                     size = 0.2)
div_out_margalef2

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Diversity allocation & Square root
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

## Shannon-Weaver Diversity Index
dist_sqrt_out_shannon1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.index = "shannon", metric = "pooled",
                     size = 0.2)
dist_sqrt_out_shannon1

dist_sqrt_out_shannon2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.index = "shannon", metric = "mean",
                     size = 0.2)
dist_sqrt_out_shannon2

##  Gini-Simpson Index
dist_sqrt_out_simpson1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.index = "simpson", metric = "pooled",
                     size = 0.2)
dist_sqrt_out_simpson1

dist_sqrt_out_simpson2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.index = "simpson", metric = "mean",
                     size = 0.2)
dist_sqrt_out_simpson2

## McIntosh Diversity Index
dist_sqrt_out_mcintosh1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.index = "mcintosh", metric = "pooled",
                     size = 0.2)
dist_sqrt_out_mcintosh1

dist_sqrt_out_mcintosh2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.index = "mcintosh", metric = "mean",
                     size = 0.2)
dist_sqrt_out_mcintosh2

## Richness
div_out_richness1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.index = "richness", metric = "pooled",
                     size = 0.2)
div_out_richness1

div_out_richness2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.index = "richness", metric = "mean",
                     size = 0.2)
div_out_richness2

## Brillouin Diversity Index
div_out_brillouin1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.fun = div_fun_brillouin, metric = "pooled",
                     size = 0.2)
div_out_brillouin1

div_out_brillouin2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.fun = div_fun_brillouin, metric = "mean",
                     size = 0.2)
div_out_brillouin2

## Margalef's richness Index
div_out_margalef1 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.fun = div_fun_margalef, metric = "pooled",
                     size = 0.2)
div_out_margalef1

div_out_margalef2 <-
  allocate.diversity(data = data, names = "genotypes",
                     group = "Cluster",
                     qualitative = traits,
                     method = "div.sqrt",
                     div.fun = div_fun_margalef, metric = "mean",
                     size = 0.2)
div_out_margalef2

}
\references{
\insertAllCited
}
\seealso{
\code{\link[SampleCore]{allocate.basic}},
  \code{\link[SampleCore]{allocate.distance}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select.distance.R
\name{select.distance}
\alias{select.distance}
\title{Selection of Entries from Clusters/Groups on the basis of Genetic Distances}
\usage{
select.distance(
  data,
  names,
  group,
  alloc,
  dist.mat,
  always.selected = NULL,
  method = c("mean.medoid", "median.medoid", "nearest.centroid", "nearest.median",
    "mean.peripheral", "median.peripheral", "eccentricity", "farness.centrality",
    "kennard.stone", "duplex", "honigs", "farthest.point", "nearest.neighbour", "naes",
    "optim.medoid", "hclust.random", "hclust.medoid"),
  hclust.method = c("average", "single", "complete", "ward.D", "mcquitty", "median",
    "centroid", "ward.D2")
)
}
\arguments{
\item{data}{The data as a data frame object. The data frame should possess
one row per individual and columns with the individual names and multiple
trait/character data.}

\item{names}{Name of column with the accession names as a character string.}

\item{group}{Name of column with the accession group/cluster names as a
character string.}

\item{alloc}{A named numeric vector specifying the number of entries to be
selected. Names should correspond to the levels of the "\code{"group"}
column, and values indicate the number of elements to be selected from each
level.}

\item{dist.mat}{A precomputed distance matrix of distance measures between the
accessions in \code{data}.}

\item{always.selected}{Names of accessions to be always included in the core
set as a character vector.}

\item{method}{The method for sampling accessions from each cluster/group.
Either \code{"mean.medoid"}, \code{"median.medoid"},
\code{"nearest.centroid"}, \code{"nearest.median"},
\code{"mean.peripheral"}, \code{"median.peripheral"},
\code{"eccentricity"}, \code{"farness.centrality"}, \code{"kennard.stone"},
\code{"duplex"}, \code{"honigs"}, \code{"farthest.point"},
\code{"nearest.neighbour"}, \code{"naes"}, \code{"optim.medoid"},
\code{"hclust.random"} or \code{"hclust.medoid"}. See \strong{Methods}.}

\item{hclust.method}{The hierarchical clustering method to be used. Either
\code{"ward.D"}, \code{"ward.D2"}, \code{"single"}, \code{"complete"},
\code{"average"} (= UPGMA), \code{"mcquitty"} (= WPGMA), \code{"median"} (=
WPGMC) or \code{"centroid"} (= UPGMC).}
}
\value{
A named list where each element contains the selected entry
  identifiers for a cluster/group.
}
\description{
Select entries from cluster/groups in the entire collection by genetic
distance based sampling according to allocation specified. \loadmathjax
}
\details{
For each cluster/group, entries are selected by several methods from
within-cluster/group genetic distances between accessions according to the
allocation provided (See \strong{Methods}).

Entries listed as \code{always.selected} are mandatorily included in the
selection. Warnings are issued if requested allocation is smaller than the
number of always-selected entries in a cluster/group and/or when the
cluster/group does not contain enough remaining entries to fulfill the
allocation.
}
\section{Methods}{


\subsection{Centrality Based Methods}{

  Selects accessions that are most representative/closest to the
  cluster/group center.

  \subsection{Medoid-like Representative Sampling by Minimal Mean
  Distance}{Selects medoid-like representatives as accessions with the
  smallest average distance to all others within the group
  \insertCite{kaufman_clustering_1987,kaufman_finding_1990}{SampleCore}.

  For each accession \mjseqn{g}, the mean distance to all other accessions
  \mjseqn{h} is computed as:

  \mjsdeqn{\bar{d}_g = \frac{1}{G} \sum_{h=1}^{G} d_{gh}}

  Accessions are ranked by \mjseqn{\bar{d}_g} in ascending order and the top
  \mjseqn{n} are selected.

  }

  \subsection{Medoid-like Representative Sampling by Minimal Median
  Distance}{

  Selects medoid-like representatives as accessions with the smallest median
  distance to all others within the group. This method is less influenced by
  outliers
  \insertCite{kaufman_clustering_1987,kaufman_finding_1990}{SampleCore}.

  For each accession \mjseqn{g}, the median distance to all other accessions
  \mjseqn{h} is computed as:

  \mjsdeqn{\tilde{d}_g = \text{median}_{h=1,\dots,G}(d_{gh})}

  Accessions are ranked by \mjseqn{\tilde{d}_g} in ascending order and the
  top \mjseqn{n} are selected.

  }

  \subsection{Representative Sampling by Proximity to Group
  Centroid}{Selects accessions closest to the group centroid in principal
  coordinate space, computed via multivariate dispersion analysis using
  \code{\link[vegan]{betadisper}}
  \insertCite{anderson_distance-based_2006,anderson_multivariate_2006}{SampleCore}.

  The distance of each accession \mjseqn{g} to the group centroid \mjseqn{C}
  in PCoA space is:

  \mjsdeqn{\delta_g = \| \mathbf{p}_g - \mathbf{c} \|}

  Where \mjseqn{\mathbf{p}_g} is the PCoA coordinate vector of accession
  \mjseqn{g} and \mjseqn{\mathbf{c}} is the group centroid. Accessions are
  ranked by \mjseqn{\delta_g} in ascending order and the top \mjseqn{n} are
  selected.

  }

  \subsection{Representative Sampling by Proximity to Group Spatial Median}{
  Selects accessions closest to the group spatial median in principal
  coordinate space, computed via multivariate dispersion analysis using
  \code{\link[vegan]{betadisper}}
  \insertCite{oneill_theory_2000}{SampleCore}.

  The distance of each accession \mjseqn{g} to the group spatial median
  \mjseqn{M} is:

  \mjsdeqn{\delta_g^* = \| \mathbf{p}_g - \mathbf{m} \|}

  where \mjseqn{\mathbf{m}} is the spatial median of the group in PCoA
  space. Accessions are ranked by \mjseqn{\delta_g^*} in ascending order and
  the top \mjseqn{n} are selected.

  }

}

\subsection{Peripheral/Extremity Based Methods}{

  Selects accessions that are most dissimilar from the rest in a
  cluster/group i.e. the accessions which are in the boundary or outliers.

  \subsection{Peripheral Sampling by Maximal Mean Distance}{Selects the most
  peripheral accessions as those with the largest average distance to all
  others within the group
  \insertCite{kaufman_clustering_1987,kaufman_finding_1990}{SampleCore}.

  \mjsdeqn{\bar{d}_g = \frac{1}{G} \sum_{h=1}^{G} d_{gh}}

  Accessions are ranked by \mjseqn{\bar{d}_g} in descending order and the
  top \mjseqn{n} are selected.

  }

  \subsection{Peripheral Sampling by Maximal Median Distance}{Selects the
  most peripheral accessions as those with the largest median distance to
  all others within the group
  \insertCite{kaufman_clustering_1987,kaufman_finding_1990}{SampleCore}.

  \mjsdeqn{\tilde{d}_g = \text{median}_{h=1,\dots,G}(d_{gh})}

  Accessions are ranked by \mjseqn{\tilde{d}_g} in descending order and the
  top \mjseqn{n} are selected.

  }

  \subsection{Peripheral Sampling by Maximal Eccentricity}{Selects
  accessions with the largest eccentricity — the maximum distance to any
  other accession in the group
  \insertCite{hage_eccentricity_1995}{SampleCore}.

  \mjsdeqn{e_g = \max_{h=1,\dots,G} d_{gh}}

  Accessions are ranked by \mjseqn{e_g} in descending order and the top
  \mjseqn{n} are selected. Eccentricity captures the worst-case
  dissimilarity of an accession rather than its average behaviour.

  }

  \subsection{Peripheral Sampling by Maximal Farness Centrality}{Selects
  accessions with the greatest total distance to all others, i.e. those most
  remote from the rest of the group
  \insertCite{sabidussi_centrality_1966}{SampleCore}.

  \mjsdeqn{f_g = \sum_{h=1}^{G} d_{gh}}

  Accessions are ranked by \mjseqn{f_g} in descending order and the top
  \mjseqn{n} are selected. Farness centrality is proportional to
  \mjseqn{\bar{d}_g} and differs from \code{mean.peripheral} only in that it
  uses the raw sum rather than the mean, producing identical rankings.

  }

}

\subsection{Space-Filling/Coverage Methods}{

  Select accessions that are spread maximally across the feature space in a
  cluster/group i.e. diversity sampling.

  \subsection{Space-Filling Sampling via the Kennard-Stone
  Algorithm}{Selects \mjseqn{n} accessions that maximally and uniformly
  cover the distance space via the Kennard-Stone algorithm
  \insertCite{kennard_computer_1969}{SampleCore} (See
  \code{\link[prospectr]{kenStone}}).

  Starting from the pair of accessions with the largest pairwise distance:

  \mjsdeqn{\lbrace g_1, g_2 \rbrace = \underset{g,h}{\arg\max}\, d_{gh}}

  each subsequent accession \mjseqn{g_k} is selected by maximising its
  minimum distance to the already-selected set \mjseqn{S}:

  \mjsdeqn{g_k = \underset{g \notin S}{\arg\max} \min_{s \in S} d_{gs}}

  This greedy procedure ensures even space coverage without relying on
  cluster structure.

  }

  \subsection{Space-Filling Sampling via the DUPLEX Algorithm}{Extends the
  Kennard-Stone algorithm to simultaneously construct a model set and a test
  set with similar distributions
  \insertCite{kennard_computer_1969,snee_validation_1977}{SampleCore}
  (\link[prospectr]{duplex}). Accessions are selected using Mahalanobis
  distance:

  \mjsdeqn{d_M(g, h) = \sqrt{(\mathbf{x}_g - \mathbf{x}_h)^\top \Sigma^{-1}
  (\mathbf{x}_g - \mathbf{x}_h)}}

  where \mjseqn{\Sigma} is the covariance matrix. At each step, the pair
  maximising \mjseqn{d_M} is split alternately between model and test sets,
  ensuring both sets span the full feature space.

  }

  \subsection{Space-Filling Sampling via the Honigs Algorithm}{Selects
  \mjseqn{n} accessions sequentially by maximising dissimilarity to the
  already-selected set \insertCite{honigs_unique-sample_1985}{SampleCore}
  (\link[prospectr]{honigs})

  At each step \mjseqn{k}, the accession \mjseqn{g_k} maximising total
  distance to all previously selected accessions \mjseqn{S} is chosen:

  \mjsdeqn{g_k = \underset{g \notin S}{\arg\max} \sum_{s \in S} d_{gs}}

  This favours accessions that are collectively most dissimilar to the
  current selection, producing broad coverage of the distance space.

  }.

  \subsection{Space-Filling Sampling via Farthest-Point (Max-Min)
  Algorithm}{Selects \mjseqn{n} accessions by iteratively maximising the
  minimum distance to the current selected set — also known as the
  max-min or farthest-point sampling algorithm
  \insertCite{gonzalez_clustering_1985,dyer_simple_1985,hochbaum_best_1985}{SampleCore}.

  \mjsdeqn{g_k = \underset{g \notin S}{\arg\max} \min_{s \in S} d_{gs}}

  This is equivalent to Kennard-Stone but without the symmetric
  initialisation step. It provides a deterministic, greedy approximation to
  the \mjseqn{k}-centre problem:

  \mjsdeqn{\min_{S \subset G,\, |S|=n} \max_{g \in G} \min_{s \in S} d_{gs}}

  }

}

\subsection{Density Based Methods}{

  Select points based on local neighbourhood density.

  \subsection{Density-Based Sampling by Minimal Nearest-Neighbour
  Distance}{Selects accessions residing in the densest regions of the
  distance space, identified as those with the smallest nearest-neighbour
  distance
  \insertCite{cover_nearest_1967,fix_discriminatory_1989}{SampleCore}.

  For each accession \mjseqn{g}, the nearest-neighbour distance is:

  \mjsdeqn{\text{nn}_g = \min_{h \neq g} d_{gh}}

  Accessions are ranked by \mjseqn{\text{nn}_g} in ascending order and the
  top \mjseqn{n} are selected. Small \mjseqn{\text{nn}_g} indicates that
  \mjseqn{g} resides in a dense cluster; this method preferentially samples
  from high-density regions.

  }

}

\subsection{Cluster Based Methods}{

  These methods partition the cluster/group space into sub-clusters/groups,
  then samples from each one.

  \subsection{Globally Optimal Medoid Sampling via Partitioning Around
  Medoids (PAM)}{

  Selects a set of \mjseqn{n} medoids that jointly minimise the total
  distance of every accession to its nearest medoid, via
  \code{\link[cluster]{pam}}.

  The objective function minimised is:

  \mjsdeqn{\min_{S \subset G,\, |S|=n} \sum_{g=1}^{G} \min_{s \in S} d_{gs}}

  Unlike \code{"mean.medoid"}, medoids are co-optimised as a set, ensuring
  they collectively represent the full distribution of the group rather than
  independently scoring each accession.

  }

  \subsection{Cluster-Based Sampling via K-means (Naes Method)}{Partitions
  accessions into \mjseqn{n} clusters via k-means applied to the distance
  matrix (See \code{\link[prospectr]{naes}}), then selects the accession
  closest to each cluster centre as the representative
  \insertCite{naes_design_1987,naes_user-friendly_2017}{SampleCore}.

  The k-means objective minimised is:

  \mjsdeqn{\min \sum_{k=1}^{n} \sum_{g \in C_k} d_{g, \mu_k}^2}

  where \mjseqn{C_k} is the \mjseqn{k}-th cluster and \mjseqn{\mu_k} is its
  centre. One representative per cluster is returned, ensuring broad,
  partition-aware coverage.

  }

  \subsection{Cluster-Based Sampling via Hierarchical Clustering with
  Random Selection}{Partitions accessions into \mjseqn{n} clusters by
  cutting a hierarchical clustering dendrogram at height \mjseqn{k = n},
  then randomly samples one accession from each cluster
  \insertCite{ward_Hierarchical_1963,li_studies_2002}{SampleCore}.

  The dendrogram is built by agglomerative hierarchical clustering using the
  linkage criterion specified by \code{\link[stats]{hclust}}. For
  clusters \mjseqn{C_1, \dots, C_n}, one accession is drawn uniformly at
  random from each:

  \mjsdeqn{g_k \sim \text{Uniform}(C_k), \quad k = 1, \dots, n}

  This introduces stochasticity within a structured partition, balancing
  coverage with randomness.

  }

  \subsection{Cluster-Based Sampling via Hierarchical Clustering with Medoid
  Selection}{Partitions accessions into \mjseqn{n} clusters by cutting a
  hierarchical clustering dendrogram at height \mjseqn{k = n}, then selects
  the within-cluster medoid as the representative of each cluster
  \insertCite{kaufman_clustering_1987,ward_Hierarchical_1963}{SampleCore}.

  For each cluster \mjseqn{C_k}, the medoid is the accession minimising
  total within-cluster distance:

  \mjsdeqn{g_k^* = \underset{g \in C_k}{\arg\min} \sum_{h \in C_k} d_{gh}}

  This combines the structured partitioning of hierarchical clustering with
  deterministic, centrality-based representative selection.

  }

}
}

\examples{
\dontshow{if (requireNamespace("cluster", quietly = TRUE) & requireNamespace("ggplot2", quietly = TRUE)) withAutoprint(\{ # examplesIf}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Prepare example data
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

library(cluster)
library(ggplot2)

data(cassava_EC_gp)

set.seed(123)
data <- cassava_EC_gp[sample(1:nrow(cassava_EC_gp), 500), ]

quant <- c("NMSR", "TTRN", "TFWSR", "TTRW", "TFWSS", "TTSW", "TTPW", "AVPW",
           "ARSR", "SRDM")
qual <- c("CUAL", "LNGS", "PTLC", "DSTA", "LFRT", "LBTEF", "CBTR", "NMLB",
          "ANGB", "CUAL9M", "LVC9M", "TNPR9M", "PL9M", "STRP", "STRC",
          "PSTR")

data[, qual] <- lapply(data[, qual], as.factor)

# Get the Gower's distance matrix
dist_matrix <- daisy(x = data[, c(qual, quant)],
                     metric = "gower")


data <- cbind(genotypes = rownames(data), data)
row.names(data) <- NULL

# Prepare inputs
counts <- c(I = 16, II = 15, III = 9, IV = 18, V = 20, VI = 8)

mand_accns <-
  c("TMe-2018", "TMe-801", "TMe-3191", "TMe-1830", "TMe-1790")

gp_vec <- setNames(as.character(data[, "Cluster"]), data[, "genotypes"])

\donttest{
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Fetch selected accessions by centrality based methods
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  # Medoid-like Representative Sampling by Minimal Mean Distance
  sel_mean_medoid_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "mean.medoid")
  sel_mean_medoid_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_mean_medoid_out,
                                use.names = FALSE)) +
    labs(title = "mean.medoid")

  # Medoid-like Representative Sampling by Minimal Median Distance
  sel_median_medoid_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "median.medoid")
  sel_median_medoid_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_median_medoid_out,
                                use.names = FALSE)) +
    labs(title = "median.medoid")

  # Representative Sampling by Proximity to Group Centroid
  sel_group_centroid_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "nearest.centroid")
  sel_group_centroid_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_group_centroid_out,
                                use.names = FALSE)) +
    labs(title = "nearest.centroid")

  # Representative Sampling by Proximity to Group Spatial Median
  sel_group_median_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "nearest.median")
  sel_group_median_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_group_median_out,
                                use.names = FALSE)) +
    labs(title = "nearest.median")

  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Fetch selected accessions by peripheral/extremity based methods
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  # Peripheral Sampling by Maximal Mean Distance
  sel_mean_peripheral_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "mean.peripheral")
  sel_mean_peripheral_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_mean_peripheral_out,
                                use.names = FALSE)) +
    labs(title = "mean.peripheral")

  # Peripheral Sampling by Maximal Median Distance
  sel_median_peripheral_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "median.peripheral")
  sel_median_peripheral_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_median_peripheral_out,
                                use.names = FALSE)) +
    labs(title = "median.peripheral")

  # Peripheral Sampling by Maximal Eccentricity
  sel_eccentricity_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "eccentricity")
  sel_eccentricity_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_eccentricity_out,
                                use.names = FALSE)) +
    labs(title = "eccentricity")

  # Peripheral Sampling by Maximal Farness Centrality
  sel_far_cent_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "farness.centrality")
  sel_far_cent_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_far_cent_out,
                                use.names = FALSE)) +
    labs(title = "farness.centrality")

  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Fetch selected accessions by space-Filling/coverage methods
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  # Space-Filling Sampling via the Kennard-Stone Algorithm
  sel_ks_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "kennard.stone")
  sel_ks_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_ks_out,
                                use.names = FALSE)) +
    labs(title = "kennard.stone")

  # Space-Filling Sampling via the DUPLEX Algorithm
  sel_duplex_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "duplex")
  sel_duplex_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_duplex_out,
                                use.names = FALSE)) +
    labs(title = "duplex")

  # Space-Filling Sampling via the Honigs Algorithm
  sel_honigs_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "honigs")
  sel_honigs_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_honigs_out,
                                use.names = FALSE)) +
    labs(title = "honigs")

  # Space-Filling Sampling via Farthest-Point (Max-Min) Algorithm
  sel_far_pt_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "farthest.point")
  sel_far_pt_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_far_pt_out,
                                use.names = FALSE)) +
    labs(title = "farthest.point")

  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Fetch selected accessions by density based methods
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  # Density-Based Sampling by Minimal Nearest-Neighbour Distance
  sel_nn_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "nearest.neighbour")
  sel_nn_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_nn_out,
                                use.names = FALSE)) +
    labs(title = "nearest.neighbour")

  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  # Fetch selected accessions by cluster based methods
  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  # Globally Optimal Medoid Sampling via Partitioning Around Medoids (PAM)
  sel_pam_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "optim.medoid")
  sel_pam_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_pam_out,
                                use.names = FALSE)) +
    labs(title = "optim.medoid")

  # Cluster-Based Sampling via K-means (Naes Method)
  sel_naes_out <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "naes")
  sel_naes_out

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_naes_out,
                                use.names = FALSE)) +
    labs(title = "naes")

  # Cluster-Based Sampling via Hierarchical Clustering with Random Selection

  ## UPGMA
  sel_hclust_random_out1 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.random",
                    hclust.method = "average")
  sel_hclust_random_out1

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_random_out1,
                                use.names = FALSE)) +
    labs(title = "hclust.random", subtitle = "average")

  ## Single-linkage
  sel_hclust_random_out2 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.random",
                    hclust.method = "single")
  sel_hclust_random_out2

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_random_out2,
                                use.names = FALSE)) +
    labs(title = "hclust.random", subtitle = "single")

  ## Complete-linkage
  sel_hclust_random_out3 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.random",
                    hclust.method = "complete")
  sel_hclust_random_out3

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_random_out3,
                                use.names = FALSE)) +
    labs(title = "hclust.random", subtitle = "complete")

  ## Ward's D
  sel_hclust_random_out4 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.random",
                    hclust.method = "ward.D")
  sel_hclust_random_out4

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_random_out4,
                                use.names = FALSE)) +
    labs(title = "hclust.random", subtitle = "ward.D")

  ## WPGMA
  sel_hclust_random_out5 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.random",
                    hclust.method = "mcquitty")
  sel_hclust_random_out5

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_random_out5,
                                use.names = FALSE)) +
    labs(title = "hclust.random", subtitle = "mcquitty")

  ## WPGMC
  sel_hclust_random_out6 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.random",
                    hclust.method = "median")
  sel_hclust_random_out6

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_random_out6,
                                use.names = FALSE)) +
    labs(title = "hclust.random", subtitle = "median")

  ## UPGMC
  sel_hclust_random_out7 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.random",
                    hclust.method = "centroid")
  sel_hclust_random_out7

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_random_out7,
                                use.names = FALSE)) +
    labs(title = "hclust.random", subtitle = "centroid")

  ## Ward's D2
  sel_hclust_random_out8 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.random",
                    hclust.method = "ward.D2")
  sel_hclust_random_out8

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_random_out8,
                                use.names = FALSE)) +
    labs(title = "hclust.random", subtitle = "ward.D2")

  # Cluster-Based Sampling via Hierarchical Clustering with Medoid Selection

  ## UPGMA
  sel_hclust_medoid_out1 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.medoid",
                    hclust.method = "average")
  sel_hclust_medoid_out1

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_medoid_out1,
                                use.names = FALSE)) +
    labs(title = "hclust.medoid", subtitle = "average")

  ## Single-linkage
  sel_hclust_medoid_out2 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.medoid",
                    hclust.method = "single")
  sel_hclust_medoid_out2

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_medoid_out2,
                                use.names = FALSE)) +
    labs(title = "hclust.medoid", subtitle = "single")

  ## Complete-linkage
  sel_hclust_medoid_out3 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.medoid",
                    hclust.method = "complete")
  sel_hclust_medoid_out3

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_medoid_out3,
                                use.names = FALSE)) +
    labs(title = "hclust.medoid", subtitle = "complete")

  ## Ward's D
  sel_hclust_medoid_out4 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.medoid",
                    hclust.method = "ward.D")
  sel_hclust_medoid_out4

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_medoid_out4,
                                use.names = FALSE)) +
    labs(title = "hclust.medoid", subtitle = "ward.D")

  ## WPGMA
  sel_hclust_medoid_out5 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.medoid",
                    hclust.method = "mcquitty")
  sel_hclust_medoid_out5

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_medoid_out5,
                                use.names = FALSE)) +
    labs(title = "hclust.medoid", subtitle = "mcquitty")

  ## WPGMC
  sel_hclust_medoid_out6 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.medoid",
                    hclust.method = "median")
  sel_hclust_medoid_out6

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_medoid_out6,
                                use.names = FALSE)) +
    labs(title = "hclust.medoid", subtitle = "median")

  ## UPGMC
  sel_hclust_medoid_out7 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.medoid",
                    hclust.method = "centroid")
  sel_hclust_medoid_out7

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_medoid_out7,
                                use.names = FALSE)) +
    labs(title = "hclust.medoid", subtitle = "centroid")

  ## Ward's D2
  sel_hclust_medoid_out8 <-
    select.distance(data = data, names = "genotypes",
                    group = "Cluster", alloc = counts,
                    dist.mat = dist_matrix,
                    always.selected = mand_accns,
                    method = "hclust.medoid",
                    hclust.method = "ward.D2")
  sel_hclust_medoid_out8

  plot_dist(d = dist_matrix, method = "isomds",
            gp = gp_vec,
            highlight =  unlist(sel_hclust_medoid_out8,
                                use.names = FALSE)) +
    labs(title = "hclust.medoid", subtitle = "ward.D2")
}
\dontshow{\}) # examplesIf}
}
\references{
\insertAllCited
}
\seealso{
\code{\link[SampleCore]{select.random}},
  \code{\link[SampleCore]{select.diversity}}
}
